Type: Package
Package: BatchQC
Title: Batch Effects Quality Control Software
Version: 2.9.0
Date: 2026-02-10
Authors@R: c(
    person("Jessica", "Anderson", , "anderson.jessica@rutgers.edu", 
    role = c("aut"), comment = c(ORCID = "0000-0002-0542-9872")),
    person("W. Evan", "Johnson", , "wj183@njms.rutgers.edu",
    role = c("aut", "fnd"), comment = c(ORCID = "0000-0002-6247-6595")),
    person("Yaoan", "Leng", , "leng@bu.edu", role = c("ctb", "cre"),
    comment = c(ORCID = "0009-0002-3957-5250")),
    person("Solaiappan", "Manimaran", , "manimaran_1975@hotmail.com", role = "aut"),
    person("Heather", "Selby", , "selbyh@bu.edu", role = "ctb"),
    person("Claire", "Ruberman", , "claireruberman@gmail.com", role = "ctb"),
    person("Kwame", "Okrah", , "kwame.okrah@gmail.com", role = "ctb"),
    person("Hector Corrada", "Bravo", , "hcorrada@gmail.com", role = "ctb"),
    person("Michael", "Silverstein", , "msilver4@bu.edu", role = "ctb"),
    person("Regan", "Conrad", , "rdconrad@bu.edu", role = "ctb"),
    person("Zhaorong", "Li", , "zhaorong@bu.edu", role = "ctb"),
    person("Evan", "Holmes", , "evanjh19@gmail.com", role = "ctb"),
    person("Solomon", "Joseph", , "sj1136@njms.rutgers.edu", role = "ctb"),
    person("Howard", "Fan", , "hf268@njms.rutgers.edu", role = "ctb"),
    person("Sean", "Lu", , "sl1729@njms.rutgers.edu", role = "ctb")
  )
Description: Sequencing and microarray samples often are collected or
    processed in multiple batches or at different times. This often
    produces technical biases that can lead to incorrect results in the
    downstream analysis. BatchQC is a software tool that streamlines batch
    preprocessing and evaluation by providing interactive diagnostics,
    visualizations, and statistical analyses to explore the extent to
    which batch variation impacts the data. BatchQC diagnostics help
    determine whether batch adjustment needs to be done, and how
    correction should be applied before proceeding with a downstream
    analysis. Moreover, BatchQC interactively applies multiple common
    batch effect approaches to the data and the user can quickly see the
    benefits of each method. BatchQC is developed as a Shiny App. The
    output is organized into multiple tabs and each tab features an
    important part of the batch effect analysis and visualization of the
    data. The BatchQC interface has the following analysis groups:
    Summary, Differential Expression, Median Correlations, Heatmaps,
    Circular Dendrogram, PCA Analysis, Shape, ComBat and SVA.
License: MIT + file LICENSE
URL: https://github.com/wejlab/BatchQC
BugReports: https://github.com/wejlab/BatchQC/issues
Depends: R (>= 4.5.0)
Imports: data.table, DESeq2, dplyr, EBSeq, edgeR, FNN, ggdendro,
        ggnewscale, ggplot2, ggpubr, Harman, limma, matrixStats,
        methods, MASS, pheatmap, RColorBrewer, reader, reshape2, scran,
        shiny, shinyjs, shinythemes, stats, SummarizedExperiment, sva,
        S4Vectors, tibble, tidyr, tidyverse, umap, utils
Suggests: BiocManager, BiocStyle, bladderbatch, curatedTBData,
        devtools, knitr, lintr, MultiAssayExperiment, plotly,
        rmarkdown, spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
biocViews: BatchEffect, GeneExpression, GraphAndNetwork, Microarray,
        Normalization, PrincipalComponent, Sequencing, Software,
        Visualization, QualityControl, RNASeq, Preprocessing,
        DifferentialExpression, ImmunoOncology
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Collate: 'Normalization.R' 'batch_correction.R' 'color_palette.R'
        'compute_aic.R' 'covariates_not_confounded.R' 'data.R'
        'dendrogram.R' 'dendrogram_alpha_numeric_check.R'
        'dendrogram_color_palette.R' 'differential_expression.R'
        'explained_variation.R' 'global_vars.R' 'heatmap.R'
        'heatmap_num_to_char_convertor.R' 'import.R' 'kBET-utils.R'
        'kBET.R' 'lambda_statistic.R' 'negative_binomial_check.R'
        'pca.R' 'preprocess.R' 'runApp.R' 'summary_statistics.R'
        'umap.R' 'variation_ratios.R' 'volcano_plot.R'
git_url: https://git.bioconductor.org/packages/BatchQC
git_branch: devel
git_last_commit: c30deb7
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:29:50 UTC; biocbuild
Author: Jessica Anderson [aut] (ORCID: <https://orcid.org/0000-0002-0542-9872>),
  W. Evan Johnson [aut, fnd] (ORCID:
    <https://orcid.org/0000-0002-6247-6595>),
  Yaoan Leng [ctb, cre] (ORCID: <https://orcid.org/0009-0002-3957-5250>),
  Solaiappan Manimaran [aut],
  Heather Selby [ctb],
  Claire Ruberman [ctb],
  Kwame Okrah [ctb],
  Hector Corrada Bravo [ctb],
  Michael Silverstein [ctb],
  Regan Conrad [ctb],
  Zhaorong Li [ctb],
  Evan Holmes [ctb],
  Solomon Joseph [ctb],
  Howard Fan [ctb],
  Sean Lu [ctb]
Maintainer: Yaoan Leng <leng@bu.edu>

# Generated by roxygen2: do not edit by hand

export(BatchQC)
export(DE_analyze)
export(EV_plotter)
export(EV_table)
export(PCA_plotter)
export(batch_correct)
export(batch_design)
export(batchqc_explained_variation)
export(bisect)
export(bladder_data_upload)
export(color_palette)
export(compute_aic)
export(compute_lambda)
export(confound_metrics)
export(cor_props)
export(covariates_not_confounded)
export(cramers_v)
export(dendrogram_alpha_numeric_check)
export(dendrogram_color_palette)
export(dendrogram_plotter)
export(goodness_of_fit_nb)
export(heatmap_num_to_char_converter)
export(heatmap_plotter)
export(is_design_balanced)
export(kBET)
export(normalize_SE)
export(plot_kBET)
export(process_dendrogram)
export(pval_plotter)
export(pval_summary)
export(ratio_plotter)
export(run_kBET)
export(run_lambda)
export(std_pearson_corr_coef)
export(summarized_experiment)
export(summary_stats_EV_table)
export(tb_data_upload)
export(umap)
export(variation_ratios)
export(volcano_plot)
import(DESeq2)
import(EBSeq)
import(Harman)
import(RColorBrewer)
import(SummarizedExperiment)
import(dplyr)
import(edgeR)
import(ggdendro)
import(ggnewscale)
import(ggplot2)
import(ggpubr)
import(pheatmap)
import(reader)
import(reshape2)
import(scran)
import(shinyjs, except=show)
import(shinythemes)
import(sva)
import(tibble)
import(tidyr)
import(tidyverse)
import(umap)
import(utils)
importFrom(FNN,get.knn)
importFrom(MASS,glm.nb)
importFrom(RColorBrewer,brewer.pal)
importFrom(S4Vectors,DataFrame)
importFrom(data.table,":=")
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,rbindlist)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,makeContrasts)
importFrom(limma,removeBatchEffect)
importFrom(limma,topTable)
importFrom(limma,voom)
importFrom(methods,is)
importFrom(stats,AIC)
importFrom(stats,anova)
importFrom(stats,aov)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,dist)
importFrom(stats,dmultinom)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,glm.control)
importFrom(stats,kruskal.test)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pnbinom)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(tidyr,pivot_longer)
importFrom(utils,data)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# BatchQC

## Introduction

Sequencing and microarray samples are often collected or processed in
multiple batches or at different times. This often produces technical
biases that can lead to incorrect results in the downstream analysis.
BatchQC is a software tool that streamlines batch preprocessing and
evaluation by providing interactive diagnostics, visualizations, and
statistical analyses to explore the extent to which batch variation
impacts the data. BatchQC diagnostics help determine whether batch
adjustment needs to be done, and how correction should be applied before
proceeding with a downstream analysis. Moreover, BatchQC interactively
applies multiple common batch effect approaches to the data, and the
user can quickly see the benefits of each method. BatchQC is developed
as a Shiny App. The output is organized into multiple tabs, and each tab
features an important part of the batch effect analysis and
visualization of the data. The BatchQC interface has the following
analysis groups: Upload Data, Experimental Design, Variation Analysis,
Heatmaps, Dendrograms, PCA Analysis, Differential Expression Analysis,
and Data Download.

The package includes:

1.  Upload Data; apply desired Normalization and Batch Effect Correction
    (incl.  ComBat and ComBat-Seq)
2.  Experimental Design to view summary of data
3.  Variation Analysis
4.  Heatmap plot of gene expressions
5.  Median Correlation Plot
6.  Circular Dendrogram clustered and colored by batch, condition, and
    covariates
7.  Principal Component Analysis and plots
8.  Differential Expression Plots and Analysis using DESeq2
9.  Data Download to export any corrections or normalizations as an SE
    object

`BatchQC()` is the function that launches the Shiny App in interactive
mode.

## Installation

### Bioconductor Version

When pushed to Bioconductor (aka not yet possible, this will download
the old version of BatchQC): To begin, install
[Bioconductor](http://www.bioconductor.org/) and then install BatchQC:

``` r
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("BatchQC")
```

### Github Version

To install the most up-to-date version of BatchQC, please install
directly from github. You will need the devtools package. You can
install both of these with the following commands:

``` r
if (!require("devtools", quietly = TRUE)) {
    install.packages("devtools")   
}
library(devtools)
install_github("wejlab/BatchQC")
```

### Load BatchQC and Launch Shiny App

You should now be able to load BatchQC and launch the shiny app.

#' This function allows you to plot PCA
#' @param se SummarizedExperiment object
#' @param nfeature number of features
#' @param color choose a color
#' @param shape choose a shape
#' @param batch variable representing batch (for ellipses)
#' @param assays array of assay names from `se`
#' @param xaxisPC the PC to plot as the x axis
#' @param yaxisPC the PC to plot as the y axis
#' @param log_option TRUE if data should be logged before plotting (recommended
#' for sequencing counts), FALSE if data should not be logged (for instance,
#' data is already logged); FALSE by default
#' @usage PCA_plotter(se, nfeature, color, shape, batch, assays, xaxisPC,
#' yaxisPC, log_option = FALSE)
#' @return List containing PCA info, PCA variance and PCA plot
#' @examples
#' library(scran)
#' se <- mockSCE()
#' se_object_ComBat_Seq <- BatchQC::batch_correct(se, method = "ComBat-Seq",
#'                                             assay_to_normalize = "counts",
#'                                             batch = "Mutation_Status",
#'                                             covar = "Treatment",
#'                                             output_assay_name =
#'                                                 "ComBat_Seq_Corrected")
#' pca_plot <- BatchQC::PCA_plotter(se = se_object_ComBat_Seq,
#'                             nfeature = 2, color = "Mutation_Status",
#'                             shape = "Treatment", batch = "batch",
#'                             assays = c("counts", "ComBat_Seq_Corrected"),
#'                             xaxisPC = 1, yaxisPC = 2, log_option = FALSE)
#' pca_plot$plot
#' pca_plot$var_explained
#'
#' @export
PCA_plotter <- function(se, nfeature, color, shape, batch,
    assays, xaxisPC, yaxisPC, log_option = FALSE) {
    pca_plot_data <- data.frame()
    var_explained_data <- NULL
    coldata <- data.frame(colData(se))
    for (assay in assays){
        if (! assay %in%  names(se@assays)) {
            warning(sprintf('"%s" is not an available assay', assay))
            next
        }else {
            # Preprocess data
            data <- preprocess(se, assay, nfeature, log_option)
            centered <- data - rowMeans(data) / matrixStats::rowSds(data)
            coldata <- data.frame(colData(se))
            pca <- stats::prcomp(t(centered), center = FALSE)

            # Get variance explained
            var_explained <- summary(pca)$importance["Proportion of Variance", ]
            var_explained_df <- stats::setNames(as.data.frame(var_explained),
                assay)
            if (is.null(var_explained_data)) {
                var_explained_data <- var_explained_df
            }else {
                var_explained_data <- cbind(var_explained_data,
                    var_explained_df)
            }
            # Extract PC data
            pca_data <- as.data.frame(pca$x)

            # Annotate with assay name
            pca_data['assay'] <- assay

            # Merge metadata
            pca_md <- cbind(coldata, pca_data)
            pca_md$sample <- rownames(coldata)

            # Add to data
            pca_plot_data <- rbind(pca_plot_data, pca_md)
        }
    }
    # Reorder data
    pca_plot_data$assay <- factor(pca_plot_data$assay, levels = assays)
    var_explained_data <- var_explained_data[c(xaxisPC, yaxisPC), ,
                                            drop = FALSE]
    plot <- plot_data(pca_plot_data, color, shape, batch, xaxisPC, yaxisPC)

    return(list(PCA = pca_plot_data, var_explained = var_explained_data,
        plot = plot))
}

#' This function formats the PCA plot using ggplot
#' @param pca_plot_data Data for all assays to plot
#' @param color variable that will be plotted as color
#' @param shape variable that will be plotted as shape
#' @param batch variable representing batch for the ellipses
#' @param xaxisPC the PC to plot as the x axis
#' @param yaxisPC the PC to plot as the y axis
#' @import ggplot2
#' @return PCA plot

plot_data <- function(pca_plot_data, color, shape, batch, xaxisPC, yaxisPC) {
    pca_plot_data[, c(color)] <- factor(pca_plot_data[, color])
    pca_plot_data[, c(shape)] <- factor(pca_plot_data[, shape])

    xaxisPC <- paste0('PC', xaxisPC)
    yaxisPC <- paste0('PC', yaxisPC)
    PCAplot <- ggplot(pca_plot_data,
        aes_string(x = xaxisPC,
            y = yaxisPC,
            color = color,
            shape = shape,
            group = batch)) +
        geom_point(size = 3) +
        stat_ellipse(aes_string(color = batch, group = batch)) +
        facet_wrap(vars(assay), ncol = 2, scales = 'free')

    return(PCAplot)
}

#' This function calculates p-values for each gene given counts, estimated NB
#' size, and estimated NB mean
#' @param counts a vector of gene expression values (in counts)
#' @param size an estimated size parameter of the NB distributions for the gene
#' @param mu a vector of estimated mu parameter of the NB distributions for
#'   different samples of the gene
#' @importFrom stats pnbinom ks.test
#' @return a p-value based on estimated NB size and mean
#' @keywords internal

counts2pvalue <- function(counts, size, mu) {
    counts <- as.numeric(counts)
    if (max(counts) <= 3) {
        p.fit <- NA
    }else {
        p <- pnbinom(counts, size = size, mu = mu)
        p.fit <- ks.test(p, 'punif')$p.value
    }
    return(p.fit)
}

#' This function calculates goodness-of-fit p-values for each condition level
#' for each gene size, and estimated NB mean
#' @param condition string; name of the se colData with the condition status
#' @param size numeric; an estimated size parameter of the NB distributions for
#'   the gene
#' @param mu_matrix matrix;estimated mu parameter of the NB distributions for
#'   different samples of each gene
#' @param count_matrix string; name of the assay with gene expression matrix
#'   (in counts)
#' @return a data frame of p-values for each gene under each biological
#'   condition
#' @keywords internal

pvalues_all_genes <- function(condition, size, mu_matrix, count_matrix) {
    # Count the number of levels in condition
    unique_conditions <- unique(condition)

    all_pvalues <- vapply(seq_len(length(unique_conditions)), function(j) {
        index_j <- which(condition == unique_conditions[j])
        # For one condition level, calculate the goodness-of-fit p-values
        pvalues_level <- vapply(seq_len(length(size)), function(i) {
            mu_gene <- mu_matrix[i, index_j]
            count_condition <- count_matrix[i, index_j]
            pvalue <- counts2pvalue(counts = count_condition,
                size = size[i],
                mu = mu_gene)
            return(pvalue)
        }, double(1))
        return(pvalues_level)
    }, double(length(size)))

    all_pvalues <- as.data.frame(all_pvalues,
        row.names = row.names(count_matrix))

    colnames(all_pvalues) <- unique_conditions

    return(all_pvalues)
}

#' Performs down sampling for negative binomial model fit check.
#'
#' @param count_matrix matrix; contains the feature data (should be counts)
#' @param num_genes integer; number of genes to use in down sampling
#' @return list containing "sampled" a down sampled assay and "count_matrix"
#'   with the down sampled count_matrix
#' @keywords internal

nb_down_sample <- function(count_matrix, num_genes) {
    # Ensure the number of genes is greater than the desired number for sampling
    if (dim(count_matrix)[1] < num_genes) {
        num_genes <- dim(count_matrix)[1]
    }

    # Down sample
    if (dim(count_matrix)[1] > num_genes) {
        sampled <- sample(row.names(count_matrix), num_genes)
        col_names_prior <- colnames(count_matrix)
        count_matrix <- count_matrix[sampled, ]
    }else {
        sampled <- row.names(count_matrix)
    }

    return(list(sampled = sampled, count_matrix = count_matrix))
}

#' This function calculates goodness-of-fit pvalues for all genes by looking at
#' how the NB model by edgeR or DESeq2 fit the data
#' @import DESeq2
#' @import edgeR
#' @import SummarizedExperiment
#' @importFrom S4Vectors DataFrame
#' @param se the se object; se object where all the data is contained
#' @param count_matrix string; name of the assay with gene expression matrix
#'   (in counts)
#' @param condition string; name of the se colData with the condition status
#' @param other_variables string; name of the se colData containing other
#'   variables of interest that should be considered in the model
#' @param method string; method to use for the parametric or non-parametric
#'   test;either "DESeq2" or "edgeR"; default is "edgeR"
#' @param num_genes down sample value, default is 500 (or all genes if less)
#' @param small_sample_cutoff value at which non-parametric test will be used
#'   (considered "large sample size") vs parametric will be used (considered
#'   "small sample size); default is 21
#' @return a matrix of p-values where each row is a gene and each column is a
#'   level within the condition of interest
#' @export
#' @examples
#' # example code for small sample
#' library(scran)
#' se <- mockSCE(ncells = 20)
#' se$Treatment <- as.factor(se$Treatment)
#' se$Mutation_Status <- as.factor(se$Mutation_Status)
#' nb_results <- goodness_of_fit_nb(se = se, count_matrix = "counts",
#'   condition = "Treatment", other_variables = "Mutation_Status",
#'   method = "edgeR")
#' nb_results[1]
#' nb_results[2]
#' nb_results[3]
#'
#' # example code for large sample
#' library(scran)
#' se <- mockSCE(ncells = 150)
#' se$Treatment <- as.factor(se$Treatment)
#' se$Mutation_Status <- as.factor(se$Mutation_Status)
#' nb_results <- goodness_of_fit_nb(se = se, count_matrix = "counts",
#'   condition = "Treatment", other_variables = "Mutation_Status",
#'   method = "edgeR")
#' nb_results[1]
#' nb_results[2]
#' nb_results[3]

goodness_of_fit_nb <- function(se, count_matrix, condition,
    other_variables = NULL, method = "edgeR", num_genes = 500,
    small_sample_cutoff = 21) {
    # Obtain needed data from se object
    count_matrix <- SummarizedExperiment::assays(se)[[count_matrix]]
    condition <- SummarizedExperiment::colData(se)[[condition]]
    condition <- as.factor(condition)
    num_samples <- dim(count_matrix)[2]
    dw_sample <- nb_down_sample(count_matrix, num_genes)
    count_matrix <- dw_sample$count_matrix
    sampled <- dw_sample$sampled
    conditions_df <- NULL
    model_formula <- ""

    if (!is.null(other_variables)) {
        for (i in seq_len(length(other_variables))) {
            conditions_df <- DataFrame(c(conditions_df,
                SummarizedExperiment::colData(se)[[other_variables[i]]]))
            model_formula <- paste0(model_formula,
                " + ",
                other_variables[i])
        }
    }
    colnames(conditions_df) <- other_variables
    for (i in seq_len(length(colnames(conditions_df)))){
        conditions_df[, i] <- as.factor(conditions_df[, i])
    }

    if (method == "DESeq2") {
        if (num_samples < small_sample_cutoff) {
            result <- DESeq2_small_size(count_matrix, condition,
                other_variables, conditions_df, model_formula, num_samples,
                small_sample_cutoff)
        }else {
            result <- DESeq_large_analysis(count_matrix, condition,
                other_variables, conditions_df, model_formula, num_samples,
                sampled, small_sample_cutoff)
        }
    } else if (method == "edgeR") {
        if (num_samples < small_sample_cutoff) {
            result <- edgeR_small_size(count_matrix, condition, other_variables,
                conditions_df, model_formula, num_samples, small_sample_cutoff)
        }else {
            result <- edgeR_large_analysis(count_matrix, condition,
                other_variables, conditions_df, model_formula, num_samples,
                sampled, small_sample_cutoff)
        }
    }
    return(result)
}

#' This function calculated the goodness of fit of DESeq2 for small sample sizes
#' (intended for less than 20 samples).
#' @import DESeq2
#' @import SummarizedExperiment
#' @importFrom S4Vectors DataFrame
#' @param count_matrix matrix containing the data to be analyzed
#' @param condition a vector containing a factor of the condition of interest
#'   (typically batch)
#' @param other_variables a vector of strings of other variables of interest
#' @param conditions_df data frame containing information for the other
#'   variables of interest (columns in order of the other_variables vector)
#' @param model_formula the stat formula to be used in the DESeq analysis
#' @param num_samples total number of samples to analyze
#' @param small_sample_cutoff value at which non-parametric test was used
#'   (considered "large sample size") vs parametric was used (considered
#'   "small sample size")
#' @return a list containing the string recommendation, the histogram and a
#'   reference for the original source of the test
DESeq2_small_size <- function(count_matrix, condition, other_variables,
    conditions_df, model_formula, num_samples, small_sample_cutoff) {
    # Use DESeq2 to fit the NB model
    if (is.null(other_variables)) {
        dds <- DESeqDataSetFromMatrix(count_matrix,
            S4Vectors::DataFrame(condition), ~ condition)
    }else {
        dds <- DESeqDataSetFromMatrix(count_matrix,
            S4Vectors::DataFrame(condition, conditions_df),
            as.formula(paste0("~ condition", model_formula)))
    }
    dds <- DESeq(dds)
    # The size parameters estimated by DESeq2 for each gene
    size <- 1 / dispersions(dds)
    # The mu parameters estimated by DESeq2 for each count
    mu_matrix <- assays(dds)[["mu"]]
    # Count the number of levels in condition
    unique_conditions <- unique(condition)
    num_unique_conditions <- length(unique_conditions)

    # For each condition level, get goodness-of-fit p-values for each genes
    all_pvalues <- vapply(seq_len(length(unique_conditions)), function(j) {
        index_j <- which(condition == unique_conditions[j])
        # For one condition level, calculate the goodness-of-fit p-values
        pvalues_level <-  vapply(seq_len(length(size)), function(i) {
            mu_gene <- mu_matrix[i, index_j]
            count_condition <- count_matrix[i, index_j]
            pvalue <- counts2pvalue(counts = count_condition,
                size = size[i],
                mu = mu_gene)
            return(pvalue)
        }, double(1))
        return(pvalues_level)
    }, double(length(size)))

    all_pvalues <- as.data.frame(all_pvalues, row.names =
            row.names(count_matrix))
    colnames(all_pvalues) <- unique_conditions
    recommendation <- nb_proportion(all_pvalues, 0.01, 0.01, num_samples,
        small_sample_cutoff, method = "DESeq2")
    res_histogram <- nb_histogram(all_pvalues)
    reference <- paste0("Adapted for small sample sizes from: Li, Y., ",
        "Ge, X., Peng, F. et al. Exaggerated false positives by popular ",
        "differential expression methods when analyzing human population ",
        "samples. Genome Biol 23, 79 (2022). ",
        "https://doi.org/10.1186/s13059-022-02648-4")
    return(list(recommendation = recommendation, res_histogram = res_histogram,
        reference = reference))
}

#' This function calculated the goodness of fit of DESeq2 for larger sample
#' sizes (intended for more than 150 samples).
#' @import DESeq2
#' @import SummarizedExperiment
#' @importFrom S4Vectors DataFrame
#' @importFrom stats na.omit
#' @param count_matrix matrix containing the data to be analyzed
#' @param condition a vector containing a factor of the condition of interest
#'   (typically batch)
#' @param other_variables a vector of strings of other variables of interest
#' @param conditions_df data frame containing information for the other
#'   variables of interest (columns in order of the other_variables vector)
#' @param model_formula the stat formula to be used in the DESeq analysis
#' @param num_samples total number of samples to analyze
#' @param sampled the down sampled matrix
#' @param small_sample_cutoff value at which non-parametric test was used
#'   (considered "large sample size") vs parametric was used (considered
#'   "small sample size")
#' @return a list containing the string recommendation

DESeq_large_analysis <- function(count_matrix, condition, other_variables,
    conditions_df, model_formula, num_samples, sampled,
    small_sample_cutoff) {
    considered_significant <- 0.05
    dds <- permuted_DESeq(count_matrix, condition, other_variables,
        conditions_df, model_formula)
    res <- results(dds)
    # count the number of DEGs
    num_DEGs <- sum(res$padj <= considered_significant)
    all_padj_values <- NULL
    all_pvalues <- NULL
    for (i in 2:length(resultsNames(dds))){
        padj_values <- as.data.frame(results(dds,
            name = resultsNames(dds)[i])$padj, row.names = sampled)
        all_padj_values <- as.data.frame(c(all_padj_values, padj_values))
        p_values <- as.data.frame(results(dds,
            name = resultsNames(dds)[i])$pvalue, row.names = sampled)
        all_pvalues <- as.data.frame(c(all_pvalues, p_values))
    }

    rownames(all_padj_values) <- sampled
    rownames(all_pvalues) <- sampled
    all_padj_values <- stats::na.omit(all_padj_values)
    all_pvalues <- stats::na.omit(all_pvalues)
    num_genes <- dim(count_matrix)[1]
    colnames(all_padj_values) <- resultsNames(dds)[2:length(resultsNames(dds))]
    colnames(all_pvalues) <- resultsNames(dds)[2:length(resultsNames(dds))]
    levels_of_condition <- length(levels(condition))
    pvals_condition <- as.data.frame(
        all_pvalues[, seq_len(levels_of_condition - 1)])
    colnames(pvals_condition) <- resultsNames(dds)[2:levels_of_condition]
    rownames(pvals_condition) <- rownames(all_pvalues)
    adj_pvals_condition <- as.data.frame(
        all_padj_values[, seq_len(levels_of_condition - 1)])
    colnames(adj_pvals_condition) <-
        resultsNames(dds)[2:levels_of_condition]
    rownames(adj_pvals_condition) <- rownames(all_padj_values)
    threshold <- 0.001 * num_genes
    recommendation <- nb_proportion(adj_pvals_condition, considered_significant,
        threshold, num_samples, small_sample_cutoff, method = "DESeq2")
    res_histogram <- nb_histogram(all_pvalues) #all_padj_values)
    reference <- paste0("Paper Reference: Li, Y., ",
        "Ge, X., Peng, F. et al. Exaggerated false positives by popular ",
        "differential expression methods when analyzing human population ",
        "samples. Genome Biol 23, 79 (2022). ",
        "https://doi.org/10.1186/s13059-022-02648-4")
    return(list(recommendation = recommendation,
        res_histogram = res_histogram, reference = reference))
}

#' This function calculated the goodness of fit of edgeR for small sample sizes
#' (intended for less than or equal to 20 samples).
#' @import edgeR
#' @import SummarizedExperiment
#' @importFrom S4Vectors DataFrame
#' @param count_matrix matrix containing the data to be analyzed
#' @param condition a vector containing a factor of the condition of interest
#'   (typically batch)
#' @param other_variables a vector of strings of other variables of interest
#' @param conditions_df data frame containing information for the other
#'   variables of interest (columns in order of the other_variables vector)
#' @param model_formula the stat formula to be used in the DESeq analysis
#' @param num_samples total number of samples to analyze
#' @param small_sample_cutoff value at which non-parametric test was used
#'   (considered "large sample size") vs parametric was used (considered
#'   "small sample size")
#' @return a list containing the string recommendation, the histogram and a
#'   reference for the original source of the test
edgeR_small_size <- function(count_matrix, condition, other_variables,
    conditions_df, model_formula, num_samples, small_sample_cutoff) {
    y <- DGEList(counts = count_matrix, group = condition)
    keep <- filterByExpr(y)
    y <- y[keep, , keep.lib.sizes = FALSE]
    y <- calcNormFactors(y)
    count_norm <- edgeR::cpm(y)
    if (is.null(other_variables)) {
        design <- model.matrix(~condition)
    }else {
        y$samples <- cbind.data.frame(y$samples, conditions_df)
        design <- model.matrix(as.formula(
            paste0("~ condition", model_formula)), data = y$sample)
    }
    y <- estimateDisp(y, design)

    # The size parameters and mu estimated by edgeR for each gene
    size <- 1 / y$tagwise.dispersion
    sizefactor <- y$samples$norm.factors
    libsize.factor <- mean(colSums(count_matrix)) / mean(colSums(count_norm))
    cpm.mean <- rowMeans(count_norm)

    mu_matrix <- cpm.mean %*% t(sizefactor) * libsize.factor

    # For each condition level, get goodness-of-fit p-values for each genes
    all_pvalues <- pvalues_all_genes(condition, size, mu_matrix, y$counts)

    recommendation <- nb_proportion(all_pvalues, 0.01, 0.01, num_samples,
        small_sample_cutoff, method = "edgeR")
    res_histogram <- nb_histogram(all_pvalues)
    reference <- paste0("Adapted for small sample sizes from: Li, Y., ",
        "Ge, X., Peng, F. et al. Exaggerated false positives by popular ",
        "differential expression methods when analyzing human population ",
        "samples. Genome Biol 23, 79 (2022). ",
        "https://doi.org/10.1186/s13059-022-02648-4")
    return(list(recommendation = recommendation, res_histogram = res_histogram,
        reference = reference))
}

#' This function calculated the goodness of fit of edgeR for larger sample
#' sizes (intended for more than 150 samples).
#' @import edgeR
#' @import SummarizedExperiment
#' @importFrom S4Vectors DataFrame
#' @importFrom stats na.omit
#' @param count_matrix matrix containing the data to be analyzed
#' @param condition a vector containing a factor of the condition of interest
#'   (typically batch)
#' @param other_variables a vector of strings of other variables of interest
#' @param conditions_df data frame containing information for the other
#'   variables of interest (columns in order of the other_variables vector)
#' @param model_formula the stat formula to be used in the DESeq analysis
#' @param num_samples total number of samples to analyze
#' @param sampled the down sampled matrix
#' @param small_sample_cutoff value at which non-parametric test was used
#'   (considered "large sample size") vs parametric was used (considered
#'   "small sample size")
#' @return a list containing the string recommendation

edgeR_large_analysis <- function(count_matrix, condition, other_variables,
    conditions_df, model_formula, num_samples,
    sampled, small_sample_cutoff) {
    considered_significant <- 0.05
    fit <- permuted_edgeR(count_matrix, condition, other_variables,
        conditions_df, model_formula)
    qlf <- glmQLFTest(fit, coef = 2)
    qlf_i <- topTags(qlf, n = nrow(count_matrix), p.value = 1)@.Data[[1]]
    # count the number of DEGs
    num_DEGs <- sum(qlf_i$FDR <= considered_significant)
    all_padj_values <- NULL
    all_pvalues <- NULL
    for (i in 2:length(colnames(fit$coefficients))){
        qlf <- glmQLFTest(fit, coef = colnames(fit$coefficients)[i])
        qlf_i <- topTags(qlf, n = nrow(count_matrix), p.value = 1)@.Data[[1]]
        padj_values <- as.data.frame(qlf_i$FDR, row.names = sampled)
        all_padj_values <- as.data.frame(c(all_padj_values, padj_values))
        p_values <- as.data.frame(qlf_i$PValue, row.names = sampled)
        all_pvalues <- as.data.frame(c(all_pvalues, p_values))
    }
    rownames(all_padj_values) <- sampled
    rownames(all_pvalues) <- sampled
    all_padj_values <- stats::na.omit(all_padj_values)
    all_pvalues <- stats::na.omit(all_pvalues)
    num_genes <- dim(count_matrix)[1]
    colnames(all_padj_values) <- colnames(fit$coefficients)[-1]
    colnames(all_pvalues) <- colnames(fit$coefficients)[-1]
    levels_of_condition <- length(levels(condition))
    pvals_condition <- as.data.frame(
        all_pvalues[, seq_len(levels_of_condition - 1)])
    colnames(pvals_condition) <-
        colnames(fit$coefficients)[2:levels_of_condition]
    rownames(pvals_condition) <- rownames(all_pvalues)
    adj_pvals_condition <- as.data.frame(
        all_padj_values[, seq_len(levels_of_condition - 1)])
    colnames(adj_pvals_condition) <-
        colnames(fit$coefficients)[2:levels_of_condition]
    rownames(adj_pvals_condition) <- rownames(all_padj_values)
    threshold <- 0.001 * num_genes
    recommendation <- nb_proportion(adj_pvals_condition, considered_significant,
        threshold, num_samples, small_sample_cutoff, method = "edgeR")
    res_histogram <- nb_histogram(all_pvalues) #all_padj_values)
    reference <- paste0("Paper Reference: Li, Y., ",
        "Ge, X., Peng, F. et al. Exaggerated false positives by popular ",
        "differential expression methods when analyzing human population ",
        "samples. Genome Biol 23, 79 (2022). ",
        "https://doi.org/10.1186/s13059-022-02648-4")
    return(list(recommendation = recommendation,
        res_histogram = res_histogram, reference = reference))
}

#' This function performs DESeq on the permuted dataset.
#' @import DESeq2
#' @import SummarizedExperiment
#' @param count_matrix matrix containing the data to be analyzed
#' @param condition a vector containing a factor of the condition of interest
#'   (typically batch)
#' @param other_variables a vector of strings of other variables of interest
#' @param conditions_df data frame containing information for the other
#'   variables of interest (columns in order of the other_variables vector)
#' @param model_formula the stat formula to be used in the DESeq analysis
#' @return a DESeq2 object

permuted_DESeq <- function(count_matrix, condition, other_variables,
    conditions_df, model_formula) {

    conditions_perm <- sample(condition)

    # Do DE analysis on permuted data
    if (is.null(other_variables)) {
        dds <- DESeqDataSetFromMatrix(count_matrix,
            DataFrame(conditions_perm), ~ conditions_perm)
    }else {
        dds <- DESeqDataSetFromMatrix(count_matrix,
            DataFrame(conditions_perm, conditions_df),
            as.formula(paste0("~ conditions_perm", model_formula)))
    }
    dds <- DESeq(dds)
    return(dds)
}

#' This function performs edgeR on the permuted dataset
#' adjusted pvalues.
#' @import edgeR
#' @import SummarizedExperiment
#' @param count_matrix matrix containing the data to be analyzed
#' @param condition a vector containing a factor of the condition of interest
#'   (typically batch)
#' @param other_variables a vector of strings of other variables of interest
#' @param conditions_df data frame containing information for the other
#'   variables of interest (columns in order of the other_variables vector)
#' @param model_formula the stat formula to be used in the DESeq analysis
#' @return edgeR fit

permuted_edgeR <- function(count_matrix, condition, other_variables,
    conditions_df, model_formula) {

    conditions_perm <- sample(condition)

    y <- DGEList(counts = count_matrix, group = conditions_perm)
    y <- calcNormFactors(y)

    if (is.null(other_variables)) {
        design <- model.matrix(~condition)
    }else {
        y$samples <- cbind.data.frame(y$samples, conditions_df)
        design <- model.matrix(as.formula(paste0("~ condition",
            model_formula)),
            data = y$sample)
    }

    y <- estimateDisp(y, design)
    fit <- glmQLFit(y, design)

    return(fit)
}

#' This function creates a histogram from the negative binomial goodness-of-fit
#' adjusted pvalues.
#' @import tibble
#' @import tidyr
#' @import ggplot2
#' @param p_val_table table of adjusted p-values from the nb test
#' @return a histogram of the number of genes within a p-value range

nb_histogram <- function(p_val_table) {
    # tidy the data so there is a gene, condition and pval column
    p_val_table <- tibble::rownames_to_column(p_val_table, "features")
    p_val_table <- tidyr::pivot_longer(p_val_table,
        cols = 2:length(colnames(p_val_table)),
        names_to = "condition",
        values_to = "p_val")

    nb_histogram <- ggplot2::ggplot(p_val_table, aes_string(x = "p_val")) +
        xlab("p-value") +
        ggplot2::geom_histogram() +
        ggplot2::facet_grid(condition ~ .)

    return(nb_histogram)
}

#' This function determines the proportion of p-values below a specific value
#' and compares to the previously determined threshold
#' @import tibble
#' @import tidyr
#' @import ggplot2
#' @param p_val_table table of p-values from the nb test
#' @param low_pval value of the p-value cut off to use in proportion
#' @param threshold the value to compare the proportion of p-values to for data
#'   sets
#' @param num_samples the number of samples in the analysis
#' @param small_sample_cutoff value at which non-parametric test will be used
#'   (considered "large sample size") vs parametric will be used (considered
#'   "small sample size); default is 20
#' @param method string; method utilized for the parametric or non-parametric
#'   test; either "DESeq2" or "edgeR"
#' @return a statement about whether DESeq2 is appropriate to use for analysis

nb_proportion <- function(p_val_table, low_pval, threshold, num_samples,
    small_sample_cutoff, method) {

    if (num_samples > 20 & num_samples < 150) {
        caution_statement <- paste0("These methods have not been tested on ",
        "data sets with between 21-149 samples. Thus, caution should be had ",
        "when interpretting the results. ")
    }else {
        caution_statement <- ""
    }

    if (method == "DESeq2") {
        caution_statement <- paste0("The DESeq2 method has not been fully ",
            "tested. Please interpret results with caution. ",
            caution_statement)
    }

    if (num_samples < small_sample_cutoff) {
        proportion_below_value <- mean(p_val_table < low_pval, na.rm = TRUE)
        nb_fit <- proportion_below_value < threshold

        if (nb_fit) {
            recommendation <- paste0("may use ", method, " for your analysis.")
        }else {
            recommendation <- paste0("should not use ", method,
                " for your analysis.")
        }

        commentary <- paste0(caution_statement, "With a p-value cut off of ",
            low_pval, ", ", (round(proportion_below_value, 2) * 100),
            "% of your features are below the cutoff. Thus based on a ",
            "threshold of ", threshold, ", you ", recommendation)
    } else {
        ngene_pval <- nrow(p_val_table)
        count_below_value_pval <- 0
        for (i in seq_len(nrow(p_val_table))){
            if (min(p_val_table[i, ]) < low_pval) {
                count_below_value_pval <- count_below_value_pval + 1
            }
        }
        proportion <- count_below_value_pval / ngene_pval
        nb_fit_pval <- proportion < threshold
        commentary <- paste0(caution_statement,
            commentary(nb_fit_pval, count_below_value_pval, proportion,
                low_pval, method))
    }

    return(commentary)
}


#' This function creates the commentary recommendation when there are more than
#' 20 samples.
#' @param nb_fit_pval Boolean representing if the p-val count is below threshold
#' @param count_below_value_pval number of features below p-val threshold
#' @param proportion numeric; proportion of genes below the p-value
#' @param low_pval pval threshold
#' @param method string; method utilized for the parametric or non-parametric
#'   test; either "DESeq2" or "edgeR"
#' @return a commentary string statement
#'
commentary <- function(nb_fit_pval, count_below_value_pval, proportion,
    low_pval, method) {
    if (nb_fit_pval) {
        if (count_below_value_pval == 0) {
            recommendation <- paste0("you may use ", method,
                " for your analysis.")
        }else {
            recommendation <- paste0("you should be cautious about using ",
                method, " for your analysis. You have significant features,",
                " and thus you are at risk of receiving false results.")
        }
    }else {
        recommendation <- paste0(
            "we do not recommend that you should use ", method,
            " for your analysis.")
    }

    commentary <- paste0("With an FDR adjusted p-value cut off of ", low_pval,
        ", ", count_below_value_pval, " of your condition variable features ",
        "are below the cutoff. If ", method, "'s assumptions are met, we would",
        " expect a uniform distribution of significant features. Since ",
        round(proportion, 2) * 100, "% of features have a significant adjusted",
        " pvalue (<", low_pval, "), ", recommendation)

    return(commentary)
}

### DENDROGRAM TAB ###

## Plot dendrogram
observeEvent(input$dend_plot, {
    req(reactivevalue$se)

    batch_to_display <- input$dend_batch_to_display
    category_to_display <- input$dend_category_to_display

    den_results <- dendrogram_plotter(reactivevalue$se,
        input$dend_assay_name,
        batch_to_display,
        category_to_display)

    output$dendrogram <- renderPlot({
        plot(den_results$dendrogram)
    }, height = function() {
        session$clientData$output_dendrogram_width
    })

    output$circular_dendrogram <- renderPlot({
        plot(den_results$circular_dendrogram)
    }, height = function() {
        session$clientData$output_circular_dendrogram_width
    })
})

tabPanel(
  "Experimental Design",

  # Application title
  titlePanel("Experimental Design"),

  sidebarLayout(sidebarPanel(
    h3("Experimental Design"),
    selectizeInput('design_batch',
        'Select Batch Variable',
        choices = "",
        options = list(placeholder = 'Please select an option below',
            onInitialize = I('function() { this.setValue(""); }')))
  ),
  mainPanel(tabsetPanel(
    tabPanel(
      "Batch Design",
      selectizeInput("design_covariate",
          "Select Covariate:",
          choices = "",
          options = list(placeholder = 'Please select an option below',
                        onInitialize = I('function() { this.setValue(""); }'))),
      tableOutput('batch_design'),
      br(),
      h4(strong("Usage")),
      h5("Tables displayed here summarize the batch status of samples in a given condition. Use these tables to preview the experimental design of batches in your sample conditions. These tables will be used to estimate batch effect for sample conditions."),
      h5("First, select the uploaded metadata variable that describes the batch status of your samples in the dropdown menu of Select Batch Variable. Next, select the metadata variable for which condition you would like to view the status of batch in.")
    ),
    tabPanel(
      "Confounding Statistics",
      tableOutput("confounding_table"),
      br(),
      h4(strong("Usage")),
      h5("The table displayed here is a summary of the batch effect estimated to be present in your sample conditions. The metric used for this estimation is Cramer's V. Batch effect values are estimated between 1 and 0. A value closer to 0 indicates lower batch effect and a value closer to 1 indicates high batch effect."),
    )
  )))
)

globalVariables(c("chosen", "P.Value", "adj.P.Val", "effects", "pval", ".",
                ".", "mean_val", "as.data.table", "rbindlist", ":=", "padj",
                "p.adjust", "pvalue", "var", "comparison", "currentlevel",
                "reflevel", "log2FoldChange", "fvalue", "median_val",
                "analysis_design"))

#' Differential Expression Analysis
#'
#' This function runs DE analysis on a count matrix (DESeq), a normalized log or
#' log-CPM matrix (limma), an edgeR TMM-normalized matrix (edgeR) or perform
#' ANOVA or Kruskal-Wallis test on the data contained in the se object.
#' @param se SummarizedExperiment object
#' @param method DE analysis method option
#' ('DESeq2', 'limma', 'edgeR', 'ANOVA', or 'Kruskal-Wallis')
#' @param batch metadata column in the se object representing batch
#' @param conditions metadata columns in the se object representing additional
#'   analysis covariates
#' @param assay_to_analyze Assay in the se object (either counts for DESeq2 or
#'   normalized data for limma or edgeR) for DE analysis
#' @param padj_method correction method for adjusted p-value from
#' p.adjust.methods
#' @return A named list containing the log2FoldChange, fvalue (ANOVA only),
#' pvalue and adjusted pvalue (padj) for each analysis returned by DESeq2,
#' limma, edgeR, ANOVA, or Kruskal-Wallis.
#' @import SummarizedExperiment
#' @import DESeq2
#' @import scran
#' @import edgeR
#' @importFrom stats model.matrix as.formula t.test aov coef p.adjust var
#' @importFrom stats kruskal.test median
#' @importFrom limma lmFit eBayes topTable makeContrasts contrasts.fit
#' @importFrom data.table as.data.table rbindlist :=
#' @examples
#' library(scran)
#' se <- mockSCE()
#' differential_expression <- BatchQC::DE_analyze(se = se,
#'                                                 method = "DESeq2",
#'                                                 batch = "Treatment",
#'                                                 conditions = c(
#'                                                 "Mutation_Status"),
#'                                                 assay_to_analyze = "counts",
#'                                                 padj_method = "BH")
#' pval_summary(differential_expression)
#' pval_plotter(differential_expression)
#'
#' @export
DE_analyze <- function(se, method, batch, conditions, assay_to_analyze,
                        padj_method) {
    data <- assays(se)[[assay_to_analyze]]
    rownames(data) <- names(se)
    analysis_design <- as.data.frame(colData(se)[c(conditions, batch)])
    res <- list()
    design <- stats::model.matrix(stats::as.formula(paste(" ~",
            paste(colnames(analysis_design), collapse = "+"))),
            data = analysis_design)
    if (method == 'DESeq2') {
        res <- DESeq_DE(data, analysis_design, padj_method)
    }else if (method == 'limma') {
        res <- limma_DE(data, design, padj_method)
    }else if (method == 'edgeR') {
        res <- edgeR_DE(data, design, padj_method)
    }else if (method == 'ANOVA') {
        feature_list <- datatable_DE(se, assay_to_analyze, batch, conditions)
        res <- anova_DE(se, feature_list, padj_method, assay_to_analyze, batch,
                        conditions)
    }else if (method == "Kruskal-Wallis") {
        feature_list <- datatable_DE(se, assay_to_analyze, batch, conditions)
        res <- kw_DE(se, feature_list, padj_method, assay_to_analyze, batch,
                    conditions)
    }else {
        stop(
        "Please select a method:
        'DESeq2', 'limma', 'edgeR', 'ANOVA', or 'Kruskal-Wallis'"
        )
    }
    return(res)
}

DESeq_DE <- function(data, analysis_design, padj_method) {
    for (item in data){
        if (round(item) != item) {
            stop("Data contains non-integers")
        }else if (item < 0) {
            stop("Data: data contains negative integers")
        }
    }
    colnames(data) <- rownames(analysis_design)
    res <- list()
    data[is.na(data)] <- 0
    dds <- DESeq2::DESeqDataSetFromMatrix(countData = data,
                                    colData = analysis_design,
                                    design = stats::as.formula(paste(" ~ ",
                                                paste(colnames(analysis_design),
                                                collapse = "+"))))
    dds <- DESeq2::DESeq(dds)
    for (covar in DESeq2::resultsNames(dds)){
        imp_data <- data.frame("log2FoldChange" =
                                DESeq2::results(dds,
                                                name = covar)$log2FoldChange,
                                "pvalue" = DESeq2::results(dds,
                                                        name = covar)$pvalue,
                                "padj" = DESeq2::results(dds, name = covar,
                                                        pAdjustMethod =
                                                        padj_method)$padj,
                                row.names = rownames(DESeq2::results(
                                                                dds,
                                                                name = covar)
                                                    ))
        res[[covar]] <- imp_data
    }
    return(res)
}

limma_DE <- function(data, design, padj_method) {
    fit <- limma::lmFit(data, design)
    eBayes_res <- limma::eBayes(fit)
    res <- list()
    for (i in seq_len(length(colnames(eBayes_res$coefficients)))){
        results <- limma::topTable(eBayes_res, coef = i, number = Inf,
                                    adjust.method = padj_method) %>%
            select(c(1, P.Value, adj.P.Val))
        colnames(results) <- c("log2FoldChange", "pvalue", "padj" )
        res[[colnames(eBayes_res$coefficients)[[i]]]] <- results
    }
    return(res)
}

edgeR_DE <- function(data, design, padj_method) {
    fit <- edgeR::glmQLFit(data, design)
    res <- list()
    for (i in seq_len(length(colnames(design)))){
        quasi_likelihood <- edgeR::glmQLFTest(fit, coef = i)
        results <- edgeR::topTags(quasi_likelihood,
            n = Inf, adjust.method = padj_method)$table |>
            select(logFC, PValue, 5)
        colnames(results) <- c("log2FoldChange", "pvalue", "padj" )
        res[[quasi_likelihood$comparison]] <- results
    }
    return(res)
}

datatable_DE <- function(se, assay_to_analyze, batch, conditions) {
    data <- assays(se)[[assay_to_analyze]]
    features <- rownames(data)
    analysis_design <- as.data.frame(colData(se)[c(conditions, batch)])

    assay_dt <- as.data.table(data, keep.rownames = "features")
    design_dt <- as.data.table(analysis_design, keep.rownames = "samples")

    assay_long <- data.table::melt(assay_dt, id.vars = "features",
                                    variable.name = "samples",
                                    value.name = assay_to_analyze)

    merged_dt <- assay_long[design_dt, on = "samples"]
    feature_list <- split(merged_dt, by = "features", keep.by = FALSE)
    return(feature_list)
}

anova_DE <- function(se, feature_list, padj_method, assay_to_analyze, batch,
                    conditions) {
    analysis_design <- as.data.frame(colData(se)[c(conditions, batch)])
    model <- stats::as.formula(paste(assay_to_analyze, "~",
                                    paste(colnames(analysis_design),
                                            collapse = "+")))
    res <- list()
    all_res <- list()
    for (feature in names(feature_list)) {
        feature_dt <- data.table::as.data.table(feature_list[[feature]])
        anov_model <- aov(model, data = feature_dt)
        model_summary <- anova(anov_model)
        result_vars <- setdiff(rownames(model_summary), "Residuals")

        for (var_name in result_vars) {
            var_levels <- as.character(unique(feature_dt[[var_name]]))
            if (length(var_levels) > 1) {
                pval <- model_summary[var_name, "Pr(>F)"]
                fval <- model_summary[var_name, "F value"]
                means_dt <- feature_dt[, .(mean_val = mean(get(assay_to_analyze)
                                                            )), by = var_name]
                for (i in seq_len(length(var_levels) - 1)) {
                    for (j in (i + 1):length(var_levels)) {
                        ref_level <- var_levels[i]
                        current_level <- var_levels[j]
                        ref_mean <- means_dt[get(var_name) == ref_level,
                                            mean_val]
                        current_mean <- means_dt[get(var_name) == current_level,
                                                mean_val]

                        comp_res <- data.table(
                            feature = feature,
                            log2FoldChange = log2(current_mean / ref_mean),
                            fvalue = fval,
                            pvalue = pval,
                            var = var_name,
                            reflevel = ref_level,
                            currentlevel = current_level
                        )
                        all_res[[length(all_res) + 1]] <- comp_res
                    }
                }
            }else {
                stop("Each factor needs to have more than two levels!")
            }
        }
    }
    res <- format_DE(all_res, padj_method, method = "ANOVA")
    return(res)
}

format_DE <- function(all_res, padj_method, method) {
    res <- list()
    if (length(all_res) > 0) {
        combined_dt <- rbindlist(all_res)

        combined_dt[, padj := p.adjust(pvalue, method = padj_method), by = var]

        combined_dt[, comparison := paste0(currentlevel, ":", reflevel)]
        combined_dt[, comparison := paste0(var, ":", comparison)]

        for (i in unique(combined_dt$comparison)) {
            var_data <- combined_dt[comparison == i]
            if (method == "ANOVA") {
                var_df <- as.data.frame(var_data[, .(log2FoldChange,
                                                    fvalue,
                                                    pvalue,
                                                    padj)])
            }else if (method == "Kruskal-Wallis") {
                var_df <- as.data.frame(var_data[, .(log2FoldChange,
                                                    pvalue,
                                                    padj)])
            }
            rownames(var_df) <- var_data$feature
            res[[i]] <- var_df
        }
    }
    return(res)
}

kw_DE <- function(se, feature_list, padj_method, assay_to_analyze, batch,
                    conditions) {
    analysis_design <- as.data.frame(colData(se)[c(conditions, batch)])
    res <- list()
    all_res <- list()
    for (feature in names(feature_list)) {
        feature_dt <- data.table::as.data.table(feature_list[[feature]])
        for (var_name in colnames(analysis_design)){
            model <- stats::as.formula(paste(assay_to_analyze, "~", var_name))
            kw_res <- kruskal.test(model, data = feature_dt)
            var_levels <- as.character(unique(feature_dt[[var_name]]))
            if (length(var_levels) > 1) {
                pval <- kw_res$p.value
                median_dt <- feature_dt[,
                                        .(median_val =
                                            median(get(assay_to_analyze))),
                                        by = var_name]
                for (i in seq_len(length(var_levels) - 1)) {
                    for (j in (i + 1):length(var_levels)) {
                        ref_level <- var_levels[i]
                        current_level <- var_levels[j]
                        ref_median <- median_dt[get(var_name) == ref_level,
                                                median_val]
                        current_median <- median_dt[get(var_name)
                                                    == current_level,
                                                    median_val]
                        comp_res <- data.table(
                            feature = feature,
                            log2FoldChange = log2(current_median / ref_median),
                            pvalue = pval,
                            var = var_name,
                            reflevel = ref_level,
                            currentlevel = current_level
                        )
                        all_res[[length(all_res) + 1]] <- comp_res
                    }
                }
            }else {
                stop("Each factor needs to have more than two levels!")
            }
        }
    }
    res <- format_DE(all_res, padj_method, method = "Kruskal-Wallis")
    return(res)
}



#' Returns summary table for p-values of explained variation
#'
#' @param res_list Differential Expression analysis result (a named list of
#'   dataframes corresponding to each analysis completed with a "pvalue" column)
#' @return summary table for p-values of explained variation for each analysis
#'
#' @examples
#' library(scran)
#' se <- mockSCE()
#' differential_expression <- BatchQC::DE_analyze(se = se,
#'                                                 method = "DESeq2",
#'                                                 batch = "Treatment",
#'                                                 conditions = c(
#'                                                 "Mutation_Status"),
#'                                                 assay_to_analyze = "counts",
#'                                                 padj_method = "BH")
#' pval_summary(differential_expression)
#'
#' @export
pval_summary <- function(res_list) {

    pval_sum_table <- vector()
    for (res_table in res_list){
        pval_sum_table <- as.data.frame(cbind(pval_sum_table, res_table$pvalue))

    }

    colnames(pval_sum_table) <- names(res_list)
    rownames(pval_sum_table) <- rownames(res_list[[1]])

    return(pval_table = pval_sum_table)
}


#' P-value Plotter
#' This function allows you to plot p-values of explained variation
#' @param DE_results Differential Expression analysis result (a named list of
#' dataframes corresponding to each analysis completed with a "pvalue" column)
#' @importFrom tidyr pivot_longer
#' @import ggplot2
#' @importFrom data.table data.table
#' @return boxplots of pvalues for each condition
#' @examples
#' library(scran)
#' se <- mockSCE()
#' differential_expression <- BatchQC::DE_analyze(se = se,
#'                                                 method = "DESeq2",
#'                                                 batch = "Treatment",
#'                                                 conditions = c(
#'                                                 "Mutation_Status"),
#'                                                 assay_to_analyze = "counts",
#'                                                 padj_method = "BH")
#' pval_summary(differential_expression)
#' pval_plotter(differential_expression)
#'
#' @export
pval_plotter <- function(DE_results) {
    pval_table <- data.frame(row.names = row.names(DE_results[[1]]))
    for (covar in DE_results){
        pval_table <- cbind(pval_table, covar$pvalue)
    }

    colnames(pval_table) <- names(DE_results)

    if ("(Intercept)" %in% colnames(pval_table)) {
        pval_table <- pval_table |>
            select(-"(Intercept)")
    }

    pval_table <- tidyr::pivot_longer(pval_table,
                                        seq_along(colnames(pval_table)),
        names_to = "effects",
        values_to = "pval")

    covar_boxplot <- ggplot(pval_table,
        aes(x = effects, y = pval, fill = effects)) +
        geom_violin(width = 1.4) +
        geom_boxplot(width = 0.1) +
        scale_x_discrete(name = "") +
        scale_y_continuous(name = "P-Values", limits = c(0, 1)) +
        coord_flip() +
        labs(title =
                "Distribution of Batch and Covariate Effects (P-Values)
                Across Genes") +
        theme(legend.position = "none", plot.title = element_text(hjust = 0.5))
    return(covar_boxplot = covar_boxplot)
}

#' This function allows you to make a batch design matrix
#' @param se summarized experiment object
#' @param covariate string, biological covariate
#' @param batch string, batch variable
#' @import tidyverse
#' @import dplyr
#' @import tidyr
#' @return design table
#' @examples
#' library(scran)
#' se <- mockSCE()
#' batch_design_tibble <- batch_design(se, batch = "Mutation_Status",
#'                                                 covariate = "Treatment")
#' batch_design_tibble
#'
#' @export

batch_design <- function(se, batch, covariate) {
    ### Create a batch design table for the provided covariate
    design <- colData(se) %>% as_tibble %>%
        group_by_at(covariate) %>%
        dplyr::count(across(batch)) %>%
        pivot_wider(names_from = batch, values_from = n)
    design <- replace(design, is.na(design), 0)
    return(design)
}


#' This function allows you to calculate correlation properties
#' @param bd batch design
#' @return correlation properties
#' @examples
#' library(scran)
#' se <- mockSCE()
#' batch_design_tibble <- batch_design(se, batch = "Mutation_Status",
#'                                                 covariate = "Treatment")
#' correlation_property <- BatchQC::cor_props(batch_design_tibble)
#' correlation_property
#'
#' @export

cor_props <- function(bd) {
    # Subset matrix to design only
    m <- bd[, -1, ]
    rowsums <- rowSums(m)
    colsums <- colSums(m)
    tablesum <- sum(rowsums)
    expected <- matrix(0, nrow(m), ncol(m))
    for (i in seq_len(nrow(m))) {
        for (j in seq_len(ncol(m))) {
            expected[i, j] <- rowsums[i] * colsums[j] / tablesum
        }
    }
    chi <- sum((m - expected)^2 / expected)
    mmin <- min(nrow(m), ncol(m))

    out <- list("chi" = chi, "mmin" = mmin, "tablesum" = tablesum)
    return(out)
}


#' Calculate a standardized Pearson correlation coefficient
#' @param bd batch design
#' @return standardized Pearson correlation coefficient
#' @examples
#' library(scran)
#' se <- mockSCE()
#' batch_design_tibble <- batch_design(se, batch = "Mutation_Status",
#'                                                 covariate = "Treatment")
#' pearson_cor_result <- BatchQC::std_pearson_corr_coef(batch_design_tibble)
#' pearson_cor_result
#'
#' @export

std_pearson_corr_coef <- function(bd) {
    c <- cor_props(bd)
    r <- sqrt(c$chi * c$mmin / ((c$chi + c$tablesum) * (c$mmin - 1)))
    return(r)
}

#' This function allows you to calculate Cramer's V
#' @param bd batch design
#' @return Cramer's V
#' @examples
#' library(scran)
#' se <- mockSCE()
#' batch_design_tibble <- batch_design(se, batch = "Mutation_Status",
#'                                                 covariate = "Treatment")
#' cramers_v_result <- BatchQC::cramers_v(batch_design_tibble)
#' cramers_v_result
#'
#' @export
#'
cramers_v <- function(bd) {
    c <- cor_props(bd)
    v <- sqrt(c$chi / (c$tablesum * (c$mmin - 1)))
    return(v)
}

#'
#' Combine std. Pearson correlation coefficient and Cramer's V
#' @param se summarized experiment
#' @param batch batch variable
#' @return metrics of confounding
#' @examples
#' library(scran)
#' se <- mockSCE()
#' confound_table <- BatchQC::confound_metrics(se, batch = "Mutation_Status")
#' confound_table
#'
#' @export

confound_metrics <- function(se, batch) {
    # Covariates are non-batch
    cols <- names(colData(se))
    covs <- cols[cols != batch]
    metrics <- list("Pearson Correlation Coefficient" = std_pearson_corr_coef,
                    "Cramer's V" = cramers_v)
    metric.mat <- matrix(nrow = length(covs), ncol = length(metrics),
                            dimnames = list(covs, names(metrics)))

    for (c in covs){
        # Get batch design
        bd <- batch_design(se, batch, c)
        for (m in names(metrics)){
            # Compute metric and place in appropriate slot
            metric.mat[c, m] <- metrics[[m]](bd)
        }
    }
    # Add metrics to se
    return(metric.mat)
}

#' Create a umap plot; wrapper function for umap package plus custom plotting
#' @param se_object se_object; containing data of interest
#' @param assay_of_interest string; the assay in the se_object to plot
#' @param batch string; representing batch
#' @param covar string; representing biological variable
#' @param neighbors integer; number of nearest neighbors, default 15 per umap;
#'   lower values prioritize local structure, higher values will represent
#'   bigger picture but lose finer details
#' @param min_distance numeric; how close points appear in final layout; higher
#'   values puts less emphasis on global structure; must be less than spread
#' @param spread numeric; dispersion of points in umap
#' @param exploratory Boolean; default is FALSE, if TRUE, a 5x5 grid with
#'   k = 15, 25, 50, 100 and min_distance = 0.1, .2, .5, .75, .99 will
#'   be plotted
#' @param log_option Boolean; default is FALSE, if TRUE,
#'   log(assay_of_interest + 1) is computed and used for plotting; useful for
#'   count data
#' @import umap
#' @return umap plot
#' @examples
#' library(scran)
#' se <- mockSCE()
#' se$Treatment <- as.factor(se$Treatment)
#' se$Mutation_Status <- as.factor(se$Mutation_Status)
#' umap_plot <- BatchQC::umap(se_object = se, assay_of_interest = "counts",
#' batch = "Treatment", covar = "Mutation_Status", log_option = TRUE)
#' umap_plot
#'
#' @export

umap <- function(se_object, assay_of_interest, batch, covar, neighbors = 15,
    min_distance = 0.1, spread = 1, exploratory = FALSE, log_option = FALSE) {
    plot_assay <- t(assays(se_object)[[assay_of_interest]])

    if (log_option) {
        plot_assay <- log(plot_assay + 1)
    }

    if (!exploratory) {
        umap_data <- umap::umap(plot_assay, n_neighbors = neighbors,
            min_dist = min_distance, spread = spread)
        df <- data.frame(x = umap_data$layout[, 1],
            y = umap_data$layout[, 2],
            batch = colData(se_object)[[batch]],
            covar = colData(se_object)[[covar]])

        plot <- ggplot(df, aes(x, y, color = batch, shape = covar)) +
            geom_point(size = 3)
    }else {
        if (dim(plot_assay)[1] < 15) {
            stop("Exploratory option is only valid on data sets with
                more than 15 samples.")
        }
        nearest_neighbors <- possible_k_neighbors(dim(plot_assay)[1])
        min_distance <- possible_distances(spread)
        all_plot_data <- data.frame()

        for (j in min_distance){
            for (i in nearest_neighbors){
                umap_data <- umap::umap(plot_assay, n_neighbors = i,
                    min_dist = j, spread = spread)
                df <- data.frame(x = umap_data$layout[, 1],
                    y = umap_data$layout[, 2],
                    batch = colData(se_object)[[batch]],
                    covar = colData(se_object)[[covar]],
                    num_neighbors = rep(i,
                        times = length(colData(se_object)[[batch]])),
                    min_distance = rep(j,
                        times = length(colData(se_object)[[batch]])))
                all_plot_data <- rbind(all_plot_data, df)
            }
        }

        plot <- ggplot(all_plot_data, aes(x, y, color = batch, shape = covar)) +
            geom_point() +
            facet_grid(min_distance ~ num_neighbors,
                scales = "free", as.table = FALSE)
    }
    return(plot)
}

#' Create a vector of possible nearest neighbor values from 5, 15, 25, 50, and
#' 100
#' @param data_size size of the data set used to create umaps
#' @return k nearest neighbor list

possible_k_neighbors <- function(data_size) {
    if (data_size > 100) {
        return(c(15, 25, 50, 100))
    }else if (data_size > 50) {
        return(c(15, 25, 50))
    }else if (data_size > 25) {
        return(c(15, 25))
    }else if (data_size > 15) {
        return(c(15))
    }
}

#' Create potential min_distance values for exploratory analysis based on the
#' value of spread
#' @param spread numeric; the value of spread used in the exploratory analysis
#' @return vector of min_distance values to use in exploratory analysis

possible_distances <- function(spread) {
    if (spread >= 4) {
        return(c(0.1, spread / 4, spread / 2, spread - 1, spread - 0.00001))
    }else if (spread >= 3) {
        return(c(0.1, 1, 2, 2.5, 2.9999))
    }else if (spread >= 2) {
        return(c(0.1, 0.5, 1, 1.5, 1.9999))
    }else if (spread >= 1) {
        return(c(0.1, 0.2, 0.5, 0.75, 0.9999999))
    }
}

tabPanel(
  "kBET",

  # Application title
  titlePanel("k-Nearest Neighbor Batch Effect Test"),

  # Sidebar with a slider input for number of bins
  sidebarLayout(
    sidebarPanel(
      # List of assays to plot from se
      selectizeInput("kbet_assay",
        "Assay",
        choices = c(),
        multiple = FALSE,
        options = list(
          placeholder = "Please select an option below",
          onInitialize = I('function() { this.setValue(""); }')
        )
      ),
      selectizeInput("kbet_batch",
        "Select batch",
        choices = c(),
        multiple = FALSE,
        selected = NULL,
        options = list(
          placeholder = "Please select an option below",
          onInitialize = I('function() { this.setValue(""); }')
        )
      ),
      checkboxInput("kbet_advanced_options", "Show Advanced Options",
        value = FALSE
      ),
      conditionalPanel(
        condition = "input.kbet_advanced_options == 1",
        numericInput("knet_k0",
          "Number of nearest neighbors to test on (for NULL, set as 0)",
          value = 0,
          min = 0
        ),
        selectizeInput("kbet_doPCA",
          "Perform a pca prior to knn search?",
          choices = c("Yes", "No"),
          multiple = FALSE,
          selected = "Yes"
        ),
        numericInput("kbet_dimPCA",
          "Number of dimensions to consider if performing a pca prior to knn search",
          value = 50,
          min = 1
        ),
        selectizeInput("kbet_heuristic",
          "Compute an optimal neighbourhood size k?",
          choices = c("Yes", "No"),
          multiple = FALSE,
          selected = "Yes"
        ),
        numericInput("kbet_alpha",
          "Significance level",
          value = 0.05,
          min = 0,
          max = 1
        )
      ),
      actionButton("run_kBET", label = "Run kBET")
    ),

    # Show a plot of the generated distribution
    mainPanel(
      h4(strong("Usage")),
      h5("The k-Nearest Neighbor Batch Effect Test (kBET) is a statistical method used to quantify and test for batch effects in high-dimensional data. evaluate whether samples from different batches are well-mixed in local neighborhoods of the data. A higher rejection rate indicates a more prevalent batch effect in the data."),
      plotOutput("kBET_plot")
    )
  )
)

#' kBET rejection rate
#' @description This function runs the k-nearest neighbor batch effect test
#' (kBET) to evaluate whether the data has detectable batch effect.
#'
#' @param se SummarizedExperiment object
#' @param assay_to_normalize string; assay from se object to do normalization
#' @param batch character string of column name that represents batch
#' @param k0 integer representing number of nearest neighbors to test on
#' (neighborhood size)
#' @param knn n x k matrix of nearest neighbors for each cell (optional)
#' @param testSize integer representing number of data points to test
#' @param do.pca Boolean, if TRUE, perform a pca prior to knn search
#' (defaults to TRUE)
#' @param dim.pca Boolean, if do.pca=TRUE, choose the number of dimensions to
#' consider (defaults to 50)
#' @param heuristic Boolean, if true, compute an optimal neighborhood size k
#' (defaults to TRUE)
#' @param n_repeat numeric representing 'n_repeat' subsets to evaluate in order
#' to create a statistics on batch estimates
#' @param alpha numeric for significance level
#' @param addTest Boolean, if TRUE, perform an LRT-approximation to the
#' multinomial test AND a multinomial exact test (if appropriate)
#' @param verbose Boolean, if TRUE, display stages of current computation
#' (defaults to FALSE)
#' @param adapt Boolean, if TRUE, frequencies will be adapted (defaults to TRUE)
#'
#' @returns list object from kBET() function
#'    \enumerate{
#'    \item \code{summary} - a rejection rate for the data,
#'         an expected rejection rate for random
#'         labeling and the significance for the observed result
#'    \item \code{results} - detailed list for each tested cells;
#'         p-values for expected and observed label distribution
#'    \item \code{average.pval} - significance level over the averaged
#'         batch label distribution in all neighbourhoods
#'    \item \code{stats} - extended test summary for every sample
#'    \item \code{params} - list of input parameters and adapted parameters,
#'    respectively
#'    \item \code{outsider} - only shown if \code{adapt=TRUE}. List of samples
#'         without mutual nearest neighbour: \itemize{
#'     \item \code{index} - index of each outsider sample)
#'     \item \code{categories} - tabularised labels of outsiders
#'     \item \code{p.val} - Significance level of outsider batch label
#'         distribution vs expected frequencies.
#'     If the significance level is lower than \code{alpha},
#'     expected frequencies will be adapted}
#'    }
#' @import SummarizedExperiment
#' @import ggplot2
#'
#' @examples
#' library(scran)
#' se <- mockSCE()
#' kBET_result <- BatchQC::run_kBET(
#'   se=se,
#'   assay_to_normalize="counts",
#'   batch="Treatment"
#' )
#'
#' BatchQC::plot_kBET(kBET_result)
#'
#' @export
run_kBET <- function(
    se, assay_to_normalize, batch, k0 = NULL, knn = NULL,
    testSize = NULL, do.pca = TRUE, dim.pca = 50, heuristic = TRUE,
    n_repeat = 100, alpha = 0.05, addTest = FALSE, verbose = FALSE,
    adapt = TRUE) {
    # run kBET
    batch.estimate <- kBET(
        df = as.matrix(assays(se)[[assay_to_normalize]]),
        batch = data.frame(colData(se))[, batch],
        k0 = k0, knn = knn,
        testSize = testSize, do.pca = do.pca,
        dim.pca = dim.pca, heuristic = heuristic,
        n_repeat = n_repeat, alpha = alpha,
        addTest = addTest, verbose = verbose,
        plot = FALSE, adapt = adapt
    )

    return(batch.estimate)
}

#' kBET - k-nearest neighbour batch effect test
#'
#' @description adapted from kBET package (https://github.com/theislab/kBET).
#' \code{kBET} runs a chi square test to evaluate
#' the probability of a batch effect.
#'
#' @param df dataset (rows: cells, columns: features)
#' @param batch batch id for each cell or a data frame with
#' both condition and replicates
#' @param k0 number of nearest neighbours to test on (neighbourhood size)
#' @param knn an n x k matrix of nearest neighbours for each cell (optional)
#' @param testSize number of data points to test,
#' (10 percent sample size default, but at least 25)
#' @param do.pca perform a pca prior to knn search? (defaults to TRUE)
#' @param dim.pca if do.pca=TRUE, choose the number of dimensions
#' to consider (defaults to 50)
#' @param heuristic compute an optimal neighbourhood size k
#' (defaults to TRUE)
#' @param n_repeat to create a statistics on batch estimates,
#' evaluate 'n_repeat' subsets
#' @param alpha significance level
#' @param adapt In some cases, a number of cells do not contribute
#' to any neighbourhood
#' and this may cause an imbalance in observed and expected batch
#' label frequencies.
#' Frequencies will be adapted if adapt=TRUE (default).
#' @param addTest perform an LRT-approximation to the multinomial
#' test AND a multinomial exact test (if appropriate)
#' @param plot if stats > 10, then a boxplot of the resulting
#' rejection rates is created
#' @param verbose displays stages of current computation (defaults to FALSE)
#' @return list object
#'    \enumerate{
#'    \item \code{summary} - a rejection rate for the data,
#'         an expected rejection rate for random
#'         labeling and the significance for the observed result
#'    \item \code{results} - detailed list for each tested cells;
#'         p-values for expected and observed label distribution
#'    \item \code{average.pval} - significance level over the averaged
#'         batch label distribution in all neighbourhoods
#'    \item \code{stats} - extended test summary for every sample
#'    \item \code{params} - list of input parameters and adapted parameters,
#'    respectively
#'    \item \code{outsider} - only shown if \code{adapt=TRUE}. List of samples
#'         without mutual nearest neighbour: \itemize{
#'     \item \code{index} - index of each outsider sample)
#'     \item \code{categories} - tabularised labels of outsiders
#'     \item \code{p.val} - Significance level of outsider batch label
#'         distribution vs expected frequencies.
#'     If the significance level is lower than \code{alpha},
#'     expected frequencies will be adapted}
#'    }
#' @return If the optimal neighbourhood size (k0) is smaller than 10, NA is
#' returned.
#' @examples
#' library(scran)
#' se <- mockSCE()
#' df <- as.matrix(assays(se)[["counts"]])
#' batch <- data.frame(colData(se))[, "Treatment"]
#'
#' batch.estimate <- kBET(df, batch)
#'
#' @importFrom FNN get.knn
#' @import ggplot2
#' @import tidyverse
#' @importFrom stats quantile pchisq
#' @importFrom RColorBrewer brewer.pal
#' @importFrom utils data
#' @importFrom methods is
#' @include kBET-utils.R
#' @name kBET
#' @export
kBET <- function(
    df, batch, k0 = NULL, knn = NULL, testSize = NULL, do.pca = TRUE,
    dim.pca = 50, heuristic = TRUE, n_repeat = 100, alpha = 0.05,
    addTest = FALSE, verbose = FALSE, plot = TRUE, adapt = TRUE) {
    # preliminaries:
    initialize.kbet.res <- initialize.kbet(
        df, batch, k0, knn, testSize, do.pca, dim.pca, heuristic, n_repeat,
        alpha, addTest, verbose, adapt
    )
    if (addTest) {
        kbet.res <- run.kbet.addTest(
            initialize.kbet.res, adapt, alpha, n_repeat
        )
        if (n_repeat > 1 & plot) {
            plot_kbet_helper(
                kbet.res$kBET.observed, kbet.res$kBET.expected,
                kbet.res$lrt.observed, kbet.res$lrt.expected,
                kbet.res$exact.observed, kbet.res$exact.expected, n_repeat)
        }
    } else { # kBET only
        kbet.res <- run.kbet.only(initialize.kbet.res, adapt, alpha, n_repeat)
        if (n_repeat > 1 & plot) {
            plot_kbet_helper(
                kbet.res$kBET.observed, kbet.res$kBET.expected,
                n_repeat = n_repeat)
        }
    }
    rejection <- summarize_kbet_results(
        kbet.res$rejection, kbet.res$kBET.expected, kbet.res$kBET.observed,
        kbet.res$kBET.signif, kbet.res$lrt.expected, kbet.res$lrt.observed,
        kbet.res$lrt.signif, kbet.res$exact.observed, kbet.res$exact.expected,
        kbet.res$exact.signif, n_repeat, addTest)
    # collect parameters
    rejection$params <- list(
        k0 = k0, testSize = testSize, do.pca = do.pca, dim.pca = dim.pca,
        heuristic = heuristic, n_repeat = n_repeat, alpha = alpha,
        addTest = addTest, verbose = verbose, plot = plot
    )
    # add outsiders
    if (adapt) {
        rejection$outsider <- list(
            index = initialize.kbet.res$outsider,
            categories = table(batch[initialize.kbet.res$outsider]),
            p.val = initialize.kbet.res$p.out
        )
    }
    rejection
}

#' bisect - a generic bisection function
#' @description adapted from kBET package (https://github.com/theislab/kBET).
#' Provides recursive bisection algorithm for an arbitrary function. It
#' evaluates the function \code{foo} at the bounds and replaces one of the
#' boundaries until a maximum is found or the interval becomes too small
#'
#' @param foo a function mapping a one-dim argument to one-dim value
#'
#' @param bounds a vector of length 2 with real valued numbers
#'     (i.e. two arguments of \code{foo})
#' @param known tells for which of the arguments a value is known
#'     (defaults to NULL)
#' @param ... additional parameters for \code{foo}
#' @param tolx break condition for argument (defaults to 10)
#' @param toly break condition for value (defaults to 0.01)
#' @return A range of bounds where \code{foo} is maximal.
#' @importFrom stats dist
#'
#' @examples
#' get_maximum <- bisect(function(x) {
#'   -(x - 2)^2
#' }, c(-5, 50))
#'
#' @export
bisect <- function(foo, bounds, known = NULL, ..., tolx = 5, toly = 0.01) {
    if (is.null(known)) {
        evalFoo <- vapply(bounds, foo, ..., FUN.VALUE = numeric(1))
        if (diff(unlist(evalFoo)) < -toly && diff(bounds) > tolx) {
            bounds[2] <- round(sum(bounds) / 2, 0)
            known <- c(unlist(evalFoo)[1], 0)
            bisect(foo, bounds, known, ..., tolx = tolx, toly = toly)
        } else if (diff(unlist(evalFoo)) > toly && diff(bounds) > tolx) {
            bounds[1] <- round(sum(bounds) / 2, 0)
            known <- c(0, unlist(evalFoo)[2])
            bisect(foo, bounds, known, ..., tolx = tolx, toly = toly)
        } else if (dist(unlist(evalFoo)) < toly) {
            center <- vapply(
                round(sum(bounds) / 2, 0), foo, ..., FUN.VALUE = numeric(1)
            )
            dist_center <- vapply(
                unlist(evalFoo), function(x, y) {
                    dist(c(x, y))}, center, FUN.VALUE = numeric(1))
            if (max(abs(dist_center)) < toly || dist(bounds) < tolx) {
                bounds  # return interval and stop recursion
            } else if (dist_center[1] > toly) {
                bounds[2] <- round(sum(bounds) / 2, 0)
                known <- c(unlist(evalFoo)[1], 0)
                bisect(foo, bounds, known, ..., tolx = tolx, toly = toly)
            } else if (dist_center[2] > toly) {
                bounds[1] <- round(sum(bounds) / 2, 0)
                known <- c(0, unlist(evalFoo)[2])
                bisect(foo, bounds, known, ..., tolx = tolx, toly = toly)
            }
        }
    } else {
        new.eval <- which(known == 0)
        old.eval <- which(known != 0)
        evalFoo <- vapply(bounds[new.eval], foo, ..., FUN.VALUE = numeric(1))
        result <- numeric(length(known))
        result[new.eval] <- unlist(evalFoo)
        result[old.eval] <- known[old.eval]
        if (diff(result) < -toly && diff(bounds) > tolx) {
            bounds[2] <- round(sum(bounds) / 2, 0)
            known <- c(unlist(evalFoo)[1], 0)
            bisect(foo, bounds, known, ..., tolx = tolx, toly = toly)
        } else if (diff(result) > toly && diff(bounds) > tolx) {
            bounds[1] <- round(sum(bounds) / 2, 0)
            known <- c(0, unlist(evalFoo)[1])
            bisect(foo, bounds, known, ..., tolx = tolx, toly = toly)
        } else if (dist(result) < toly || dist(bounds) < tolx) {
            bounds  # return interval and stop recursion
        }
    }
}

#' kBET Rejection Plotter
#' @description This function generates a boxplot of observed and expected
#' rejection rates for the provided kBET output list object
#'
#' @param kBET_res list object output from kBET function
#'
#' @returns ggplot object containing kBET rejection boxplot
#'
#' @examples
#' library(scran)
#' se <- mockSCE()
#' df <- as.matrix(assays(se)[["counts"]])
#' batch <- data.frame(colData(se))[, "Treatment"]
#'
#' batch.estimate <- kBET(df, batch)
#' plot_kBET(batch.estimate)
#'
#' @export
plot_kBET <- function(kBET_res) {
    # create ggplot object for plotting kBET's rejection rate
    plot.data <- data.frame(
        class = rep(c("observed", "expected"),
            each = length(kBET_res$stats$kBET.observed)
        ),
        data = c(
            kBET_res$stats$kBET.observed,
            kBET_res$stats$kBET.expected
        )
    )
    g <- ggplot(plot.data, aes(class, data)) +
        geom_boxplot(aes(fill = class)) +
        labs(x = "Test", y = "Rejection rate", title = "kBET test results") +
        theme_bw() +
        theme(legend.position = "none") +
        scale_y_continuous(limits = c(0, 1))

    return(g)
}

#' Color palette
#'
#' This function creates the base color palette used in BatchQC
#' @param n numeric object representing number of colors to be created
#' @param first_hue numeric object to set the first hue value
#' @param last_hue numeric object to set the final hue value
#' @import tibble
#' @import dplyr
#' @return color_list list of colors generated
#' @examples
#' library(scran)
#' n <- 100
#' color_list <- color_palette(n)
#' color_list
#'
#' @export
color_palette <- function(n, first_hue = 25, last_hue = 360) {
    hues <- seq(first_hue, last_hue, length = n + 1)
    color_list <- grDevices::hcl(h = hues, l = c(25, 75),
                                c = 100)[seq_len(n)]
    return(color_list)
}

## Update batch effect corrected assay name
observe( {
    req(input$correction_assay, input$correction_batch, input$correction_method)
    if (!is.null(input$correction_covariates)) {
        if (!is.null(input$group_for_batch)) {
            updateTextInput(session = session,
                inputId = 'corrected_assay_name',
                'Name for the corrected assay',
                value = paste(input$correction_assay,
                    input$correction_batch,
                    input$group_for_batch,
                    input$correction_method,
                    paste(input$correction_covariates,
                        collapse = '_'),
                    sep = '_'))
        }else {
            updateTextInput(session = session, inputId = 'corrected_assay_name',
                'Name for the corrected assay',
                value = paste(input$correction_assay,
                    input$correction_batch,
                    input$correction_method,
                    paste(input$correction_covariates,
                        collapse = '_'),
                    sep = '_'))
        }
    }else {
        if (!is.null(input$group_for_batch)) {
            updateTextInput(session = session, inputId = 'corrected_assay_name',
                'Name for the corrected assay',
                value = paste(input$correction_assay,
                    input$correction_batch,
                    input$group_for_batch,
                    input$correction_method,
                    sep = '_'))
        }else if (input$correction_method == 'sva') {
            updateTextInput(session = session, inputId = 'corrected_assay_name',
                            'Name for the corrected assay',
                            value = paste(input$correction_assay,
                                          input$correction_batch,
                                          input$group_for_batch,
                                          input$correction_method,
                                          input$psva,
                                          sep = '_'))
        }else if (input$correction_method == 'svaseq') {
            updateTextInput(session = session, inputId = 'corrected_assay_name',
                            'Name for the corrected assay',
                            value = paste(input$correction_assay,
                                          input$correction_batch,
                                          input$group_for_batch,
                                          input$correction_method,
                                          input$num_sv,
                                          sep = '_'))
        }else {
            updateTextInput(session = session, inputId = 'corrected_assay_name',
                'Name for the corrected assay',
                value = paste(input$correction_assay,
                    input$correction_batch,
                    input$correction_method,
                    sep = '_'))
        }
    }
})

## Complete NB Check
observeEvent(input$nb_check, {
    req(input$correction_assay,
        input$condition_of_interest, input$nb_method)
    withBusyIndicatorServer("nb_check", {
        check_res <- goodness_of_fit_nb(reactivevalue$se,
            input$correction_assay,
            input$condition_of_interest,
            input$nb_variables,
            input$nb_method,
            input$num_genes,
            input$small_cutoff)
        output$recommendation <- renderText(check_res$recommendation)
        output$nb_histogram <- renderPlot(check_res$res_histogram)
        output$reference <- renderText(check_res$reference)
    })
})

## Run AIC

observeEvent(input$compute_aic, {
    req(reactivevalue$se, input$correction_assay,
        input$aic_batch, input$aic_covar, input$aic_nb_maxit,
        input$aic_zero_filt_percent)
    tryCatch({{
        msg <- sprintf('Start the AIC analysis')
        withProgress(message = msg, {
            setProgress(0.5, 'Analyzing...')
            aic_res <- compute_aic(reactivevalue$se,
                input$correction_assay,
                input$aic_batch,
                input$aic_covar,
                input$aic_nb_maxit,
                input$aic_zero_filt_percent)

            output$aic_table <- renderTable({
                aic_res[["AIC_table"]]
            }, rownames = TRUE)

            output$aic_boxplot <- renderPlot({
                aic_res[["AIC_boxplot"]]
            })

            setProgress(1, 'Complete!')
        })
    }},
        error = function(error) {
            showNotification('Counts must be non-negative values only.',
                type = "error")
            print(error)
        })
    })

## Run batch effect correction
observeEvent(input$correct, {
    req(input$correction_assay, input$correction_batch, input$correction_method)
    tryCatch({{
        msg <- sprintf('Start the batch correction process')
        withProgress(message = msg, {
            setProgress(0.5, 'Correcting...')
            reactivevalue$se <- batch_correct(reactivevalue$se,
                input$correction_method,
                input$correction_assay,
                input$correction_batch,
                group = NULL,
                input$correction_covariates,
                input$corrected_assay_name,
                psva = input$sva_psva,
                num_sv = input$svaseq_num_sv,
                limit = input$Harman_limit,
                numrepeats = input$Harman_numrepeats)
            setProgress(1, 'Complete!')
        })
    }},
        error = function(error) {
            showNotification('Confounding', type = "error")
            print(error)
        })
    setupSelections()
    showNotification('Batch Correction Completed', type = "message")
})

## Update normalized assay name
observe({
    req(input$normalization_method, input$correction_assay)
    updateTextInput(session = session, inputId = 'normalized_assay_name',
        'Name for the normalized Assay',
        value = paste(input$correction_assay,
            input$normalization_method, sep = '_'))
})


## Normalize a selected assay
observeEvent(input$normalize, {
    req(input$normalization_method, input$correction_assay,
        input$normalized_assay_name)
    withBusyIndicatorServer("normalize", {
        reactivevalue$se <- normalize_SE(reactivevalue$se,
            input$normalization_method,
            input$log,
            input$correction_assay,
            input$normalized_assay_name,
            input$normalized_covariate,
            input$normalized_batch)
        setupSelections()
    })
})

## -----------------------------------------------------------------------------
library(BatchQC)
data(protein_data)
data(protein_sample_info)
se_object <- BatchQC::summarized_experiment(protein_data, protein_sample_info)

## -----------------------------------------------------------------------------
data(signature_data)
data(batch_indicator)
se_object <- BatchQC::summarized_experiment(signature_data, batch_indicator)

## ----eval = FALSE-------------------------------------------------------------
# if (!requireNamespace("bladderbatch", quietly = TRUE))
#     BiocManager::install("bladderbatch")
# se_object <- BatchQC::bladder_data_upload()

## ----eval = FALSE-------------------------------------------------------------
# if (!requireNamespace("curatedTBData", quietly = TRUE))
#     BiocManager::install("curatedTBData")
# se_object <- BatchQC::tb_data_upload()

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

#' This function allows you to add normalized count matrix to the SE object
#' @param se SummarizedExperiment Object
#' @param method string; Normalization Method, either 'CPM', 'DESeq', 'edgeR',
#'   'voom', or 'none' for log(x+1) only
#' @param log_bool True or False; True to log normalize the data set after
#'   normalization method
#' @param assay_to_normalize string; SE assay to do normalization on
#' @param output_assay_name string; name for the resulting normalized assay
#' @param condition string; the biological variable of interest, required for
#'   voom, default 'NULL'
#' @param batch string; the batch variable, required for voom, default 'NULL'
#' @return the original SE object with normalized assay appended
#' @import SummarizedExperiment
#' @import EBSeq
#' @import edgeR
#' @import reader
#' @examples
#' library(scran)
#' se <- mockSCE()
#' se_CPM_normalized <- BatchQC::normalize_SE(se, method = "CPM",
#'                                 log_bool = FALSE,
#'                                 assay_to_normalize = "counts",
#'                                 output_assay_name =
#'                                     "CPM_normalized_counts")
#' se_DESeq_normalized <- BatchQC::normalize_SE(se, method = "DESeq",
#'                                 log_bool = FALSE,
#'                                 assay_to_normalize = "counts",
#'                                 output_assay_name =
#'                                     "DESeq_normalized_counts")
#' se_CPM_normalized
#' se_DESeq_normalized
#'
#' @export
normalize_SE <- function(se, method, log_bool, assay_to_normalize,
    output_assay_name, condition = NULL, batch = NULL) {
    se <- se
    if (method == 'CPM') {
        assays(se)[[output_assay_name]] <-
            (assays(se)[[assay_to_normalize]]) /
            colSums(assays(se)[[assay_to_normalize]]) * (10^6)

    }else if (method == 'DESeq') {
        assays(se)[[output_assay_name]] <- EBSeq::GetNormalizedMat(
            assays(se)[[assay_to_normalize]],
            EBSeq::MedianNorm(assays(se)[[assay_to_normalize]]))

    }else if (method == "edgeR") {
        dge <- edgeR::DGEList(counts = assays(se)[[assay_to_normalize]])
        dge <- edgeR::normLibSizes(dge)
        assays(se)[[output_assay_name]] <- dge$counts

    }else if (method == "voom") {
        if (is.null(condition) || is.null(batch)) {
            warning("You must provide the condition and batch variables.")
            return()
        }else {
            condition <- colData(se)[[condition]]
            batch <- colData(se)[[batch]]
            design <- model.matrix(~condition + batch)
            voom_result <- voom(counts = assays(se)[[assay_to_normalize]],
                design)
            assays(se)[[output_assay_name]] <- voom_result$E
        }
    }else {
        assays(se)[[output_assay_name]] <- assays(se)[[assay_to_normalize]]
    }

    if (log_bool) {
        assays(se)[[output_assay_name]] <-
            log(assays(se)[[output_assay_name]] + 1)
    }
    return(se)
}

#' Heatmap numeric to character converter
#'
#' This function converts any found numerics to characters
#' @param ann_col column data of heatmap
#' @import tibble
#' @import dplyr
#' @return ann_col modified column data of heatmap
#' @examples
#' library(scran)
#' se <- mockSCE()
#' col_info <- colData(se)
#' ann_col <- heatmap_num_to_char_converter(ann_col = col_info)
#' ann_col
#'
#' @export
#'
heatmap_num_to_char_converter <- function(ann_col) {

    # Loop through each column in ann_col
    for (col_name in names(ann_col)) {

        # Get unique values for the column
        unique_val <- unique(ann_col[[col_name]])

        # Loop through each value in unique_values
        for (i in seq_along(unique_val)) {
            val <- unique_val[i]

            # Check if the value is numeric
            if (is.numeric(as.numeric(val)) && !is.na(as.numeric(val))) {

                # Convert the numeric value to character and add an "_"
                new_val <- paste0(as.character(val))

                # Replace the original value with new_val to that column
                ann_col[ann_col[[col_name]] == val,
                            col_name] <- as.character(new_val)

            } else {
                break
            }
        }
    }
    return(ann_col)
}

#' This function creates a summarized experiment object from count
#' and metadata files uploaded by the user
#' @param counts counts matrix
#' @param columndata metadata dataframe
#' @return a summarized experiment object
#' @import SummarizedExperiment
#' @import reader
#' @import utils
#'
#' @examples
#' data(protein_data)
#' data(protein_sample_info)
#' se_object <- summarized_experiment(protein_data, protein_sample_info)
#'
#' @export

summarized_experiment <- function(counts, columndata) {

    counts <- counts[rowSums(counts) > 0, ]
    mutual_sample <- intersect(colnames(counts), rownames(columndata))
    counts <- counts[, mutual_sample]
    columndata <- columndata[mutual_sample, ]

    se <- SummarizedExperiment(assays = list(features = counts),
                                colData = columndata,
                                metadata = list(metadata = columndata))
    se <- se[which(rownames(se) != 'NA')]
    return(se)
}

#' Process Dendrogram
#'
#' This function processes count data for dendrogram plotting
#' @param se SummarizedExperiment object
#' @param assay assay to plot
#' @import tibble
#' @import ggdendro
#' @import dplyr
#' @return named list of dendrogram data
#' @return dendrogram_segments is data representing segments of the dendrogram
#' @return dendrogram_ends is data representing ends of the dendrogram
#' @examples
#' library(scran)
#' se <- mockSCE()
#' process_dendro <- BatchQC::process_dendrogram(se, "counts")
#' process_dendro
#'
#' @export
process_dendrogram <- function(se, assay) {
    data <- t(assays(se)[[assay]])
    dat <- as.data.frame(data) %>%
        mutate(sample_name = paste("sample", seq_len(nrow(data)), sep = "_"))
    rownames(dat) <- dat$sample_name
    sample_name <- dat$sample_name
    metadata <- cbind(as.data.frame(colData(se)), sample_name)
    metadata[] <- lapply(metadata, as.character)
    dat <- dat %>% select(-c(sample_name))
    dist_matrix <- stats::dist(dat, method = "euclidean")

    dendrogram <- stats::as.dendrogram(
        stats::hclust(
            dist_matrix,
            method = "complete")
    )

    dendrogram_data <- dendro_data(dendrogram)
    dendrogram_segments <- dendrogram_data$segments
    dendrogram_ends <- dendrogram_segments %>%
        filter(yend == 0) %>%
        left_join(dendrogram_data$labels, by = "x") %>%
        dplyr::rename(sample_name = label) %>%
        filter(!is.na(sample_name)) %>%
        left_join(metadata, by = "sample_name")

    return(list(dendrogram_ends = dendrogram_ends,
                dendrogram_segments = dendrogram_segments))
}

#' Dendrogram Plot
#'
#' This function creates a dendrogram plot
#' @param se SummarizedExperiment object
#' @param assay assay to plot
#' @param batch_var sample metadata column representing batch
#' @param category_var sample metadata column representing category of interest
#' @import ggdendro
#' @import ggplot2
#' @import dplyr
#' @import ggnewscale
#' @return named list of dendrogram plots
#' @return dendrogram is a dendrogram ggplot
#' @return circular_dendrogram is a circular dendrogram ggplot
#' @examples
#' library(scran)
#' se <- mockSCE()
#' dendrogram_plot <- BatchQC::dendrogram_plotter(se,
#'                                             "counts",
#'                                             "Mutation_Status",
#'                                             "Treatment")
#' dendrogram_plot$dendrogram
#' dendrogram_plot$circular_dendrogram
#'
#' @export
dendrogram_plotter <- function(se, assay, batch_var, category_var) {
    # Need to add in an if statement
    # if(batch_var == "batch_var"){
    # rename the "batch_var" column to "batch"
    #}
    dends <- process_dendrogram(se, assay)
    dendrogram_ends <- dends$dendrogram_ends
    dendrogram_segments <- dends$dendrogram_segments

    #Color palette
    batch_color <- dendrogram_color_palette(
        col = batch_var, dendrogram_info = dendrogram_ends)
    category_color <- dendrogram_color_palette(
        col = category_var, dendrogram_info = dendrogram_ends)
    geom_label_batch <- dendrogram_alpha_numeric_check(
        dendro_var = dendrogram_ends[, batch_var])
    geom_label_category <- dendrogram_alpha_numeric_check(
        dendro_var = dendrogram_ends[, category_var])

    # Create dendrogram plot
    dendrogram <- ggplot() +
        geom_segment(data = dendrogram_segments,
                    aes(x = x, y = y, xend = xend, yend = yend)) +
        geom_segment(data = dendrogram_ends,
                    aes(x = x, y = y.x, xend = xend, yend = yend,
                        color = dendrogram_ends[, batch_var])) +
        scale_color_manual(labels = geom_label_batch, values = batch_color,
                        name = batch_var,
                        guide_legend(override.aes = batch_color, order = 1)) +
        new_scale_color() + # To separate the color palette
        geom_text(data = dendrogram_ends,
                aes(x = x, y = y.y - 1.5, label = as.character(
                    as.numeric(factor(dendrogram_ends[, category_var]))),
                    color = dendrogram_ends[, category_var]),
                check_overlap = TRUE, size = 2.2) +
        guides(color = guide_legend(override.aes = list(
            label = "\u2014", alpha = 1))) +
        scale_color_manual(labels = geom_label_category,
                        values = category_color,
                        name = category_var)  +
        scale_y_reverse(expand = c(0.2, 0)) +
        coord_flip() +
        theme(axis.text.y = element_blank(), axis.ticks.y = element_blank()) +
        theme_bw() + ylab("Distance")

    circular_dendrogram <- dendrogram + coord_polar(theta = "x")

    return(list(dendrogram = dendrogram,
                circular_dendrogram = circular_dendrogram))
}

#' Dendrogram color palette
#'
#' This function creates the color palette used in the dendrogram plotter
#' @param col string object representing color of the label
#' @param dendrogram_info dendrogram_ends object
#' @import tibble
#' @import ggdendro
#' @import dplyr
#' @import RColorBrewer
#' @return annotation_color vector of colors corresponding to col variable
#' @examples
#' library(scran)
#' se <- mockSCE()
#' process_dendro <- BatchQC::process_dendrogram(se, "counts")
#' dendrogram_ends <- process_dendro$dendrogram_ends
#' col <- process_dendro$condition_var
#' dendro_colors <- dendrogram_color_palette(col = "Treatment",
#'                                     dendrogram_info = dendrogram_ends)
#' dendro_colors
#'
#' @export

dendrogram_color_palette <- function(col, dendrogram_info) {

    # Create unique_vars dataframe
    unique_vars <- unique(dendrogram_info[, col]) %>%
        as.data.frame() %>% rownames_to_column("row_id")

    # Determine color count and palette
    color_count <- length(unique_vars$.)

    if (color_count < 3) {
        color_list <- c("Red", "Blue", "Green", "Yellow")
    } else {
        color_list <- color_palette(n = color_count)
    }

    palette <- color_list %>%
        as.data.frame() %>%
        dplyr::rename("color" = ".") %>%
        rownames_to_column(var = "row_id")

    # Join the palette and unique_vars
    color_list <- left_join(unique_vars, palette, by = "row_id") %>%
        select(-row_id)

    # Create a named vector for annotation_color
    annotation_color <- stats::setNames(color_list$color, color_list$.)

    return(annotation_color)
}

### UPLOAD TAB ###

## Obtain the counts matrix and count table location
observeEvent(input$counts, {
    req(input$counts)
    reactivevalue$counts_location <- input$counts$datapath
    reactivevalue$counts <- read.table(reactivevalue$counts_location,
        header = TRUE,
        row.names = 1,
        sep = get.delim(reactivevalue$counts_location,
            n = 10,
            delims = c('\t', ',')),
        check.names = FALSE)
    output$counts_header <- renderDT((datatable(reactivevalue$counts)))
    output$counts_dimensions <- renderText(paste(dim(reactivevalue$counts),
        c('observations and', 'samples')))
})

## Obtain the metadata matrix and metadata table location
observeEvent(input$md, {
    req(input$md)
    reactivevalue$metadata_location <- input$md$datapath

    reactivevalue$metadata <- read.table(reactivevalue$metadata_location,
        header = TRUE, row.names = 1,
        sep = get.delim(reactivevalue$metadata_location,
            n = 10,
            delims = c('\t', ',')))

    output$metadata_header <- renderDT(datatable(reactivevalue$metadata))
})

## Obtain count matrix and metadata from the summarized experiment input
observeEvent(input$se, {
    req(input$se)
    reactivevalue$se_location <- input$se$datapath
    se <- readRDS(input$se$datapath)
    reactivevalue$se <- se
    output$counts_header <- renderDT(datatable(assays(reactivevalue$se)[[1]]))
    output$counts_dimensions <- renderText(paste(dim(reactivevalue$se),
        c('observations and', 'samples')))

    reactivevalue$metadata <- as.data.table(colData(reactivevalue$se))
    output$metadata_header <- renderDT(datatable(reactivevalue$metadata))
})

## Obtain count matrix and count location for example data
observeEvent(input$exampleData, {
    req(input$exampleData)
    if (input$exampleData == "proteinData") {

        data(protein_data)
        reactivevalue$counts <- protein_data
        output$counts_header <- renderDT(datatable(reactivevalue$counts))
        output$counts_dimensions <- renderText(paste(dim(reactivevalue$counts),
            c('observations and', 'samples')))

        data(protein_sample_info)
        reactivevalue$metadata <- protein_sample_info
        output$metadata_header <- renderDT(datatable(reactivevalue$metadata))
    }else if (input$exampleData == "signatureData") {
        data(signature_data)
        reactivevalue$counts <- signature_data
        output$counts_header <- renderDT(datatable(reactivevalue$counts))
        output$counts_dimensions <- renderText(paste(dim(reactivevalue$counts),
            c('observations and', 'samples')))

        data(batch_indicator)
        reactivevalue$metadata <- batch_indicator
        output$metadata_header <- renderDT(datatable(reactivevalue$metadata))
    }else if (input$exampleData == "bladderData") {
        bladder_data <- bladder_data_upload()
        reactivevalue$counts <- assays(bladder_data)$features
        output$counts_header <- renderDT(datatable(reactivevalue$counts))
        output$counts_dimensions <- renderText(paste(dim(reactivevalue$counts),
            c('observations and', 'samples')))

        reactivevalue$metadata <- as.data.frame(colData(bladder_data))
        output$metadata_header <- renderDT(datatable(reactivevalue$metadata))

    }else if (input$exampleData == "TBData") {
        TB_data <- tb_data_upload()
        reactivevalue$counts <- assays(TB_data)$features
        output$counts_header <- renderDT(datatable(reactivevalue$counts))
        output$counts_dimensions <- renderText(paste(dim(reactivevalue$counts),
            c('observations and', 'samples')))

        reactivevalue$metadata <- as.data.frame(colData(TB_data))
        output$metadata_header <- renderDT(datatable(reactivevalue$metadata))
     }
})

## Create summarized experiment object and set up plot options
observeEvent(input$submit, {
    withBusyIndicatorServer("submit", {
        if (input$uploadChoice == "countFile" &
                !is.null(reactivevalue$counts_location) &
                !is.null(reactivevalue$metadata_location)) {
            se <- summarized_experiment(reactivevalue$counts,
                reactivevalue$metadata)
            se <- se[which(rownames(se) != 'NA')]
        } else if (input$uploadChoice == "seObject" &
                !is.null(input$se$datapath)) {
            se <- readRDS(reactivevalue$se_location)
            se <- se[rowSums(se@assays@data[[1]]) > 0, ]
        } else if (input$uploadChoice == "example") {
            se <- summarized_experiment(reactivevalue$counts,
                reactivevalue$metadata)
            if (input$exampleData == "proteinData") {
                assayNames(se) <- "mass_to_charge_ratio"
            }else if (input$exampleData == "signatureData") {
                assayNames(se) <- "log_intensity"
            }else if (input$exampleData == "bladderData") {
                assayNames(se) <- "log_CPM"
            }
        }

        reactivevalue$se <- se
        reactivevalue$metadata <- data.frame(colData(reactivevalue$se))
        output$se_download <- renderPrint(reactivevalue$se)
        output$downloadData <- downloadHandler(
            filename = function() {
                paste("se", ".RDS", sep = "")
            },
            content = function(file) {
                saveRDS(reactivevalue$se, file)
            }
        )

        # Display metadata table
        output$metadata <- renderDataTable(data.table(data.frame(
            colData(reactivevalue$se)),
            keep.rownames = TRUE))

        # Add options to input selections
        setupSelections()
    })
})

tabPanel("umap",

    # Application title
    titlePanel("umap Analysis"),

    # Sidebar
    sidebarLayout(
        sidebarPanel(
            # List of assays to plot from se object
            selectizeInput('umap_assay',
                'Assay to plot',
                choices = c(),
                multiple = FALSE),

            # Options for plots
            selectizeInput('batch_variable',
                'Select batch varaiable (to display color)',
                choices = c(),
                multiple = FALSE,
                selected = NULL,
                options = list(placeholder = 'Please select an option below',
                    onInitialize = I('function() { this.setValue(""); }'))),
            selectizeInput('covar_variable',
                'Select biological varaiable (to display shape)',
                choices = c(),
                multiple = FALSE,
                selected = NULL,
                options = list(placeholder = 'Please select an option below',
                    onInitialize = I('function() { this.setValue(""); }'))),
            numericInput('num_neighbors',
                'Choose how many nearest neighbors to use',
                value = 15,
                min = 0,
                max = 500),
            numericInput('distance',
                'min distance between points',
                value = 0.1,
                min = 0,
                max = 1),
            numericInput('spread',
                'spread',
                value = 1,
                min = 0,
                max = 100),
            checkboxInput('exploratory',
                'create an exploratory matrix',
                value = FALSE),
            actionButton('umap_plot', label = 'Here we go!')
        ),

        # Show a plot of the generated distribution
        mainPanel(
            plotOutput('umap')
        )
    )
)

### UPDATING SELECTIONS ###

setupSelections <- function() {
    # Experimental design
    updateSelectizeInput(session = session, inputId = "design_batch",
        choices = names(colData(reactivevalue$se)),
        selected = NULL)
    updateSelectizeInput(session = session, inputId = "design_covariate",
        choices = names(colData(reactivevalue$se)),
        selected = NULL)

    # Negative Binomial Check
    updateSelectizeInput(session = session, inputId = "counts_matrix",
        choices = assayNames((reactivevalue$se)),
        selected = NULL,
        options = list(placeholder = "Please select an option below",
            onInitialize = I('function() { this.setValue(""); }')))

    updateSelectizeInput(session = session, inputId = "condition_of_interest",
        choices = names(colData(reactivevalue$se)),
        selected = NULL,
        options = list(placeholder = "Please select an option below",
            onInitialize = I('function() { this.setValue(""); }')))

    updateSelectizeInput(session = session, inputId = "nb_variables",
        choices = names(colData(reactivevalue$se)),
        selected = NULL,
        options = list(placeholder = "Please select an option below",
            onInitialize = I('function() { this.setValue(""); }')))

    # AIC Computation
    updateSelectizeInput(session = session, inputId = "aic_batch",
                        choices = names(colData(reactivevalue$se)),
                        selected = NULL,
                        options = list(placeholder = "Please select an option below",
                                        onInitialize = I('function() { this.setValue(""); }')))

    updateSelectizeInput(session = session, inputId = "aic_covar",
                        choices = names(colData(reactivevalue$se)),
                        selected = NULL,
                        options = list(placeholder = "Please select an option below",
                                        onInitialize = I('function() { this.setValue(""); }')))

    # Lambda Statistic
    updateSelectizeInput(session = session, inputId = "lambda_matrix",
        choices = assayNames((reactivevalue$se)),
        selected = NULL,
        options = list(placeholder = "Please select an option below",
            onInitialize = I('function() { this.setValue(""); }')))
    updateSelectizeInput(session = session, inputId = "batch_ind",
        choices = names(colData(reactivevalue$se)),
        selected = NULL)
    updateSelectizeInput(session = session, inputId = "group_ind",
        choices = names(colData(reactivevalue$se)),
        selected = NULL)

    # Batch Correction
    updateSelectizeInput(session = session,
        inputId = "correction_batch",
        choices = (names(colData(reactivevalue$se))),
        selected = NULL,
        options = list(placeholder = "Please select an option below",
            onInitialize = I('function() { this.setValue(""); }')))
    updateSelectizeInput(session = session,
        inputId = "correction_assay",
        choices = (assayNames((reactivevalue$se))),
        selected = NULL,
        options = list(placeholder =
                'Please select an option below',
            onInitialize =
                I('function() { this.setValue(""); }')))
    updateSelectizeInput(session = session,
        inputId = "correction_covariates",
        choices = (names(colData(reactivevalue$se))),
        selected = NULL,
        options = list(placeholder =
                'Please select an option below',
            onInitialize =
                I('function() { this.setValue(""); }')))

    # Normalization
    updateSelectizeInput(session = session,
        inputId = "normalized_batch",
        choices = (names(colData(reactivevalue$se))),
        selected = NULL,
        options = list(placeholder = "Please select an option below",
            onInitialize = I('function() { this.setValue(""); }')))
    updateSelectizeInput(session = session,
        inputId = "normalized_covariate",
        choices = (names(colData(reactivevalue$se))),
        selected = NULL,
        options = list(placeholder =
                'Please select an option below',
            onInitialize =
                I('function() { this.setValue(""); }')))

    # Heatmap
    updateSelectizeInput(session = session, inputId = 'heatmap_assay_name',
        choices = assayNames((reactivevalue$se)),
        selected = NULL)
    updateSelectInput(session = session, inputId = 'variates_to_display',
        choices = colnames(colData(reactivevalue$se)),
        selected = NULL)
    updateNumericInput(session = session, inputId = 'top_n_heatmap',
        value = dim(reactivevalue$se)[1], min = 2, max = dim(reactivevalue$se)[1])

    # Dendrogram
    updateSelectizeInput(session = session, inputId = 'dend_assay_name',
        choices = assayNames((reactivevalue$se)),
        selected = NULL)
    updateSelectInput(session = session, inputId = 'dend_batch_to_display',
        choices = colnames(colData(reactivevalue$se)),
        selected = NULL)
    updateSelectInput(session = session, inputId = 'dend_category_to_display',
        choices = colnames(colData(reactivevalue$se)),
        selected = NULL)

    # PCA
    updateSelectizeInput(session = session, inputId = 'pca_assays',
        choices = assayNames((reactivevalue$se)),
        selected = NULL)
    updateNumericInput(session = session, inputId = 'top_n_PCA',
        value = dim(reactivevalue$se)[1], min = 2, max = dim(reactivevalue$se)[1])
    updateSelectizeInput(session = session, inputId = 'variates_shape',
        choices = colnames(colData(reactivevalue$se)),
        selected = NULL)
    updateSelectizeInput(session = session, inputId = 'variates_color',
        choices = colnames(colData(reactivevalue$se)),
        selected = NULL)
    updateSelectizeInput(session = session, inputId = 'variates_batch',
        choices = colnames(colData(reactivevalue$se)),
        selected = NULL)
    updateSelectizeInput(session, inputId = 'batch',
        choices = colnames(colData(reactivevalue$se)),
        selected = NULL)
    updateSelectizeInput(session, inputId = 'group',
        choices = colnames(colData(reactivevalue$se)),
        selected = NULL)

    # umap Analysis
    updateSelectizeInput(session = session, inputId = 'umap_assay',
        choices = assayNames((reactivevalue$se)),
        selected = NULL)
    updateSelectizeInput(session = session, inputId = 'batch_variable',
        choices = colnames(colData(reactivevalue$se)),
        selected = NULL)
    updateSelectizeInput(session = session, inputId = 'covar_variable',
        choices = colnames(colData(reactivevalue$se)),
        selected = NULL)
    updateNumericInput(session = session, inputId = 'num_neighbors',
        value = 15, min = 2,
        max = dim(reactivevalue$se)[1] - 1)
    updateNumericInput(session = session, inputId = 'distance',
        value = 0.1, min = 0.000000000001)
    updateNumericInput(session = session, inputId = 'spread',
        value = 1, min = 0.000000000001)

    # Variation Analysis
    updateSelectizeInput(session = session, inputId = "variation_assay",
        choices = names(assays(reactivevalue$se)),
        selected = NULL)
    updateSelectizeInput(session = session, inputId = "variation_batch",
        choices = names(colData(reactivevalue$se)),
        selected = NULL)

    # Differential expression analysis
    updateSelectizeInput(session = session, inputId = "DE_assay",
        choices = names(assays(reactivevalue$se)),
        selected = NULL)
    updateSelectizeInput(session = session, inputId = "DE_batch",
        choices = names(colData(reactivevalue$se)),
        selected = NULL)

    # kBET Analysis
    updateSelectizeInput(session = session, inputId = "kbet_assay",
                         choices = names(assays(reactivevalue$se)),
                         selected = NULL)
    updateSelectizeInput(session = session, inputId = "kbet_batch",
                         choices = names(colData(reactivevalue$se)),
                         selected = NULL)
}

#' Compute the lambda index for determining a need for batch correction
#'
#' This function calculates the proportions of variation explained by batch,
#' group, and residual for each gene using two-way ANOVA and computes the lambda
#' index based on these three proportions.
#'
#' @param dat Numeric matrix of dimension (genes x samples) where each row
#'   represents one gene's expression across samples.
#' @param batchind Factor or numeric vector of length = ncol(dat); batch
#'   indicator for each sample
#' @param groupind Factor or numeric vector of length = ncol(dat); biological
#'   group label/indicator for each sample.
#' @importFrom stats lm anova
#' @return dataframe with columns:
#'   \describe{
#'     \item{BatchV}{Proportion of total variance explained by batch effects.}
#'     \item{GroupV}{Proportion of total variance explained by group effects.}
#'     \item{ResidV}{Proportion of total variance that is residual noise.}
#'     \item{lambda_raw}{Raw lambda index = total SS_batch / total SS_group.}
#'     \item{lambda_adj}{Adjusted lambda = lambda_raw * ResidV/(1-ResidV).}
#'   }
#' @examples
#'
#' library(scran)
#' se <- mockSCE()
#' res <- BatchQC::compute_lambda(assays(se)[["counts"]],
#'   colData(se)$Mutation_Status,
#'   colData(se)$Treatment)
#' print(res)
#'
#' @export

compute_lambda <- function(dat, batchind, groupind) {
    dat <- t(dat)
    n <- ncol(dat)
    expvar <- t(apply(dat, 2, function(x) {
        fit <- lm(x ~ factor(batchind) + factor(groupind))
        av  <- anova(fit)
        av$`Sum Sq`
    }))

    ss <- colSums(expvar)
    ss_list <- ss / sum(ss)
    lambda_raw <- ss[1] / ss[2]
    lambda_adj <- lambda_raw * ss_list[3] / (1 - ss_list[3])
    lambda_adj <- log(lambda_adj)

    res <- c(BatchV = ss_list[1], GroupV = ss_list[2], ResidV = ss_list[3],
        lambda_raw = lambda_raw, lambda_adj = lambda_adj)

    res <- t(data.frame(res))
    return(res)
}

#' Check if the experimental design is balanced or unbalanced
#'
#' Used in conjunction with the lambda
#'
#' @param se summarized experiment object
#' @param covariate string, biological covariate
#' @param batch string, batch variable
#' @return Boolean Value, TRUE if the experimental design is balanced, FALSE if
#'   the experimental design is not balanced
#' @examples
#'
#' library(scran)
#' se <- mockSCE()
#' balanced_design_check <- is_design_balanced(se, batch = "Mutation_Status",
#'                                                 covariate = "Treatment")
#' balanced_design_check
#'
#' @export

is_design_balanced <- function(se, batch, covariate) {
    balanced <- FALSE
    b_design <- batch_design(se, batch, covariate)
    pearson <- std_pearson_corr_coef(b_design)
    cramers <- cramers_v(b_design)

    if (pearson == 0 & cramers == 0) {
        balanced <- TRUE
    }

    return(balanced)
}

#' Provide a recommendation on batch correction based on lambda calculation
#'
#' This functions determines if an experimental design is balanced, then
#' calculates the lambda statistic for balanced designs and provides a
#' recommendation on if batch correction should be utilized. In general,
#' unbalanced designs always benefit from batch correction, while balanced
#' designs with a lambda greater than -2 benefit from batch correction.
#'
#' @param se summarized experiment object
#' @param assay string, the assay to analyze
#' @param condition string, condition variable
#' @param batch string, batch variable
#' @return a named list with:
#'   \describe{
#'     \item{lambda_stat}{provides the output of `compute_lambda` function}
#'     \item{correction_recommendation}{string, rec for batch correction}
#'     }
#' @examples
#'
#' library(scran)
#' se <- mockSCE()
#' lambda_calculation <- run_lambda(se,
#'                                  assay = "counts",
#'                                  batch = "Mutation_Status",
#'                                  condition = "Treatment")
#'  print(lambda_calculation$correction_recommendation)
#'  print(lambda_calculation$lambda_stat)
#'
#' @return a list with 2 parameters, 'lambda_stat' which contains the adj lambda
#'   value from lambda_compute (ln(lambda)) and 'correction_recommendation'
#'   which contains a string with a recommendation on if batch correction
#'   should be completed
#' @export

run_lambda <- function(se, assay, batch, condition) {
    recommendation <- NULL
    lambda_res <- NULL
    LAMBDA_THRESHOLD <- -7

    lambda_res <- data.frame(compute_lambda(assays(se)[[assay]],
        colData(se)[, batch],
        colData(se)[, condition]))

    if (is_design_balanced(se, batch, condition)) {
        if (lambda_res$lambda_adj > LAMBDA_THRESHOLD) {
            recommendation <- paste0("The experimental design is balanced and ",
                "the lambda statistic is ",
                round(lambda_res$lambda_adj, digits = 2),
                ", which is greater than -7. Therefore, you should consider ",
                "batch correction.")
        }else {
            recommendation <- paste0("The experimental design is balanced and ",
                "the lambda statistic is ",
                round(lambda_res$lambda_adj, digits = 2),
                ", which is less than or equal to -7. Therefore, you should ",
                "consider using your data WITHOUT batch correction.")
        }
    }else {
        recommendation <- paste0("The experimental design is unbalanced. ",
            "Therefore, you should consider applying a batch correction ",
            "method to your data. The calculated lambda statistic is ",
            round(lambda_res$lambda_adj, digits = 2), ".")
    }

    return(list(lambda_stat = lambda_res$lambda_adj,
        correction_recommendation = recommendation))
}

### EXPERIMENTAL DESIGN TAB ###

## Update experimental design
observeEvent(input$design_covariate, {
    req(input$design_batch, input$design_covariate, reactivevalue$se)
    # Create batch design table
    design <- batch_design(reactivevalue$se, input$design_batch,
                           input$design_covariate)
    output$batch_design <- renderTable(design)
})

## Update confounding design table
observeEvent(input$design_batch, {
    req(input$design_batch, reactivevalue$se)
    conf_stats <- confound_metrics(reactivevalue$se, input$design_batch)
    output$confounding_table <- renderTable(conf_stats, rownames = TRUE)
})

#' Creates Ratios of batch to variable variation statistic
#'
#' @param ex_variation_table table of explained variation results from
#'   batchqc_explained_variation
#' @param batch batch
#' @return dataframe with condition/batch ratios
#' @examples
#' library(scran)
#' se <- mockSCE()
#' se$Mutation_Status <- as.factor(se$Mutation_Status)
#' se$Treatment <- as.factor(se$Treatment)
#' expl_var_result <- batchqc_explained_variation(se, batch = "Mutation_Status",
#'                             condition = "Treatment", assay_name = "counts")
#' ratios_results <- variation_ratios(expl_var_result[[1]],
#'     batch = "Mutation_Status")
#' ratios_results
#' @export

variation_ratios <- function(ex_variation_table, batch) {
    ratio_variation_table <- as.data.frame(ex_variation_table[[batch]])
    colnames(ratio_variation_table) <- c(batch)
    for (column in names(ex_variation_table)) {
        if (column != batch & column != "Explained" & column != "Unexplained") {
            col_div_batch <- log(ex_variation_table[[column]] /
                ex_variation_table[[batch]])
            new_col <- paste0(column, "/", batch)
            ratio_variation_table[[new_col]] <- col_div_batch
        }
    }
    ratio_variation_table <- select(ratio_variation_table, -batch)
    row.names(ratio_variation_table) <- row.names(ex_variation_table)
    return(ratio_variation_table)
}

#' This function allows you to plot ratios of explained variation
#' @param ev_ratio table of ratios from variation_ratios()
#' @import reshape2
#' @import ggplot2
#' @return boxplot of ratios
#' @examples
#' library(scran)
#' se <- mockSCE()
#' se$Mutation_Status <- as.factor(se$Mutation_Status)
#' se$Treatment <- as.factor(se$Treatment)
#' expl_var_result <- batchqc_explained_variation(se, batch = "Mutation_Status",
#'                             condition = "Treatment", assay_name = "counts")
#' ratios_results <- variation_ratios(expl_var_result[[1]],
#'     batch = "Mutation_Status")
#' ratio_boxplot <- BatchQC::ratio_plotter(ratios_results)
#' ratio_boxplot
#'
#' @export
ratio_plotter <- function(ev_ratio) {
    ratio_boxplot <- ggplot(data =
            melt(as.data.frame(ev_ratio),
                id.vars = NULL),
        aes(x = variable, y = value, fill = variable)) +
        geom_boxplot() +
        scale_x_discrete(name = "") +
        scale_y_continuous(name = "Variation Source") +
        labs(title = "Ratio of Variation") +
        theme(legend.position = "none", plot.title = element_text(hjust = 0.5))
    return(ratio_boxplot)
}

tabPanel("PCA",

         # Application title
         titlePanel("PCA Analysis"),

         # Sidebar with a slider input for number of bins
         sidebarLayout(
           sidebarPanel(
             # List of assays to plot from se
             selectizeInput('pca_assays',
                            'Assays to plot',
                            choices = c(),
                            multiple = TRUE),

             # Options for both plots
             selectizeInput('variates_shape',
                            'Choose which variable to show as shape',
                            choices = c(),
                            multiple = FALSE,
                            selected = NULL,
                            options = list(placeholder =
                                               'Please select an option below',
                                           onInitialize = I('function() { this.setValue(""); }'))),
             selectizeInput('variates_color',
                            'Choose which variable to show as color',
                            choices = c(),
                            multiple = FALSE,
                            selected = NULL,
                            options = list(placeholder = 'Please select an option below',
                                           onInitialize = I('function() { this.setValue(""); }'))),
             selectizeInput('variates_batch',
                            'Select batch varaiable (to create ellipses)',
                            choices = c(),
                            multiple = FALSE,
                            selected = NULL,
                            options = list(placeholder = 'Please select an option below',
                            onInitialize = I('function() { this.setValue(""); }'))),
             checkboxInput("advanced_options_PCA",
                   label = "Advanced options",
                   value = FALSE),
             conditionalPanel(condition = "input.advanced_options_PCA == 1",
                 numericInput('top_n_PCA',
                     'Choose how many top variable features to use',
                     value = 0,
                     min = 0,
                     max = 500),
                 numericInput('firstPC',
                     'Please list the 2 PC\'s you would like to graph
                          \n x axis:',
                     value = 1,
                     min = 1,
                     max = 500),
                 numericInput('secondPC',
                     'y axis:',
                     value = 2,
                     min = 1,
                     max = 500),
                 checkboxInput("log_option",
                     label = "Select to log data (recommended for sequencing counts
                 data)", value = FALSE)),
             actionButton('PCA_plot', label = 'Here we go!')
           ),

           # Show a plot of the generated distribution
           mainPanel(
             plotOutput('PCA'),
             h4('Variance Explained'),
             tableOutput('var_explained')
           )
         )
)

tabPanel(
    "lambda/Variation Analysis",

    # Application title
    titlePanel("lambda/Variation Analysis"),

    sidebarLayout(sidebarPanel(
        h3("Variation Analysis"),
        selectizeInput('variation_assay', 'Select Assay Name', choices = "",
            options = list(placeholder = 'Please select an option below',
            onInitialize = I('function() { this.setValue(""); }'))),
        selectizeInput('variation_batch', 'Select Batch Variable', choices = "",
            options = list(placeholder = 'Please select an option below',
            onInitialize = I('function() { this.setValue(""); }'))),
        selectizeInput('variation_condition', 'Select Condition Variable(s)',
            choices = "",
            multiple = TRUE,
            options = list(placeholder = 'Please select an option below',
            onInitialize = I('function() { this.setValue(""); }'))),
        withBusyIndicatorUI(actionButton('variation', label = 'Here we go!'))
    ),
    mainPanel(
        tabsetPanel(
            tabPanel('Lambda Statistic',
                h4(strong("Usage")),
                h5("The lambda statistic can be used to compare uncorrected and
                    batch corrected datasets"),
                textOutput('lambda_rec')
            ),
            tabPanel("Explained Variation - Individual Variable",
                h5("The boxplot and p-value table display the individual, or raw
                    variation, explained by each variable."),
                plotOutput('EV_show_plot'),
                h5("Below are the summary statistics for the boxplot:"),
                dataTableOutput('EV_summary_stats'),
                h5("Below are the values for each gene that make up the
                        boxplot:"),
                dataTableOutput('EV_show_table')
            ),
            tabPanel("Explained Variation - Residual",
                h5("The boxplot and p-value table display the residual
                    varaition."),
                plotOutput('EV_residual_show_plot'),
                h5("Below are the summary statistics for the boxplot:"),
                dataTableOutput('EV_residual_summary_stats'),
                h5("Below are the values for each gene that make up the
                        boxplot:"),
                dataTableOutput('EV_residual_show_table')
            ),
            tabPanel("Individual Variation Variable/Batch Ratio",
                h5("The boxplot and table display the individual, or raw
                    variation, divided by the batch. A ratio less that 0
                    indicates that batch has a stronger affect than the
                    variable of interest."),
                conditionalPanel(condition = "input.variation_condition == ''",
                    h5(strong("Condition is required for ratio"))
                ),
                conditionalPanel(condition = "input.variation_condition != ''",
                    plotOutput('EV_ratio_plot'),
                    h5("Below are the values for each gene that make up the
                        boxplot:"),
                    dataTableOutput('EV_ratio_table')
                    )
            ),
            tabPanel("Residual Variation Variable/Batch Ratio",
                h5("The boxplot and table display the residual divided by the
                    batch. A ratio less that 1 indicates that batch has a
                    stronger affect than the variable of interest."),
                conditionalPanel(condition = "input.variation_condition == ''",
                    h5(strong("Condition is required for residual ratio"))
                ),
                conditionalPanel(condition = "input.variation_condition != ''",
                    plotOutput('EV_residual_ratio_plot'),
                    h5("Below are the values for each gene that make up the
                        boxplot:"),
                    dataTableOutput('EV_residual_ratio_table')
                )
            )
        )
        )
    )
)

### VARIATION ANALYSIS TAB ###

## Update covariate options to only those that are not confounded with batch
observeEvent(input$variation_batch, {
    req(reactivevalue$se, input$variation_batch)
    covariate_choices <- covariates_not_confounded(reactivevalue$se,
        input$variation_batch)
    updateSelectizeInput(session = session, inputId = "variation_condition",
        choices = covariate_choices, selected = NULL)
})

## Display variation and p-value plots and tables
observeEvent(input$variation, {
    req(input$variation_batch,
        input$variation_assay, reactivevalue$se)
    withBusyIndicatorServer("variation", {
        tryCatch({
            if (is.null(input$variation_condition)) {
                EV_results <- batchqc_explained_variation(se = reactivevalue$se,
                    batch = input$variation_batch,
                    assay_name = input$variation_assay)
                lambda_res <- list(lambda_stat = NULL,
                    correction_recommendation =
                        "Condition is required for lambda analysis.")
                EV_ratios <- NULL
                EV_residual_ratios <- NULL
            } else {
                EV_results <- batchqc_explained_variation(se = reactivevalue$se,
                    batch = input$variation_batch,
                    condition = input$variation_condition,
                    assay_name = input$variation_assay)
                EV_ratios <- variation_ratios(EV_results$EV_table_ind,
                    input$variation_batch)
                EV_residual_ratios <- variation_ratios(EV_results$EV_table_type2,
                    input$variation_batch)
                lambda_res <- run_lambda(se = reactivevalue$se,
                    assay = input$variation_assay,
                    batch = input$variation_batch,
                    condition = input$variation_condition
                   )

                output$EV_ratio_plot <- renderPlot({
                    ratio_plotter(EV_ratios)
                })

                output$EV_ratio_summary_stats <- renderDataTable({
                    summary_stats_EV_table(EV_ratios)
                })

                output$EV_ratio_table <- renderDataTable({
                    EV_ratios
                })

                output$EV_residual_ratio_plot <- renderPlot({
                    ratio_plotter(EV_residual_ratios)
                })

                output$EV_residual_ratio_summary_stats <- renderDataTable({
                    summary_stats_EV_table(EV_residual_ratios)
                })

                output$EV_residual_ratio_table <- renderDataTable({
                    EV_residual_ratios
                })
            }
        })

        output$lambda_rec <- renderText(lambda_res$correction_recommendation)

        output$EV_show_plot <- renderPlot({
            EV_plotter(EV_results$EV_table_ind)
        })

        output$EV_summary_stats <- renderDataTable({
            summary_stats_EV_table(EV_results$EV_table_ind)
        })

        output$EV_show_table <- renderDataTable({
            EV_table(EV_results$EV_table_ind)
        })
        output$EV_residual_show_plot <- renderPlot({
            EV_plotter(EV_results$EV_table_type2)
        })

        output$EV_residual_summary_stats <- renderDataTable({
            summary_stats_EV_table(EV_results$EV_table_type2)
        })

        output$EV_residual_show_table <- renderDataTable({
            EV_table(EV_results$EV_table_type2)
        })
    })
})

#' Heatmap Plotter
#'
#' This function allows you to plot a heatmap
#' @param se SummarizedExperiment
#' @param assay normalized or corrected assay
#' @param nfeature number of features to display
#' @param annotation_column choose column
#' @param log_option TRUE if data should be logged before plotting (recommended
#' for sequencing counts), FALSE if data should not be logged (for instance,
#' data is already logged)
#' @import pheatmap
#' @return heatmap plot
#' @examples
#' library(scran)
#' se <- mockSCE()
#' heatmaps <- BatchQC::heatmap_plotter(se,
#'                                 assay = "counts",
#'                                 nfeature = 15,
#'                                 annotation_column = c("Mutation_Status",
#'                                 "Treatment"), log_option = FALSE)
#' correlation_heatmap <- heatmaps$correlation_heatmap
#' correlation_heatmap
#'
#' heatmap <- heatmaps$topn_heatmap
#' heatmap
#'
#' @export
heatmap_plotter <- function(se, assay, nfeature, annotation_column,
    log_option) {
    data <- preprocess(se, assay, nfeature, log_option = FALSE)

    for (i in seq_len(nrow(data))) {
        data[i, ] <- (data[i, ] - mean(data[i, ])) / stats::sd(data[i, ])
    }

    col_info <- data.frame(colData(se))

    cor <- cor(data)
    if (!is.null(annotation_column)) {
        if (length(annotation_column) == 1) {
            col_info <- data.frame(col_info[, annotation_column],
                                row.names = rownames(col_info))
        }else {
            col_info <- col_info[, annotation_column]
        }

        # Numeric to Character conversion
        col_info <- heatmap_num_to_char_converter(ann_col = col_info)

        correlation_heatmap <- pheatmap(cor,
                                        annotation_col = col_info,
                                        # annotation_colors = ann_colors,
                                        annotation_row = col_info,
                                        show_colnames = FALSE,
                                        show_rownames = FALSE,
                                        annotation_names_col = FALSE,
                                        annotation_names_row = FALSE,
                                        silent = TRUE)

        topn_heatmap <- pheatmap(data, annotation_col = col_info,
                                    # annotation_colors = ann_colors,
                                    show_colnames = FALSE,
                                    annotation_names_col = FALSE,
                                    show_rownames = FALSE,
                                    silent = TRUE)

        return(list(correlation_heatmap = correlation_heatmap,
                    topn_heatmap = topn_heatmap))
    }
}

globalVariables(c("mod"))
#' Batch Correct
#' This function allows you to Add batch corrected count matrix to the SE object
#' @param se SummarizedExperiment object
#' @param method Normalization Method ("ComBat-Seq", "ComBat", "limma", "sva",
#' svaseq, "Harman")
#' @param assay_to_normalize Which assay use to do normalization
#' @param batch The batch
#' @param group The group variable
#' @param covar list of covariates
#' @param output_assay_name name of results assay
#' @param psva boolean; default: FALSE. Only used if normalization method is
#' "sva". If set to TRUE and no covariate input, psva function from the sva
#' package will be used to remove batch effect.
#' @param num_sv boolean; default: FALSE. Only used if normalization method is
#' "svaseq". The number of estimated latent factor is set to 1 for a small
#' number of samples. If set to TRUE, svaseq function will estimate the number
#' of latent factors for you.
#' @param limit numeric; default: 0.95. Only used if normalization method is
#' "Harman". Indicates the limit of confidence in which to stop removing a batch
#' effect. Must be between 0 and 1.
#' @param numrepeats integer; default: 100000L. Only used if normalization
#' method is "Harman". The number of repeats in which to run the simulated batch
#' mean distribution estimator using the random selection algorithm.
#' @usage batch_correct(se, method, assay_to_normalize, batch, group = NULL,
#' covar, output_assay_name, psva, num_sv, limit, numrepeats)
#' @return a summarized experiment object with normalized assay appended
#' @import SummarizedExperiment
#' @import sva
#' @examples
#' library(scran)
#' se <- mockSCE()
#' se <- BatchQC::batch_correct(se, method = "ComBat-Seq",
#'                                     assay_to_normalize = "counts",
#'                                     batch = "Mutation_Status",
#'                                     covar = "Treatment",
#'                                     output_assay_name =
#'                                         "ComBat_Seq_Corrected")
#' se <- BatchQC::batch_correct(se, method = "ComBat",
#'                                     assay_to_normalize = "counts",
#'                                     batch = "Mutation_Status",
#'                                     covar = "Treatment",
#'                                     output_assay_name =
#'                                         "ComBat_Corrected")
#' se
#'
#' @export
batch_correct <- function(se, method, assay_to_normalize, batch, group = NULL,
    covar, output_assay_name, psva = FALSE, num_sv = FALSE, limit = 0.95,
    numrepeats = 100000L) {
    se <- se
    var_of_interest <- batch
    batch <- data.frame(colData(se))[, batch]
    if (method == 'ComBat-Seq') {
        se <- ComBat_seq_correction(se, assay_to_normalize, batch, group, covar,
            output_assay_name)
    } else if (method == 'ComBat') {
        se <- ComBat_correction(se, assay_to_normalize, batch, covar,
            output_assay_name)
    } else if (method == 'limma') {
        se <- limma_correction(se, assay_to_normalize, batch, covar,
            output_assay_name)
    } else if (method == 'sva') {
        se <- sva_correction(se, assay_to_normalize, var_of_interest, covar,
            output_assay_name, psva)
    } else if (method == "svaseq") {
        se <- svaseq_correction(se, assay_to_normalize, var_of_interest, covar,
            output_assay_name, num_sv)
    } else if (method == "Harman") {
        se <- Harman_correction(se, assay_to_normalize, batch, covar,
            output_assay_name, limit, numrepeats)
    }
    return(se)
}

#' ComBat-Seq Correction
#' This function applies ComBat-seq correction to your summarized experiment
#' object
#' @param se SummarizedExperiment object
#' @param assay_to_normalize Assay that should be corrected
#' @param batch The variable that represents batch
#' @param group The group variable
#' @param covar list of covariates
#' @param output_assay_name name of results assay
#' @usage ComBat_seq_correction(se, assay_to_normalize, batch, group, covar,
#' output_assay_name)
#' @return SE object with an added ComBat-seq corrected array
#' @import SummarizedExperiment
#' @import sva

ComBat_seq_correction <- function(se, assay_to_normalize, batch,
    group, covar, output_assay_name) {
    if (is.null(covar)) {
        assays(se)[[output_assay_name]] <- ComBat_seq(as.matrix(
            assays(se)[[assay_to_normalize]]), batch = batch)
    } else {
        if (length(covar) == 1) {
            cov <- data.frame(colData(se))[, covar]
            cov <- as.factor(cov)
            cov <- as.numeric(cov)
            cov <- as.matrix(cov)
            rownames(cov) <- rownames(data.frame(colData(se)))

            if (!is.null(group)) {
                assays(se)[[output_assay_name]] <- ComBat_seq(
                    as.matrix(assays(se)[[assay_to_normalize]]),
                    batch = batch, covar_mod = cov, group = group,
                    full_mod = TRUE)
            } else {
                assays(se)[[output_assay_name]] <- ComBat_seq(as.matrix(
                    assays(se)[[assay_to_normalize]]),
                    batch = batch, covar_mod = cov, group = group)
            }
        } else {
            cov <- data.frame(colData(se))[, covar]
            for (i in seq_len(ncol(cov))) {
                cov[, i] <- as.factor(cov[, i])
                cov[, i] <- as.numeric(cov[, i])
            }

            if (!is.null(group)) {
                assays(se)[[output_assay_name]] <- ComBat_seq(as.matrix(
                    assays(se)[[assay_to_normalize]]),
                    batch = batch, covar_mod = cov, group = group,
                    full_mod = TRUE)
            } else {
                assays(se)[[output_assay_name]] <- ComBat_seq(as.matrix(
                    assays(se)[[assay_to_normalize]]),
                    batch = batch, covar_mod = cov, group = group)
            }
        }
    }
    return(se)
}

#' ComBat Correction
#' This function applies ComBat correction to your summarized experiment object
#' @param se SummarizedExperiment object
#' @param assay_to_normalize Assay that should be corrected
#' @param batch The variable that represents batch
#' @param covar list of covariates
#' @param output_assay_name name of results assay
#' @return SE object with an added ComBat corrected array
#' @import SummarizedExperiment
#' @import sva

ComBat_correction <- function(se, assay_to_normalize, batch,
    covar, output_assay_name) {
    if (is.null(covar)) {
        assays(se)[[output_assay_name]] <-
            ComBat(dat = assays(se)[[assay_to_normalize]], batch = batch)
    } else {
        if (length(covar) == 1) {
            cov <- data.frame(colData(se))[, covar]
            cov <- as.factor(cov)
            cov <- as.numeric(cov)
            cov <- data.frame(cov)
            colnames(cov) <- covar
            rownames(cov) <- rownames(data.frame(colData(se)))

            model <- stats::model.matrix(stats::as.formula(
                paste0('~', colnames(cov))), data = cov)
            results <- ComBat(dat = assays(se)[[assay_to_normalize]],
                batch = batch,
                mod = model)
            results[is.na(results)] <- 0
            assays(se)[[output_assay_name]] <- results
        } else {
            cov <- data.frame(colData(se))[, covar]

            for (i in seq_len(ncol(cov))) {
                cov[, i] <- as.factor(cov[, i])
                cov[, i] <- as.numeric(cov[, i])
            }

            cov <- data.frame(cov)
            rownames(cov) <- rownames(data.frame(colData(se)))
            colnames(cov) <- covar

            linearmodel <- stats::as.formula(paste0('~',
                paste(colnames(cov),
                    sep = '+')))
            model <- stats::model.matrix(linearmodel, data = cov)

            results <- ComBat(dat = assays(se)[[assay_to_normalize]],
                batch = batch,
                mod = model)
            results[is.na(results)] <- 0
            assays(se)[[output_assay_name]] <- results

        }
    }
    return(se)
}

#' Limma Correction
#' This function applies limma batch correction to your provided assay
#' @param se SummarizedExperiment object
#' @param assay_to_normalize Log assay that should be corrected
#' @param batch Factor containing batch information
#' @param covar list of covariates
#' @param output_assay_name name of results assay
#' @return SE object with an added limma corrected array
#' @import SummarizedExperiment
#' @importFrom limma removeBatchEffect
#'
limma_correction <- function(se, assay_to_normalize, batch, covar,
    output_assay_name) {
    if (is.null(covar)) {
        limma_corrected <- limma::removeBatchEffect(
            assays(se)[[assay_to_normalize]],
            batch = batch
            #design = model.matrix(~)
        )
    }else if (length(covar) == 1) {
        cov <- as.numeric(colData(se)[[covar]])
        limma_corrected <- limma::removeBatchEffect(
            assays(se)[[assay_to_normalize]],
            batch = batch,
            covariates = cov
        )
    }else {
        cov <- data.frame(colData(se))[, covar]

        for (i in seq_len(ncol(cov))) {
            cov[, i] <- as.factor(cov[, i])
            cov[, i] <- as.numeric(cov[, i])
        }

        cov <- data.frame(cov)
        rownames(cov) <- rownames(data.frame(colData(se)))
        colnames(cov) <- covar

        limma_corrected <- limma::removeBatchEffect(
            assays(se)[[assay_to_normalize]],
            batch = batch,
            covariates = cov
        )
    }
    assays(se)[[output_assay_name]] <- limma_corrected
    return(se)
}

#' sva Correction
#' This function applies sva correction to a summarized experiment object
#' (implementation adapted from sva::psva)
#' @param se SummarizedExperiment object
#' @param assay_to_normalize string; name of assay that should be corrected
#' @param var_of_interest string; name of  experimental variable of interest
#' @param covar list; sting list  of covariates to include in sva analysis
#' @param output_assay_name string; name of results assay
#' @param psva boolean; default: FALSE. If set to TRUE and no covariate input,
#' psva function from the sva package will be used to remove batch effect.
#' @return SE object with an added sva corrected array
#' @import SummarizedExperiment
#' @import sva

sva_correction <- function(se, assay_to_normalize, var_of_interest,
    covar, output_assay_name, psva = FALSE) {
    if (is.null(covar)) {
        mod0 <- model.matrix(~1, data = colData(se))
        mod1_formula <- as.formula(paste0("~", var_of_interest))
        mod1 <- model.matrix(mod1_formula, data = colData(se))
        n.sv <- sva::num.sv(assays(se)[[assay_to_normalize]], mod1,
            method = "leek")
        if (psva) {
            sva_assay <- sva::psva(
                dat = assays(se)[[assay_to_normalize]],
                batch = data.frame(colData(se))[, var_of_interest],
                mod = mod,
                mod0 = mod0,
                n.sv = n.sv)
        }
    }else {
        if (length(covar) == 1) {
            mod1_formula <- as.formula(paste0("~", var_of_interest, "+", covar))
            mod0_formula <- as.formula(paste0("~", covar))
        } else {
            mod1_formula <- as.formula(paste0("~", var_of_interest, "+",
                                            paste(covar, collapse = "+")))
            mod0_formula <- as.formula(paste0("~",
                                            paste(covar, collapse = "+")))
        }
        mod1 <- model.matrix(mod1_formula, data = colData(se))
        mod0 <- model.matrix(mod0_formula, data = colData(se))
    }
    sva_object <- sva::sva(
        dat = assays(se)[[assay_to_normalize]],
        mod = mod1,
        mod0 = mod0
    )

    fsva_object <- sva::fsva(
        dbdat = assays(se)[[assay_to_normalize]],
        mod = mod0,
        sv = sva_object,
        newdat = assays(se)[[assay_to_normalize]],
        method = "fast"
        )
    fsva_adjust_db <- fsva_object$db
    colnames(fsva_adjust_db) <- colnames(assays(se)[[assay_to_normalize]])
    assays(se)[[output_assay_name]] <- fsva_adjust_db
    return(se)
}

#' svaseq Correction
#' This function applies sva correction to a summarized experiment object
#' with count based RNA-seq data
#' @param se SummarizedExperiment object
#' @param assay_to_normalize string; name of assay that should be corrected
#' @param var_of_interest string; name of  experimental variable of interest
#' @param covar list; sting list  of covariates to include in sva analysis
#' @param output_assay_name string; name of results assay
#' @param num_sv boolean; Default is FALSE: the number of estimated latent
#' factor is set to 1 for a small number of samples. If set to TRUE, svaseq
#' function will estimate the number of latent factors for you.
#' @return SE object with an added sva corrected array
#' @import SummarizedExperiment
#' @import sva

svaseq_correction <- function(se, assay_to_normalize, var_of_interest,
    covar, output_assay_name, num_sv = FALSE) {
    dat <- assays(se)[[assay_to_normalize]]
    if (is.null(covar)) {
        mod0 <- model.matrix(~1, data = colData(se))
        mod1_formula <- as.formula(paste0("~", var_of_interest))
        mod1 <- model.matrix(mod1_formula, data = colData(se))
    }else {
        if (length(covar) == 1) {
            mod1_formula <- as.formula(paste0("~", var_of_interest, "+", covar))
            mod0_formula <- as.formula(paste0("~", covar))
        } else {
            mod1_formula <- as.formula(paste0("~", var_of_interest, "+",
                                            paste(covar, collapse = "+")))
            mod0_formula <- as.formula(paste0("~",
                                            paste(covar, collapse = "+")))
        }
        mod1 <- model.matrix(mod1_formula, data = colData(se))
        mod0 <- model.matrix(mod0_formula, data = colData(se))
    }
    if (num_sv) {
        batch_unsup_sva <- svaseq(dat, mod1, mod0)$sv
    } else {
        batch_unsup_sva <- svaseq(dat, mod1, mod0, n.sv = 1)$sv
    }
    colnames(batch_unsup_sva) <- paste('sv', seq_len(ncol(batch_unsup_sva)))
    mod1Sv <- cbind(mod1, batch_unsup_sva)
    psva.fit <- lmFit(dat, mod1Sv)
    sv_coef <- psva.fit$coefficients[, colnames(batch_unsup_sva), drop = FALSE]
    sv_effects <- sv_coef %*% t(batch_unsup_sva)
    svaseq_assay <- dat - sv_effects
    assays(se)[[output_assay_name]] <- svaseq_assay
    return(se)
}

#' Harman Correction
#' This function applies Harman correction to a summarized experiment object
#' with and reconstructs the data back into the original feature space
#' @param se SummarizedExperiment object
#' @param assay_to_normalize string; name of assay that should be corrected
#' @param batch factor; The variable that represents batch
#' @param covar string; name of covariate.
#' @param output_assay_name string; name of results assay
#' @param limit numeric; default: 0.95 Indicates the limit of confidence in
#' which to stop removing a batch effect. Must be between 0 and 1
#' @param numrepeats integer; default: 100000L the number of repeats in which
#' to run the simulated batch mean distribution estimator using the random
#' selection algorithm.
#' @return SE object with an added Harman corrected reconstructed data
#' @import SummarizedExperiment
#' @import Harman

Harman_correction <- function(se, assay_to_normalize, batch,
                                covar, output_assay_name, limit = 0.95,
                                numrepeats = 100000L) {
    if (!is.factor(batch)) {
        batch <- as.factor(batch)
    }
    dat <- assays(se)[[assay_to_normalize]]
    if (is.null(covar)) {
        cov <- NULL
    }else if (length(covar) > 1) {
        warning("Harman correction only supports NULL covar or one covar.
                Setting covar = NULL")
        cov <- NULL
    }else {
        cov <- as.factor(colData(se)[[covar]])
    }

    Harman_res <- harman(datamatrix = dat, expt = cov, batch = batch,
                        limit = limit, numrepeats = numrepeats)
    assays(se)[[output_assay_name]] <- as.data.frame(reconstructData(
        Harman_res))
    return(se)
}

globalVariables(c(glm.nb, AIC, glm, gaussian, glm.control))
#' Compute the AIC for lognormal (ComBat) model, negative binomial (ComBat-seq)
#' model and the Voom model
#'
#' This function calculates the AIC based on lognormal distribution,
#' negative binomial distribution as well as the voom transformation.
#' It then compares the AICs of the three models across different genes and
#' yields AIC-based metric values. Must have non-negative data and non-discrete
#' data will only analyze the lognormal and voom distribution.
#'
#' @param se SummarizedExperiment object
#' @param assay_of_interest The assay name from se that you are interested in
#'   analyzing. This assay need to be a counts assay containing only
#'   non-negative integers.
#' @param batchind Factor or numeric vector of length = ncol(dat);
#'   batch indicator for each sample.
#' @param groupind Factor or numeric vector of length = ncol(dat);
#'   biological group label/indicator for each sample.
#' @param maxit Integer giving the maximal number of IWLS iterations. Default is
#'   25.
#' @param zero_filt_percent Numeric value between 0 and 100, the percentage of
#'   zeros allowed for each gene to be included in the AIC calculation. Genes
#'   with more than this percentage of zeros will be filtered out. Default is
#'   100.
#'
#' @description
#'   \describe{
#'     \item{nb_result}{A vector contains the AIC based on negative binomial
#'     model for individual genes.}
#'     \item{lognormal_result}{A vector contains the AIC based on lognormal
#'     model for individual genes.}
#'     \item{voom_result}{A vector contains the AIC based on voom transformation
#'     for individual genes.}
#'     \item{total_AIC}{The sum of AICs across all genes for the models in
#'     comparison.}
#'     \item{min_AIC}{The number of minimum AIC across the models in
#'     comparison for individual genes.}
#'     \item{AIC_score}{The ratios between total_AIC and min_AIC for the
#'     models in comparison. The optimal model is the one has minimum value.}
#'   }
#'
#' @return A list with a  boxplot of the AIC_boxplot and a dataframe containing
#' the total_AIC values, the frequency of the min_AIC values, the AIC_score, and
#' the median_AIC value, for the following three elements:
#' \describe{
#'     \item{nb}{The metric value calculated based on the negative binomial
#'       model.}
#'     \item{lognormal}{The metric value calculated based on the lognormal
#'       model.}
#'     \item{voom}{The metric value calculated based on the voom-based model.}
#'   }
#'
#' @examples
#' library(scran)
#' se <- mockSCE()
#' compare_aic <- compute_aic(se, assay_of_interest = "counts",
#'                             batchind = "Cell_Cycle",
#'                             groupind = c("Treatment", "Mutation_Status"))
#' print(compare_aic)
#'
#' @importFrom limma voom
#' @importFrom stats AIC gaussian glm glm.control
#' @importFrom MASS glm.nb
#' @import SummarizedExperiment
#' @export

compute_aic <- function(se, assay_of_interest, batchind,
                        groupind, maxit = 25, zero_filt_percent = 100) {
    dat <- assays(se)[[assay_of_interest]]
    analysis_design <- as.data.frame(colData(se)[c(groupind, batchind)])
    design <- stats::model.matrix(stats::as.formula(paste(" ~",
                            paste(colnames(analysis_design), collapse = "+"))),
                            data = analysis_design)
    nb_test <- TRUE
    num_models <- 3

    if (any(dat < 0)) {
        stop("Counts must be non-negative values only.")
    } else if (!all(dat == floor(dat))) {
        nb_test <- FALSE
        num_models <- 2
    }

    dat <- dat[rowSums(dat) != 0, ]
    dat <- dat[rowSums(dat == 0) <= zero_filt_percent / 100 * ncol(dat), ]

    aic_matrix <- run_AIC_models(dat, design, nb_test, maxit)
    aic_median <- apply(aic_matrix, 2, median, na.rm = TRUE)
    total_AIC <- as.data.frame(t(colSums(aic_matrix, na.rm = TRUE)),
        colnames = "")

    min_model <- apply(aic_matrix, 1, function(x) {
        if (all(is.na(x))) return(NA)
        which.min(x)})

    min_AIC <- t(as.data.frame(table(factor(min_model,
        levels = seq_along(seq(1, num_models)))))[2])
    colnames(min_AIC) <- colnames(aic_matrix)

    AIC_boxplot <- AIC_boxplots(aic_matrix, num_models)

    AIC_score <- as.data.frame(total_AIC / min_AIC, row.names = "")
    AIC_table <- rbind(total_AIC, min_AIC, AIC_score, aic_median)
    rownames(AIC_table) <- c("Total AIC", "Frequency of Minimum AIC",
        "AIC Score", "Median AIC")

    return(list(AIC_table = AIC_table, AIC_boxplot = AIC_boxplot))
}
#' Helper function that contains the code to run the lognormal, voom, and
#' negative binomial AIC models for `compute_aic`
#'
#' @importFrom limma voom
#' @importFrom stats AIC gaussian glm glm.control
#' @importFrom MASS glm.nb
#' @import SummarizedExperiment
#'
#' @param dat dataframe of the data to analyze
#' @param design stats design model to be used in the analyses
#' @param nb_test boolean; should negative binomial run (must be discrete data)
#' @param maxit integer; the max number of IWLS iterations
#'
#' @return data frame; containing the AIC results of each method
#'

run_AIC_models <- function(dat, design, nb_test, maxit) {
    lognormal_result <- apply(dat, 1, function(x) {
        tryCatch({
            lognormal_model <- glm(log(x + 1e-100) ~ design, family = gaussian)
            lognormal_AIC <- AIC(lognormal_model)
            return(lognormal_AIC)
        }, error = function(e) {
            return(NA)
        })})

    voom_dat <- voom(dat, design = design)
    voom_dat <- voom_dat$E
    voom_result <- apply(voom_dat, 1, function(x) {
        tryCatch({
            voom_lm_model <- lm(x ~ design)
            voom_lm_AIC <- AIC(voom_lm_model)
            return(voom_lm_AIC)
        }, error = function(e) {
            return(NA)
        })})

    if (nb_test) {
        nb_result <- apply(dat, 1, function(x) {
            tryCatch({
                nb_model <- glm.nb(x ~ design,
                    control = glm.control(maxit = maxit))
                nb_AIC <- AIC(nb_model)
                return(nb_AIC)
            }, error = function(e) {
                return(NA)
            })})

        aic_matrix <- cbind(nb_result, lognormal_result, voom_result)
        colnames(aic_matrix) <- c("nb_AIC", "lognormal_AIC", "voom_AIC")

    } else {
        aic_matrix <- cbind(lognormal_result, voom_result)
        colnames(aic_matrix) <- c("lognormal_AIC", "voom_AIC")
    }

    return(aic_matrix)
}

#' Boxplots for the distribution of AIC for each method
#'
#' This function creates a boxplot of all the AIC values for each gene under
#' each tested distribution to aid in identifying outliers
#'
#' @import tidyr
#'
#' @param AIC_data dataframe with the data to be plotted
#' @param num_methods integer representing the number of distribution methods
#'
#' @return AIC_boxplot; a boxplot for each method showing distribution of data

AIC_boxplots <- function(AIC_data, num_methods) {
    method <- "method"
    AIC_data <- AIC_data %>%
        as.data.frame() %>%
        rownames_to_column(var = "feature") %>%
        pivot_longer(cols = 2:(num_methods + 1),
            names_to = method,
            values_to = "AIC")

    AIC_plot <- ggplot(AIC_data, aes(x = method, y = AIC, color = method)) +
        geom_boxplot() +
        geom_point(position = position_jitterdodge(jitter.width = 0.2,
                                                    dodge.width = 0.75),
            alpha = 0.5)
}

tabPanel("Dendrograms",

         # Application title
         titlePanel("Dendrograms"),

         sidebarLayout(
             sidebarPanel(
                 selectizeInput('dend_assay_name',
                     'Choose assay to display',
                     choices = c(),
                     multiple = FALSE,
                     selected = NULL,
                     options = list(placeholder = 'Please select an option below',
                         onInitialize = I('function() { this.setValue(""); }'))),
                 selectizeInput('dend_batch_to_display',
                     'Choose leaf variable to display on dendogram',
                     choices = c(),
                     multiple = FALSE,
                     selected = NULL,
                     options = list(placeholder = 'Please select an option below',
                                    onInitialize = I('function() { this.setValue(""); }'))),
                 selectizeInput('dend_category_to_display',
                     'Choose stem variable to display on dendogram',
                     choices = c(),
                     multiple = FALSE,
                     selected = NULL),
                 actionButton('dend_plot', label = 'Here we go!')
             ),

             # Show a plot of the dendrograms
             mainPanel(
                 tabsetPanel(
                     tabPanel("Dendrogram",
                              plotOutput('dendrogram')
                     ),
                     tabPanel("Circular Dendrogram",
                              plotOutput('circular_dendrogram')
                     )
                 )
             )
         )
)

#' Dendrogram alpha or numeric checker
#'
#' This function checks if there is any numeric or strings for plotting legend
#' @param dendro_var column from dendrogram object representing category
#' @import tibble
#' @import ggdendro
#' @import dplyr
#' @return geom_label label for the legend of category variable
#' @examples
#' library(scran)
#' se <- mockSCE()
#' dendro_alpha_numeric_check <- dendrogram_alpha_numeric_check(
#'                                         dendro_var = "Treatment")
#' dendro_alpha_numeric_check
#'
#' @export
#'
dendrogram_alpha_numeric_check <- function(dendro_var) {
    numeric_or_alpha <- levels(factor(dendro_var))
    all_numeric <- TRUE

    for (n in numeric_or_alpha) {
        if (!is.numeric(n)) {
            all_numeric <- FALSE
            break
        }
    }

    if (all_numeric) {
        geom_label <- as.character(sort(as.numeric(levels(factor(
            dendro_var)))))

    } else {
        # Get unique 'dendro_var' values, convert to dataframe
        unique_strings <- levels(factor(dendro_var)) %>%
            as.data.frame() %>% rownames_to_column("unique_string_index")

        # Concatenate values from 'unique_strings' with "_"
        label_var_strings <- paste(unlist(unique_strings[1]),
                                        unlist(unique_strings[2]),
                                        sep = " - ") %>%
            as.data.frame() %>% dplyr::rename("label_var" = ".")

        geom_label <- label_var_strings[, "label_var"]
    }

    return(geom_label)
}

#' Returns list of covariates not confounded by batch; helper function for
#' explained variation and for populating shiny app condition options
#'
#' @param se Summarized experiment object
#' @param batch Batch variable
#' @return List of explained variation by batch and condition
#' @examples
#' library(scran)
#' se <- mockSCE()
#' covariates_not_confounded <- BatchQC::covariates_not_confounded(se,
#'                                             batch = "Mutation_Status")
#' covariates_not_confounded
#'
#' @export

covariates_not_confounded <- function(se, batch) {
    df <- confound_metrics(se, batch)
    covariate_options <- rownames(df)
    for (i in seq_len(dim(df)[1])) {
        if (df[i] == 1 || is.na(df[i])) {
            covariate_options <- covariate_options[!(covariate_options) %in%
                    rownames(df)[i]]
        }
    }
    return(covariate_options)
}

### KBET TAB ###

## run kBET
observeEvent(input$run_kBET, {
  req(reactivevalue$se, input$kbet_assay, input$kbet_batch)
  msg <- sprintf("Generating plot for: %s...", input$kbet_assay)
  withProgress(message = msg, {
    setProgress(.5, "Running  kBET...")
    results <- run_kBET(
      se = reactivevalue$se,
      assay_to_normalize = input$kbet_assay,
      batch = input$kbet_batch,
      k0 = input$kbet_k0,
      do.pca = (input$kbet_doPCA == "Yes"),
      dim.pca = input$kbet_dimPCA,
      heuristic = (input$kbet_heuristic == "Yes"),
      alpha = input$kbet_alpha
    )
    setProgress(.8, "Displaying figure...")
    output$kBET_plot <- renderPlot({
      plot_kBET(results)
    })
    setProgress(1, "Complete.")
  })
})

### PCA TAB ###

## Plot PCA
observeEvent(input$PCA_plot, {
    req(reactivevalue$se)
    validate(need(input$top_n_PCA <= dim(reactivevalue$se)[1] &&
                      input$top_n_PCA > 1, "Please select a value for the top
                variable features to use that falls within 2 and the size of
                your dataset"))
    assays <- input$pca_assays
    msg <- sprintf('Generating plot for: %s...', paste(assays, collapse = ', '))
    withProgress(message = msg, {
        results <- PCA_plotter(reactivevalue$se,
                            input$top_n_PCA,
                            input$variates_color,
                            input$variates_shape,
                            input$variates_batch,
                            assays,
                            input$firstPC,
                            input$secondPC,
                            input$log_option)
        setProgress(.8, 'Displaying figure...')
        output$PCA <- renderPlot({
            validate(need(input$top_n_PCA <= dim(reactivevalue$se)[1] &&
                    input$top_n_PCA > 1, "Please select a value for the top
                variable features to use that falls within 2 and the size of
                your dataset"))
            results$plot})
        output$var_explained <- renderTable(
            results$var_explained, rownames = TRUE, digits = 4)
        setProgress(1, 'Complete.')
    })
})

globalVariables(c("protein_sample_info", "protein_data", "batch_indicator",
    "signature_data", "bladder_meta", "bladder_data", "merged_IDs"))

#' Batch and Condition indicator for protein expression data
#'
#' This data consists of two batches and two conditions
#' corresponding to case and control for the protein expression data
#'
#' @name protein_sample_info
#' @docType data
#' @format A data frame with 24 rows and 2 variables:
#' \describe{
#'     \item{batch}{Batch Indicator}
#'     \item{category}{Condition (Case vs Control) Indicator}
#' }
#' @keywords datasets
#' @usage data(protein_sample_info)
"protein_sample_info"

#' Protein data with 39 protein expression levels
#'
#' This data consists of two batches and two conditions
#' corresponding to case and control. The columns are case/control
#' samples, and the rows represent 39 different proteins.
#'
#' @name protein_data
#' @docType data
#' @format A data frame with 39 rows and 24 variables
#' @keywords datasets
#' @usage data(protein_data)
"protein_data"


#' Batch and Condition indicator for signature data
#'
#' This dataset is from signature data captured when activating different growth
#' pathway genes in human mammary epithelial cells (GEO accession: GSE73628).
#' This data consists of three batches and ten different conditions
#' corresponding to control and nine different pathways.
#'
#' @name batch_indicator
#' @docType data
#' @format A data frame with 89 rows and 2 variables:
#' \describe{
#'     \item{batch}{batch}
#'     \item{condition}{condition}
#' }
#' @keywords datasets
#' @usage data(batch_indicator)
"batch_indicator"

#' Signature data with 1600 gene expression levels
#'
#' This data consists of three batches and ten conditions.
#' The columns are samples, and the rows represent
#' 1600 different genes.
#'
#' @name signature_data
#' @docType data
#' @format A data frame with 1600 rows and 89 variables
#' @keywords datasets
#' @usage data(signature_data)
"signature_data"

#' Bladder data upload
#' This function uploads the Bladder data set from the bladderbatch package.
#' This dataset is from bladder cancer data with 22,283 different microarray
#' gene expression data. It has 57 bladder samples with 3 metadata variables
#' (batch, outcome and cancer). It contains 5 batches, 3 cancer types (cancer,
#' biopsy, control), and 5 outcomes (Biopsy, mTCC, sTCC-CIS, sTCC+CIS, and
#' Normal). Batch 1 contains only cancer, 2 has cancer and controls, 3 has only
#' controls, 4 contains only biopsy, and 5 contains cancer and biopsy
#'
#' @usage bladder_data_upload()
#' @return a SE object with counts data and metadata
#'
#' @examples
#' library(bladderbatch)
#' se_object <- bladder_data_upload()
#'
#' @export

bladder_data_upload <- function() {
    if (!requireNamespace("bladderbatch")) {
        stop("You need to install the 'bladderbatch' package to use this
            data set.")
    }
    data(bladderdata, package = "bladderbatch", envir = environment())
    bladderEset <- bladderEset
    pheno <- pData(bladderEset) %>% select(-sample)
    edata <- exprs(bladderEset)
    se_object <- BatchQC::summarized_experiment(edata, pheno)
    colData(se_object)[['batch']] <- as.factor(colData(se_object)[['batch']])
    return(se_object)
}

#' BMI and matched sample names for TB data
#'
#' This is support data for the TB data set that contains the BMI data and ID
#' numbers from both the curatedTBData database and the original study the data
#' was used in
#'
#' @name merged_IDs
#' @docType data
#' @format A data frame with 91 rows and 3 columns
#' \describe{
#'     \item{subjectID_curatedTBData}{Subject ID found in curatedTBData}
#'     \item{subjectID_TB_Paper}{Subject ID in the original paper}
#'     \item{BMI}{subject's BMI from the original study}
#' }
#' @keywords datasets
#' @usage data(merged_IDs)
"merged_IDs"


#' TB data upload
#' This function uploads the TB data set from the curatedTBData package.
#'
#' @usage tb_data_upload()
#' @return a SE object with raw counts data and metadata
#' @import SummarizedExperiment
#' @import dplyr
#'
#' @examples
#' library(curatedTBData)
#' se_object <- tb_data_upload()
#'
#' @export

tb_data_upload <- function() {
    if (!requireNamespace("curatedTBData")) {
        stop("You need to install the 'curatedTBData' package to use this
            data set.")
    } else if (!requireNamespace("MultiAssayExperiment")) {
        stop("You need to install the 'MultiAssayExperiment' package to use this
            data set.")
    }
    curatedData <- curatedTBData::curatedTBData(c("GSE152218", "GSE101705"),
        dry.run = FALSE, curated.only = FALSE)

    batch1_se <- MultiAssayExperiment::experiments(
        curatedData$GSE152218)$object_raw
    batch1_data <- SummarizedExperiment::assays(batch1_se)$assay_raw
    batch1_metadata <- MultiAssayExperiment::colData(curatedData$GSE152218)
    batch1_metadata$Experiment <- rep("GSE152218", length(batch1_metadata[, 1]))
    batch1_metadata <- batch1_metadata %>% as.data.frame() %>%
        select("TBStatus", "HIVStatus", "BMI", "Experiment")
    batch2_data <- MultiAssayExperiment::experiments(
        curatedData$GSE101705)$assay_reprocess_hg38 %>%
        as.data.frame() %>%
        select(-"GSM2712712")
    batch2_metadata <- MultiAssayExperiment::colData(curatedData$GSE101705)
    batch2_metadata <-
        batch2_metadata[-which(rownames(batch2_metadata) == "GSM2712712"), ]
    batch2_metadata$Experiment <- rep("GSE101705", length(batch2_metadata[, 1]))

    batch2_metadata <- BMI_data(batch2_metadata)
    batch2_metadata <- batch2_metadata %>% as.data.frame() %>%
        select("TBStatus", "HIVStatus", "BMI", "Experiment")

    all_data <- merge(batch1_data, batch2_data, by = 0)
    rownames(all_data) <- all_data$Row.names
    all_data <- all_data %>% select(-1) %>% as.matrix() %>% round()
    all_metadata <- rbind(batch1_metadata, batch2_metadata)

    se <- summarized_experiment(all_data, all_metadata)
    colData(se)[['Experiment']] <- as.factor(colData(se)[['Experiment']])
    colData(se)[['TBStatus']] <- as.factor(colData(se)[['TBStatus']])
    colData(se)[['HIVStatus']] <- as.factor(colData(se)[['HIVStatus']])
    colData(se)[['BMIcat']] <- as.factor(vapply(colData(se)[['BMI']],
        function(x) {
            if (x < 16) {
                return("mal")
            }else {
                return("well")
                }
            }, FUN.VALUE = character(1)))
    return(se)
}

#' This function returns BMI data that comes form the data in "Comparing
#' tuberculosis gene signatures in malnourished individuals using the
#' TBSignatureProfiler" paper. Subject IDs were matched as shown on
#' "github.com/jessmcc22/BatchQCv2_Manuscript/blob/devel/R/subjectID_match.R"
#'
#' @param meta dataframe; metadata that needs to be matched to BMI
#' @usage BMI_data(meta)
#' @return dataframe provided as input with BMI info added
#'
BMI_data <- function(meta) {
    data("merged_IDs", envir = environment())
    subjectID_curatedTBData <- merged_IDs$subjectID_curatedTBData
    BMI <- merged_IDs$BMI
    IDs <- data.frame(subjectID_curatedTBData, BMI)
    meta$BMI <- rep(NA, length(meta$Experiment))
    for (i in seq_len(nrow(meta))){
        position <- which(IDs$subjectID_curatedTBData == rownames(meta)[i])
        meta$BMI[i] <- IDs$BMI[position]
    }
    return(meta)
}

#' Returns a list of explained variation by batch and condition
#' combinations
#'
#' @param se Summarized experiment object
#' @param batch Batch covariate
#' @param condition Condition covariate(s) of interest if desired, default is
#'   NULL
#' @param assay_name Assay of choice
#' @importFrom stats formula model.matrix
#' @import SummarizedExperiment
#' @return List of explained variation by batch and condition
#'
#' @examples
#' library(scran)
#' se <- mockSCE()
#' batchqc_explained_variation <- BatchQC::batchqc_explained_variation(se,
#'                                         batch = "Mutation_Status",
#'                                         condition = "Treatment",
#'                                         assay_name = "counts")
#' batchqc_explained_variation
#'
#' @export
#'

batchqc_explained_variation <- function(se, batch, condition = NULL,
    assay_name) {
    se <- check_valid_input(se, batch, condition)

    sample_info <- SummarizedExperiment::colData(se)
    assay_dat <- t(assay(se, assay_name))

    ## Total RSS
    mus <- matrix(colMeans(assay_dat), nrow(assay_dat), ncol(assay_dat),
        byrow = TRUE)
    res_total <- colSums((assay_dat - mus)^2)

    ## Full RSS and SST
    covs <- c(batch, condition)
    model_full <- stats::formula(paste("~", paste(covs, collapse = " + ")))
    design_full <- stats::model.matrix(model_full, data = sample_info)
    res_full <- get.res(assay_dat, design_full)
    EV_table_ind <- EV_table_type2 <- data.frame(Explained = res_total -
            res_full)

    ### individual SSE and Type 2 SSE
    for (j in seq_len(length(covs))){
        ### individual
        model_ind <- stats::formula(paste("~ ", covs[j], sep = ''))
        design_ind <- stats::model.matrix(model_ind, data = sample_info)
        res_ind <-  get.res(assay_dat, design_ind)
        EV_table_ind[covs[j]] <- res_total - res_ind

        ### Type2
        if (is.null(condition)) {
            EV_table_type2[covs[j]] <- EV_table_ind[covs[j]]
        } else {
            model_alt <- stats::formula(paste("~",
                paste(covs[-j], collapse = " + ")))
            design_alt <- stats::model.matrix(model_alt, data = sample_info)
            res_alt <- get.res(assay_dat, design_alt)
            EV_table_type2[covs[j]] <- res_alt - res_full
        }
    }

    EV_table_ind["Unexplained"] <- EV_table_type2["Unexplained"] <- res_full
    EV_table_ind <- round(100 * EV_table_ind / res_total, 2)
    EV_table_type2 <- round(100 * EV_table_type2 / res_total, 2)

    return(list(EV_table_ind = EV_table_ind, EV_table_type2 = EV_table_type2))
}

#' Helper function to get residuals
#'
#' @param y assay
#' @param X model matrix design
#' @return residuals
get.res <- function(y, X) {
    colSums((y - X %*% solve(t(X) %*% X) %*% t(X) %*% y)^2)
}

#' Helper function to save variables as factors if not already factors
#'
#' @param se se object
#' @param batch batch
#' @param condition condition
#' @return se se object
#' @keywords Internal

check_valid_input <- function(se, batch, condition) {
    if (!is.factor(se[[batch]])) {
        se[[batch]] <- as.factor(se[[batch]])
    }else if (!is.null(condition)) {
        for (variable in condition){
            if (!is.factor(se[[variable]])) {
                se[[variable]] <- as.factor(se[[variable]])
            }
        }
    }

    return(se)
}

#' This function allows you to plot explained variation
#' @param batchqc_ev table of explained variation from
#'   batchqc_explained_variation
#' @import reshape2
#' @import ggplot2
#' @import ggpubr
#' @return boxplot of explained variation
#' @examples
#' library(scran)
#' se <- mockSCE()
#' se$Mutation_Status <- as.factor(se$Mutation_Status)
#' se$Treatment <- as.factor(se$Treatment)
#' expl_var_result <- batchqc_explained_variation(se, batch = "Mutation_Status",
#'                             condition = "Treatment", assay_name = "counts")
#' EV_boxplot <- BatchQC::EV_plotter(expl_var_result[[1]])
#' EV_boxplot
#'
#' @export
EV_plotter <- function(batchqc_ev) {

    compare <- list()
    i <- 2
    while (i < length(colnames(batchqc_ev))) {
        j <- i + 1
        while (i < length(colnames(batchqc_ev)) &&
                j < (length(colnames(batchqc_ev)))) {
            compare <- c(compare, list(c(colnames(batchqc_ev)[i],
                colnames(batchqc_ev)[j])))
            j <- j + 1
        }
        i <- i + 1
    }

    EV_boxplot <- ggplot(data =
            melt(as.data.frame(batchqc_ev),
                id.vars = NULL),
        aes(x = variable, y = value, fill = variable)) +
        geom_boxplot() +
        scale_x_discrete(name = "") +
        scale_y_continuous(name = "Percent Explained Variation") +
        labs(title = "Percent of Variation Explained by Source") +
        theme(legend.position = "none",
            plot.title = element_text(hjust = 0.5)) +
        stat_compare_means(comparisons = compare)
    return(EV_boxplot)
}

#' EV Table
#' Returns table with percent variation explained for specified number of genes
#'
#' @param batchqc_ev explained variation results from
#'   batchqc_explained_variation
#' @importFrom data.table data.table
#' @return List of explained variation by batch and condition
#' @examples
#' library(scran)
#' se <- mockSCE()
#' se$Mutation_Status <- as.factor(se$Mutation_Status)
#' se$Treatment <- as.factor(se$Treatment)
#' exp_var_result <- BatchQC::batchqc_explained_variation(se,
#'                                     batch = "Mutation_Status",
#'                                     condition = "Treatment",
#'                                     assay_name = "counts")
#' EV_table <- BatchQC::EV_table(exp_var_result[[1]])
#'
#' EV_table
#'
#' @export
EV_table <- function(batchqc_ev) {
    EV_table <- data.table(batchqc_ev,
        keep.rownames = TRUE)
    colnames(EV_table)[1] <- "Gene Name"
    return(EV_table = EV_table)
}

#' Summary Stats EV Table
#' Returns table with min, 1st quartile, mean, 2nd quartile, and max for each
#' variable in the explained variation boxplot
#'
#' @param batchqc_ev explained variation results from
#' @return summary_stats_table dataframe containing the min, 1st quartile, mean,
#'   2nd quartile, and max for each variable in the explained variation boxplot
#'
#' @examples
#' library(scran)
#' se <- mockSCE()
#' se$Mutation_Status <- as.factor(se$Mutation_Status)
#' se$Treatment <- as.factor(se$Treatment)
#' exp_var_result <- BatchQC::batchqc_explained_variation(se,
#'                                     batch = "Mutation_Status",
#'                                     condition = "Treatment",
#'                                     assay_name = "counts")
#' summary_stats_table <- BatchQC::summary_stats_EV_table(exp_var_result[[1]])
#'
#' summary_stats_table
#'
#' @export
#'
#'
summary_stats_EV_table <- function(batchqc_ev) {
    sum_table <- do.call(cbind, lapply(batchqc_ev, summary))
    return(as.data.frame(sum_table))
}

tabPanel('Batch Correction/Normalization',
    titlePanel("Batch Correction/Normalization"),
    sidebarLayout(
        sidebarPanel(
            selectizeInput('correction_assay',
                'Choose the assay of interest',
                multiple = FALSE,
                choices = c(''),
                selected = NULL,
                options = list(placeholder =
                        'Please select an option below',
                    onInitialize = I(
                        'function() { this.setValue(""); }')
                ))
        ),
        mainPanel(
            tabsetPanel(
                tabPanel('Negative Binomial Check',
                    h4(strong("Usage")),
                    h5("This features allows you to check your data (must be counts/whole number data) to see if it conforms to the required negative binomial assumption needed for various downstream analysis. If The negative binomial assumption is not met, you should normalize your data or perform other preprocessing step and/or use other more appropraite analysis tools. We recommend using the edgeR tool as the DESeq2 tool has not been fully analyzed, but is available at the user's discretion."),
                    selectizeInput('condition_of_interest',
                        'Select the variable you are interested in analyzing',
                        multiple = FALSE,
                        choices = c(''),
                        selected = NULL, options = list(placeholder =
                                'Please select an option below',
                            onInitialize = I(
                                'function() { this.setValue(""); }'
                            ))),
                    selectizeInput('nb_variables',
                        'Select other variables you would like to include in your analysis',
                        multiple = TRUE,
                        choices = c(''),
                        selected = NULL,
                        options = list(placeholder =
                                'Please select an option below',
                            onInitialize = I(
                                'function() { this.setValue(""); }'
                            ))),
                    checkboxInput('nb_advanced_options', 'Advanced Options',
                        value = FALSE),
                    conditionalPanel(condition = "input.nb_advanced_options == 1",
                        selectizeInput('nb_method',
                            'Choose the method to utilize',
                            multiple = FALSE,
                            choices = c('DESeq2', 'edgeR'),
                            selected = 'edgeR',
                            options = list(placeholder =
                                    'Please select an option below',
                                onInitialize = I(
                                    'function() { this.setValue("edgeR"); }'
                                ))),
                        numericInput('num_genes',
                            'Number of genes to analyze (downsampling)',
                            value = 500,
                            min = 2),
                        numericInput('small_cutoff',
                            'Small Sample Size Cutoff',
                            value = 20,
                            min = 2)),
                    withBusyIndicatorUI(actionButton(inputId = 'nb_check',
                        label = 'Check Distribution')),
                    textOutput('recommendation'),
                    plotOutput('nb_histogram'),
                    textOutput('reference'),
                    br()
                ),
                tabPanel("AIC Computation",
                    h4(strong("Usage")),
                    h5("This feature allows you to compute the AIC for lognormal (ComBat) model, negative binomial (ComBat-seq) model, and the Voom model.
                       Generally, the smallest value returned by the AIC model indicates the distribution of your data that should be considered when selecting downtream tools."),
                    selectizeInput('aic_batch',
                        'Select the batch variable',
                        multiple = FALSE,
                        choices = c(''),
                        selected = NULL,
                        options = list(placeholder =
                                'Please select an option below',
                                onInitialize = I(
                                    'function() { this.setValue(""); }'
                                ))),
                    selectizeInput('aic_covar',
                        'Select the covariate variable',
                        multiple = TRUE,
                        choices = c(''),
                        selected = NULL,
                        options = list(placeholder =
                            'Please select an option below',
                            onInitialize = I(
                                'function() { this.setValue(""); }'
                            ))),
                    h5("Adjust advanced options if numerous warnings regarding theta estimation are received."),
                    h6("Note: Needing these adjustments suggests NB may not be suitable for your data!"),
                    checkboxInput('aic_advanced_options', 'Advanced Options',
                                  value = FALSE),
                    conditionalPanel(condition = "input.aic_advanced_options == 1",
                                     numericInput('aic_nb_maxit',
                                                  'Max iterations for NB model fitting (increase if too many "iteration limit reached" warnings occur)',
                                                  value = 25,
                                                  min = 25)),
                    conditionalPanel(condition = "input.aic_advanced_options == 1",
                                     numericInput('aic_zero_filt_percent',
                                                 'Percentage of zeros allowed for each gene to be included in the AIC calculation (decrease to reduce warnings).',
                                                 min = 0,
                                                 max = 100,
                                                 value = 100)),
                    actionButton(inputId = 'compute_aic', label = 'Compute AIC'),
                    br(),
                    tableOutput('aic_table'),
                    h5("Total AIC: The sum of AICs across all genes for the three models in comparison."),
                    h5("Frequency of Minimum AIC: The number of minimum AIC across the three models in comparison for individual genes."),
                    h5("AIC score: total_AIC / Frequency_min_AIC; the lowest value is likely the best fit distribution."),
                    h5("Median AIC: The median AIC value for each model"),
                    br(),
                    h5("Boxplot displaying the distribution of AIC values for each method:"),
                    plotOutput('aic_boxplot')
                ),
                tabPanel('Batch Correction',
                    h4(strong("Usage")),
                    conditionalPanel(condition = "input.correction_method == 'ComBat-Seq'",
                        h5("ComBat-Seq uses a negative binomial regression to model batch effects. It requires untransformed, raw count data to adjust for batch effect. Please use this option with a counts assay")
                    ),
                    conditionalPanel(condition = "input.correction_method == 'ComBat'",
                        h5("ComBat corrects for Batch effect using a parametric empirical Bayes framework and data should be cleaned and normalized. Therefore, please select a normalized assay to run this on.")
                    ),
                    conditionalPanel(condition = "input.correction_method == 'limma'",
                        h5("limma batch correction fits a linear model to the data, including batch and regular treatments, then removes the component due to the batch effects. Please run limma on log expression data.")
                    ),
                    conditionalPanel(condition = "input.correction_method == 'sva'",
                        h5("sva correction identifies surrogate variables to correct for unknown or known batch effects. This method here estimates the surrogate variables using sva, and use fsva frozen
                           surrogate variable analysis to remove the surrogate variables inferred from sva. Use of psva, the 2 step approach proposed by Leek and Storey 2007, is available by setting additional argument.")
                    ),
                    conditionalPanel(condition = "input.correction_method == 'svaseq'",
                                     h5("svaseq correction is a variant of sva correction for sequencing data")
                    ),
                    conditionalPanel(condition = "input.correction_method == 'Harman'",
                                     h5("a PCA and constrained optimisation based technique that maximizes the removal of batch effects from datasets, with the constraint that the probability of overcorrection (i.e. removing genuine biological signal along with batch noise) is kept to a fraction")
                    ),
                    selectizeInput('correction_method', 'Choose correction method',
                        multiple = FALSE,
                        choices = c('ComBat-Seq', 'ComBat', 'limma', 'sva', 'svaseq', 'Harman'),
                        selected = NULL,
                        options = list(placeholder =
                                'Please select an option below',
                            onInitialize = I(
                                'function() { this.setValue(""); }'
                            ))),
                    selectizeInput('correction_batch',
                        'Select the variable that represents batch (or the experimental variable for sva)',
                        multiple = FALSE,
                        choices = c(''),
                        selected = NULL,
                        options = list(placeholder =
                                'Please select an option below',
                            onInitialize = I(
                                'function() { this.setValue(""); }')
                        )),
                    selectizeInput('correction_covariates',
                        'Choose the covariates you would like to preserve (or for sva, include as adjustment variables)',
                        multiple = TRUE,
                        choices = c(''),
                        selected = NULL,
                        options = list(placeholder =
                                'Please select an option below',
                            onInitialize = I(
                                'function() { this.setValue(""); }')
                        )),
                    textInput(inputId = 'corrected_assay_name',
                        'Name for the corrected assay'),
                    conditionalPanel(condition = "input.correction_method == 'svaseq'",
                                     checkboxInput('svaseq_num_sv',
                                                   'Uncheck this if the number of samples is small (the number of latent factors that need to be estimated (n.sv) is set to 1);
                                                   otherwise, svaseq function will estimate n.sv for you.',
                                                   value = FALSE)
                    ),
                    conditionalPanel(condition = "input.correction_method == 'sva'",
                                     checkboxInput('sva_psva',
                                                   'Check this if you have no covariate and want to use psva to remove batch effect. \n
                                                   Parker HS, Leek JT, Favorov AV, Considine M, Xia X, Chavan S, Chung CH, Fertig EJ (2014) Preserving biological heterogeneity with a permuted surrogate variable analysis for genomics batch correction Bioinformatics doi: 10.1093/bioinformatics/btu375',
                                                   value = FALSE)
                    ),
                    conditionalPanel(condition = "input.correction_method == 'Harman'",
                        checkboxInput('Harman_options',
                            label = 'Advaced Options for Harman',
                            value = FALSE),
                        conditionalPanel(condition = "input.Harman_options == 1",
                                numericInput('Harman_limit',
                                    'Indicates the limit of confidence in which to stop removing a batch effect. Must be between 0 and 1',
                                    value = 0.95,
                                    min = 0,
                                    max = 1),
                                numericInput('Harman_numrepeats',
                                    'the number of repeats in which to run the simulated batch mean distribution estimator using the random selection algorithm',
                                    value = 100000L)
                        )
                    ),
                    actionButton(inputId = 'correct', label = 'Correct')
                    ),
                tabPanel('Normalization',
                    h4(strong("Usage")),
                    conditionalPanel(condition = "input.normalization_method == 'CPM'",
                        h5("CPM calculates the counts mapped to a feature relative to the total counts mapped to a sample times one million.")
                    ),
                    conditionalPanel(condition = "input.normalization_method == 'DESeq'",
                        h5("DESeq calculates the counts mapped to a feature divided by sample-specific size factors. Size factors are determined by the median ratio of gene counts relative to the geometric mean per feature.")
                        ),
                    conditionalPanel(condition = "input.normalization_method == 'edgeR'",
                        h5("edgeR calculates scale factors using a trimmed mean of M-values between each pair of samples and multiplies the scale factors with the original library size to get the normalized library size.")
                        ),
                    conditionalPanel(condition = "input.normalization_method == 'voom'",
                        h5("voom calculates logCPM, estimates mean-variance relationship and uses this to compute observation-level weights. Appropriate for count data to be used with limma.")
                    ),
                    selectizeInput('normalization_method',
                        'Choose normalization method',
                        multiple = FALSE,
                        choices = c('CPM', 'DESeq', 'edgeR', 'voom', 'none'),
                        selected = NULL,
                        options = list(placeholder =
                                'Please select an option below',
                            onInitialize = I(
                                'function() { this.setValue(""); }'
                            ))),
                    selectizeInput('normalized_batch',
                        'Select the variable that represents batch (required for voom)',
                        multiple = FALSE,
                        choices = c(''),
                        selected = NULL,
                        options = list(placeholder =
                                'Please select an option below',
                            onInitialize = I(
                                'function() { this.setValue(""); }')
                        )),
                    selectizeInput('normalized_covariate',
                        'Choose the covariates you would like to include for voom design)',
                        multiple = FALSE,
                        choices = c(''),
                        selected = NULL,
                        options = list(placeholder =
                                'Please select an option below',
                            onInitialize = I(
                                'function() { this.setValue(""); }')
                        )),
                    textInput(inputId = 'normalized_assay_name', 'Name for the normalized assay',
                        value = ''),
                    checkboxInput('log', 'log(x+1) transform the results'),
                    withBusyIndicatorUI(actionButton(inputId = 'normalize',
                        label = 'Normalize')),
                    br()
                )
            )
        )
    )
)

tabPanel("Data Download",
         # Application title
         titlePanel("Data Download"),
mainPanel('Data Download',
         h5("Download Summarized Experiment"),
         verbatimTextOutput('se_download'),
         downloadButton("downloadData", "Download"),
         br(),
         h4(strong("Usage")),
         h5("The SummarizedExperiment object containing your data may be previewed and downloaded here. The SummarizedExperiment object includes your uploaded data and metadata as well as any normalized and batch corrected versions of your data that were generated while using BatchQC. Please select the Download button to download the SummarizedExperiment object.")
  )
)

## ----eval = FALSE-------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("BatchQC")

## ----eval = FALSE-------------------------------------------------------------
# if (!require("devtools", quietly = TRUE)) {
#     install.packages("devtools")
# }
# 
# library(devtools)
# install_github("wejlab/BatchQC")

## ----load package-------------------------------------------------------------
library(BatchQC)

## ----Launch Shiny App, eval = FALSE, echo = TRUE------------------------------
# BatchQC()

## ----Create Sumarized Experiment Object---------------------------------------
# Load package data examples
data(signature_data)
data(batch_indicator)

# Create SE Object
features <- signature_data
metadata <- batch_indicator
se_object <- BatchQC::summarized_experiment(features, metadata)

# Name your assay
SummarizedExperiment::assayNames(se_object) <- "log_intensity"

# Ensure all relevat metadata varaibles are factors
se_object$batch <- as.factor(se_object$batch)
se_object$condition <- as.factor(se_object$condition)

## ----Load TB example data-----------------------------------------------------
se_object <- tb_data_upload()

## ----TB Variables-------------------------------------------------------------
assay_of_interest <- "features"
batch <- "Experiment"
covar <- "TBStatus"

## ----AIC Model Fit------------------------------------------------------------
TB_AIC <- compute_aic(se = se_object,
    assay_of_interest = assay_of_interest,
    batchind = batch,
    groupind = covar,
    maxit = 1000,
    zero_filt_percent = 15
    )
TB_AIC["AIC_table"]
TB_AIC["AIC_boxplot"]

## ----Neg Binomial Model Fit TB Data-------------------------------------------
# Set a seed if you would like reproducible results
set.seed(1)

TB_fit <- goodness_of_fit_nb(se = se_object,
    count_matrix = assay_of_interest,
    condition = covar,
    other_variables = batch,
    num_genes = 3000,
    method = "edgeR")

TB_fit$res_histogram
TB_fit$recommendation

## -----------------------------------------------------------------------------
# CPM Normalization
se_object <- BatchQC::normalize_SE(se = se_object, method = "CPM",
    log_bool = FALSE, assay_to_normalize = assay_of_interest,
    output_assay_name = "CPM_normalized_counts")

# CPM Normalization w/log
se_object <- BatchQC::normalize_SE(se = se_object, method = "CPM",
    log_bool = TRUE, assay_to_normalize = assay_of_interest,
    output_assay_name = "CPM_log_normalized_counts")

# DESeq Normalization
se_object <- BatchQC::normalize_SE(se = se_object, method = "DESeq",
    log_bool = FALSE, assay_to_normalize = assay_of_interest,
    output_assay_name = "DESeq_normalized_counts")

# DESeq Normalization w/log
se_object <- BatchQC::normalize_SE(se = se_object, method = "DESeq",
    log_bool = TRUE, assay_to_normalize = assay_of_interest,
    output_assay_name = "DESeq_log_normalized_counts")

# log adjust
se_object <- BatchQC::normalize_SE(se = se_object, method = "none",
    log_bool = TRUE, assay_to_normalize = assay_of_interest,
    output_assay_name = "log_normalized_counts")

## -----------------------------------------------------------------------------
# ComBat correction
se_object <- BatchQC::batch_correct(se = se_object, method = "ComBat",
    assay_to_normalize = "CPM_log_normalized_counts", batch = batch,
    covar = c(covar), output_assay_name = "ComBat_corrected")

## -----------------------------------------------------------------------------
batch_design_tibble <- BatchQC::batch_design(se = se_object, batch = batch, 
    covariate = covar)
batch_design_tibble

## -----------------------------------------------------------------------------
confound_table <- BatchQC::confound_metrics(se = se_object, batch = batch)
confound_table

## -----------------------------------------------------------------------------
pearson_cor_result <- BatchQC::std_pearson_corr_coef(batch_design_tibble)
pearson_cor_result

## -----------------------------------------------------------------------------
cramers_v_result <- BatchQC::cramers_v(batch_design_tibble)
cramers_v_result

## ----lambda Statistic---------------------------------------------------------
lambda_calculation <- run_lambda(se = se_object,
    assay = assay_of_interest,
    batch = batch,
    condition = covar)

lambda_calculation$correction_recommendation
lambda_calculation$lambda_stat

## -----------------------------------------------------------------------------
ex_variation <- batchqc_explained_variation(se = se_object,
    batch = batch, condition = covar, assay_name = assay_of_interest)
EV_boxplot <- BatchQC::EV_plotter(batchqc_ev = ex_variation[[1]])
EV_boxplot
BatchQC::summary_stats_EV_table(ex_variation[[1]])

## -----------------------------------------------------------------------------
EV_boxplot_type_2 <- BatchQC::EV_plotter(batchqc_ev = ex_variation[[2]])
EV_boxplot_type_2
BatchQC::summary_stats_EV_table(ex_variation[[2]])

## -----------------------------------------------------------------------------
ex_variation <- batchqc_explained_variation(se = se_object,
    batch = batch, condition = covar, assay_name = "ComBat_corrected")
EV_boxplot <- BatchQC::EV_plotter(batchqc_ev = ex_variation[[1]])
EV_boxplot
BatchQC::summary_stats_EV_table(ex_variation[[1]])
EV_boxplot_type_2 <- BatchQC::EV_plotter(batchqc_ev = ex_variation[[2]])
EV_boxplot_type_2
BatchQC::summary_stats_EV_table(ex_variation[[2]])
EV_table <- BatchQC::EV_table(batchqc_ev = ex_variation[[1]])
EV_table
EV_table_type_2 <- BatchQC::EV_table(batchqc_ev = ex_variation[[2]])
EV_table_type_2

## ----kBET---------------------------------------------------------------------
TB_kBET <- BatchQC::run_kBET(se = se_object,
    assay_to_normalize = assay_of_interest,
    batch = batch)

BatchQC::plot_kBET(TB_kBET)

## -----------------------------------------------------------------------------
heatmaps <- BatchQC::heatmap_plotter(se = se_object, assay = assay_of_interest,
    nfeature = 38, annotation_column = c(batch, covar),
    log_option = "FALSE")
correlation_heatmap <- heatmaps$correlation_heatmap
correlation_heatmap

## -----------------------------------------------------------------------------
heatmaps <- BatchQC::heatmap_plotter(se = se_object, assay = assay_of_interest,
    nfeature = 38, annotation_column = c(batch, covar),
    log_option = FALSE)
heatmap <- heatmaps$topn_heatmap
heatmap

## -----------------------------------------------------------------------------
dendrogram_plot <- BatchQC::dendrogram_plotter(se = se_object,
    assay = assay_of_interest,
    batch_var = batch,
    category_var = covar)
dendrogram_plot$dendrogram

## -----------------------------------------------------------------------------
circular_dendrogram_plot <- BatchQC::dendrogram_plotter(
    se = se_object, assay = assay_of_interest, batch_var = batch,
    category_var = covar)
circular_dendrogram_plot$circular_dendrogram

## -----------------------------------------------------------------------------
pca_plot <- BatchQC::PCA_plotter(se = se_object, nfeature = 20, color = batch,
    shape = covar, batch = batch,
    assays = c("log_normalized_counts", "ComBat_corrected"),
    xaxisPC = 1, yaxisPC = 2, log_option = FALSE)
pca_plot$plot
pca_plot$var_explained

## ----UMAP---------------------------------------------------------------------
umap_plot <- BatchQC::umap(se = se_object,
    assay_of_interest = assay_of_interest, batch = batch, covar = covar,
    neighbors = 15, min_distance = 0.1, spread = 1, log_option = TRUE)

umap_plot

## ----UMAP batch corrected-----------------------------------------------------
umap_plot_batch_corrected <- BatchQC::umap(se = se_object,
    assay_of_interest = "ComBat_corrected", batch = batch, covar = covar,
    neighbors = 15, min_distance = 0.1, spread = 1)

umap_plot_batch_corrected

## -----------------------------------------------------------------------------
differential_expression <- BatchQC::DE_analyze(se = se_object,
    method = "limma", batch = batch, conditions = c(covar),
    assay_to_analyze = assay_of_interest, padj_method = "BH")

## -----------------------------------------------------------------------------
names(differential_expression)

## -----------------------------------------------------------------------------
head(differential_expression[["TBStatusPTB"]])

## -----------------------------------------------------------------------------
pval_plotter(differential_expression)
head(pval_summary(differential_expression))

## -----------------------------------------------------------------------------
value <- round((max(abs(
    differential_expression[[length(differential_expression)]][, 1]))
    + min(
        abs(differential_expression[[length(differential_expression)]][, 1])))
    / 2)
volcano_plot(DE_results = differential_expression[["TBStatusPTB"]],
    pslider = 0.05,
    fcslider = value)

## ----eval = FALSE, echo = TRUE------------------------------------------------
# file_location <- "location/to/save/object"
# 
# saveRDS(object = se_object, file = file_location)

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

#' Volcano plot
#'
#' This function allows you to plot DE analysis results as a volcano plot
#' @param DE_results a dataframe with the results of one of the DE Analysis;
#' must include "log2FoldChange" and "pvalue" columns
#' @param pslider Magnitude of significance value threshold, default is 0.05
#' @param fcslider Magnitude of expression change value threshold
#' @return A volcano plot of expression change and significance value data
#' @import ggplot2
#' @import scran
#'
#' @examples
#' library(scran)
#' se <- mockSCE()
#' differential_expression <- BatchQC::DE_analyze(se = se,
#'                                                 method = "DESeq2",
#'                                                 batch = "Treatment",
#'                                                 conditions = c(
#'                                                 "Mutation_Status",
#'                                                 "Cell_Cycle"),
#'                                                 assay_to_analyze = "counts",
#'                                                 padj_method = "BH")
#' value <- round((max(abs(
#'     differential_expression[[length(differential_expression)]][, 1]))
#'     + min(abs(
#'     differential_expression[[length(differential_expression)]][, 1]))) / 2)
#'
#' volcano_plot(differential_expression[[1]], pslider = 0.05, fcslider = value)
#' @export
volcano_plot <- function(DE_results, pslider = 0.05, fcslider) {
    DE_results <- as.data.frame(DE_results) %>%
        select("log2FoldChange", "pvalue")
    DE_results$conditionName <- row.names(DE_results)
    pslider_factor <- pslider

    pslider_cond <- case_when(DE_results[, 2] < pslider_factor ~ "TRUE",
        DE_results[, 2] >= pslider_factor ~ "FALSE",
        TRUE ~ 'NA')
    fcslider_factor <- fcslider
    fcslider_cond <- case_when(abs(DE_results[, 1]) <
            fcslider_factor ~ "FALSE",
        abs(DE_results[, 1]) >=
            fcslider_factor ~ "TRUE",
        TRUE ~ 'NA')
    filters <- cbind(pslider_cond, fcslider_cond)
    cond <- apply(filters, 1, function(x)(length(which(x == TRUE)) == 2))
    Features <- NULL
    DE_results <- DE_results %>% mutate(Features = cond)

    log2fc <- round(DE_results[, 1], digits = 2)
    pval <- round(-log10(DE_results[, 2]), digits = 2)
    feature <- DE_results[, 3]

    p <- ggplot2::ggplot(data = DE_results,
        aes(x = log2fc, y = pval, text = feature, color = Features)) +
        geom_point() +
        scale_color_manual(values = c('FALSE' = 'blue',
            'TRUE' = 'red',
            'NA' = 'black'),
            labels = c('Threshold failed',
                'All Thresholds passed',
                'NA')) +
        xlab("Change in Expression (log2 fold change)") +
        ylab("Signifigance Value (-log10 p-value)") +
        theme(legend.position = "bottom")

    vol_plot <- p +
        geom_hline(yintercept = -log10(pslider_factor), linetype = "dashed") +
        geom_vline(xintercept = c(-fcslider_factor, fcslider_factor),
            linetype = "dashed")

    vol_plot <- plotly::ggplotly(vol_plot, tooltip = c('x', 'y', 'text'))

    return(vol_plot)
}

accepted <- c("text/csv",
    "text/comma-separated-values",
    "text/plain",
    ".csv")


tabPanel("Upload Data",
    useShinyjs(),
    tags$style(appCSS),
    tags$div(
        class = "jumbotron",
        tags$div(
            class = "container",
            fluidRow(column(7, h1("BatchQC"))),
            tags$p("Batch Effects Quality Control Software"),
            uiOutput("tab")

        )
    ),
    # Application title
    titlePanel("Upload Data"),

    # Place for uploading data
    sidebarLayout(
        sidebarPanel(
            h4("Select the type of input you would like to provide:"),
            radioButtons("uploadChoice", "",
                c("Feature File and Metadata File" = "countFile",
                    "Summarized Experiment Object" = "seObject",
                    "Example Data" = "example"
                )),
            #Only show panel if uploading count and metadata files
            conditionalPanel(condition = "input.uploadChoice == 'countFile'",
                h4("Upload feature and metadata table"),
                tags$div(tags$p(
                    'Metadata file must be a table with headers and sample names.'
                )),
                fileInput(
                    "counts",
                    "Feature table",
                    multiple = FALSE,
                    accept = accepted
                ),
                fileInput("md", "Metadata",
                    multiple = FALSE,
                    accept = accepted)),
            conditionalPanel(condition = "input.uploadChoice == 'seObject'",
                fileInput(
                    "se",
                    "Summarized Experiment",
                    multiple = FALSE,
                    accept = c(accepted, ".RDS")
                )),
            conditionalPanel(condition = "input.uploadChoice == 'example'",
                selectInput("exampleData",
                    "Example Data",
                    choices = c("", "proteinData", "signatureData",
                        "bladderData", "TBData", "No Selection"),
                    selected = ""), ),
            withBusyIndicatorUI(actionButton(inputId = 'submit',
                label = 'Upload'))
        ),

        # Show a table of the inputted data
        mainPanel(
            tabsetPanel(
                tabPanel('Input Summary',
                    h4(strong("Usage")),
                    h5("The table displayed here is a preview of the data selected to be uploaded to BatchQC. Please take a moment to verify that this is the intended data to upload, then select the Upload button to complete uploading the data to BatchQC."),
                    conditionalPanel(condition = "input.uploadChoice == 'example'",
                        h4(strong("Example Data Description")),
                        conditionalPanel(condition = "input.exampleData == 'proteinData'",
                            h5("This data set is mass-spectromery data for protein expression in an unpublished data set.")
                        ),
                        conditionalPanel(condition = "input.exampleData == 'signatureData'",
                            h5("This data set is RNA-seq data from signature data captured when activating different growth pathway genes in human mammary epithelial cells (GEO accession: GSE73628)."),
                            h6("Reference: Rahman M, et al. Activity of distinct growth factor receptor network components in breast tumors uncovers two biologically relevant subtypes. Genome Med. 2017 Apr 26;9(1):40. doi: 10.1186/s13073-017-0429-x.")
                        ),
                        conditionalPanel(condition = "input.exampleData == 'bladderData'",
                            h5("This data set is from bladder cancer data and contains Affymetrix gene expression microarray data. This data set is from the
                                    bladderbatch package which must be installed to use this data example set"),
                            h6("Reference: Leek J., et al. Tackling the widespread and critical impact of batch effects in high-throughput data. Nat Rev Genet 11, 733–739 (2010). https://doi.org/10.1038/nrg2825")
                        ),
                        conditionalPanel(condition = "input.exampleData == 'TBData'",
                            h5("This data set is from a TB cohort study from Pondicherry India and contains RNA-seq counts data."),
                            h6("Reference: Johnson WE, et al. Comparing tuberculosis gene signatures in malnourished individuals using the TBSignatureProfiler. BMC Infect Dis. 2021 Jan 22;21(1):106. doi: 10.1186/s12879-020-05598-z. PMID: 33482742; PMCID: PMC7821401.")
                        )),
                    h4(strong("Data Preview")),
                    DTOutput('counts_header'),
                    textOutput('counts_dimensions'),
                    DTOutput('se_counts'),
                    textOutput('se_dimensions'),
                    br()
                ),
                tabPanel('Full Metadata',
                    h4(strong("Usage")),
                    h5("The table displayed here shows all metadata associated with the uploaded data. Please refer to this table to view sample condition information."),
                    h4(strong("Metadata Table")),
                    DTOutput('metadata_header'),
                    DTOutput('se_meta'),
                    br()
                )
            )
        )
    )
)

tabPanel("Differential Expression Analysis",
         # Application title
         titlePanel("Differential Expression Analysis"),
         sidebarLayout(
           sidebarPanel(
            selectizeInput('DE_method',
                   'Choose analysis method',
                   multiple = FALSE,
                   choices = c('DESeq2', 'limma', 'edgeR', 'ANOVA', 'Kruskal-Wallis'), # 'wilcox'),
                   selected = NULL,
                   options = list(placeholder = 'Please select an option below',
                       onInitialize = I('function() { this.setValue(""); }'))),
            selectizeInput('DE_assay',
                 'Choose assay to analyze',
                 choices = "",
                 options = list(placeholder = 'Please select an option below',
                    onInitialize = I('function() { this.setValue(""); }'))),
            selectizeInput('DE_batch',
                 'Choose batch covariate',
                 choices = "",
                 options = list(placeholder = 'Please select an option below',
                    onInitialize = I('function() { this.setValue(""); }'))),
            selectizeInput('DE_conditions',
                 'Choose analysis covariate(s)',
                 choices = c(),
                 multiple = TRUE,
                 selected = NULL,
                 options = list(placeholder = 'Please select an option below',
                     onInitialize = I('function() { this.setValue(""); }'))),
            selectizeInput('padj_method',
                'Choose p.adjust method',
                choices = p.adjust.methods[p.adjust.methods != "none"],
                options = list(placeholder = 'Please select an option below',
                    onInitialize = I('function() { this.setValue(""); }'))),
            withBusyIndicatorUI(actionButton('DE_analyze', label = 'Here we go!'))),
           mainPanel(
             h4(strong("Usage")),
             h5("Each differential expression method requires a specific type of input. Please provide raw counts data or DESeq-normalized data for DESeq, normalized log or log-CPM data for limma, edgeR-normalized data for edgeR, and normalized data for ANOVA."),
             tabsetPanel(
               tabPanel("Results Table",
                       selectizeInput("result_to_view",
                           'Choose analysis results to display',
                           choices = "",
                           options = list(placeholder = 'Please select an option below',
                               onInitialize = I('function() { this.setValue(""); }'))),
                        DTOutput('DE_results'),
                        conditionalPanel(condition = "output.DE_results",
                                         downloadButton("downloadDEData",
                                             "Download"))),
               tabPanel("P-Value Analysis",
                        plotOutput('pval_plot'),
                        h5("P-Value Summary Table"),
                        DTOutput('pval_summary')),
               tabPanel("Volcano Plot",
                   selectizeInput('DE_res_selected',
                       'Choose analysis results to display',
                       choices = "",
                       options = list(placeholder = 'Please select an option below',
                           onInitialize = I('function() { this.setValue(""); }'))),
                   sliderInput(inputId = "pslider",
                       label = "Select the magnitude of significance value coloring:",
                       min = 0,
                       max = 1,
                       value = 0.05,
                       round = FALSE,
                       ticks = TRUE,
                       step = 0.001),
                   sliderInput(inputId = "fcslider",
                       label = "Select the magnitude of expression change value coloring:",
                       min = -10,
                       max = 10,
                       value = 0,
                       step = NULL,
                       round = FALSE,
                       ticks = TRUE),
                    actionButton("volcano_plot", "Run", class = "btn-primary"),
                    plotlyOutput('volcano', height = 'auto', width = 'auto')
               )
             )
           )
         )
)

### DIFFERENTIAL EXPRESSION ANALYSIS TAB ###
observeEvent(input$DE_batch, {
    req(reactivevalue$se, input$DE_batch)
    DE_covariate_choices <- covariates_not_confounded(reactivevalue$se,
        input$DE_batch)
    updateSelectizeInput(session = session, inputId = "DE_conditions",
        choices = DE_covariate_choices, selected = NULL)
})

observeEvent(input$DE_method, {
    req(reactivevalue$se, input$DE_batch)
    DE_covariate_choices <- covariates_not_confounded(reactivevalue$se,
        input$DE_batch)
    updateSelectizeInput(session = session, inputId = "DE_conditions",
        choices = DE_covariate_choices, selected = NULL)
})
observeEvent(input$DE_analyze, {
    req(reactivevalue$se, input$DE_analyze)

    # withProgress({
    #     setProgress(0.5, 'Calculating...')
    #     reactivevalue$DE_results <- DE_analyze(reactivevalue$se,
    #         input$DE_method, input$DE_batch, input$DE_conditions,
    #         input$DE_assay)
    #     setProgress(1, 'Complete!')
    # })

    withBusyIndicatorServer("DE_analyze", {
        reactivevalue$DE_results <- DE_analyze(reactivevalue$se,
                     input$DE_method, input$DE_batch, input$DE_conditions,
                     input$DE_assay, input$padj_method)
        display_covariate <- names(reactivevalue$DE_results)[length(reactivevalue$DE_results)]

        output$DE_results <- renderDT({
            reactivevalue$DE_results[[length(reactivevalue$DE_results)]]
        })

        output$pval_summary <- renderDT({
            pval_summary(reactivevalue$DE_results)})

        output$pval_plot <- renderPlot({
            pval_plotter(reactivevalue$DE_results)
        })

        output$downloadDEData <- downloadHandler(
            filename = function() {
                paste("DE_results", ".csv", sep = "")
            },
            content = function(file) {
                write.csv(reactivevalue$DE_results[[length(reactivevalue$DE_results)]], file)
            }
        )

        updateSelectizeInput(session = session,
            inputId = "DE_res_selected",
            choices = names(reactivevalue$DE_results),
            selected = names(reactivevalue$DE_results)[length(
                reactivevalue$DE_results)])
        updateSelectizeInput(session = session,
            inputId = "result_to_view",
            choices = names(reactivevalue$DE_results),
            selected = names(reactivevalue$DE_results)[length(
                reactivevalue$DE_results)])
        updateSliderInput(session = session,
            inputId = "fcslider",
            min = round(min(abs(reactivevalue$DE_results[[length(reactivevalue$DE_results)]][, 1]))),
            max = round(max(abs(reactivevalue$DE_results[[length(reactivevalue$DE_results)]][, 1]))),
            value = round((max(abs(reactivevalue$DE_results[[length(reactivevalue$DE_results)]][, 1]))
                + min(abs(reactivevalue$DE_results[[length(reactivevalue$DE_results)]][, 1]))) / 2))
    })

})

plotVolcanoPlotButton <- eventReactive(input$volcano_plot, {
    volcano_plot(reactivevalue$DE_results[[as.character(input$DE_res_selected)]], input$pslider, input$fcslider)
})

output$volcano <- renderPlotly({
    plotVolcanoPlotButton()
})

observeEvent(input$DE_res_selected, {
    if (input$DE_res_selected != "") {
        updateSliderInput(session = session,
            inputId = "fcslider",
            min = 0,
            max = round(max(abs(reactivevalue$DE_results[[input$DE_res_selected]][, 1]))),
            value = round(median(abs(reactivevalue$DE_results[[input$DE_res_selected]][, 1]))))

    }
    })

observeEvent(input$result_to_view, {
    output$DE_results <- renderDT({
        reactivevalue$DE_results[[input$result_to_view]]
    })

    output$downloadDEData <- downloadHandler(
        filename = function() {
            paste(input$DE_method, "_results_", input$result_to_view, ".csv", sep = "")
        },
        content = function(file) {
            write.csv(reactivevalue$DE_results[[input$result_to_view]], file)
        }
    )
})

#' @importFrom stats pchisq pnorm
# a wrapper for kBET to fix a neighbourhood size
scan_nb <- function(x, df, batch, knn) {
    res <- kBET(
        df = df, batch = batch, k0 = x, knn = knn, testSize = NULL,
        heuristic = FALSE, n_repeat = 10, alpha = 0.05,
        addTest = FALSE, plot = FALSE, verbose = FALSE, adapt = FALSE
    )
    result <- res$summary
    result$kBET.observed[1]
}

# the residual score function of kBET
residual_score_batch <- function(knn.set, class.freq, batch) {
    # knn.set: indices of nearest neighbours
    # empirical frequencies in nn-environment (sample 1)
    # ignore NA entries (which may arise from subsampling a knn-graph)
    if (all(is.na(knn.set))) { # if all values of a neighbourhood are NA
        return(NA)
    } else {
        valid.knn <- knn.set[!is.na(knn.set)]
        freq.env <- table(batch[valid.knn]) / length(valid.knn)
        full.classes <- rep(0, length(class.freq$class))
        full.classes[class.freq$class %in% names(freq.env)] <- freq.env
        exp.freqs <- class.freq$freq
        # compute chi-square test statistics
        sum((full.classes - exp.freqs)^2 / exp.freqs)
    }
}

# which batch has the largest deviance (and is underrepresented)
max_deviance_batch <- function(knn.set, class.freq, batch) {
    # knn.set: indices of nearest neighbours
    # empirical frequencies in nn-environment (sample 1)
    if (all(is.na(knn.set))) { # if all values of a neighbourhood are NA
        return(NA)
    } else {
        valid.knn <- knn.set[!is.na(knn.set)]
        freq.env <- table(batch[valid.knn]) / length(valid.knn)
        full.classes <- rep(0, length(class.freq$class))
        full.classes[class.freq$class %in% names(freq.env)] <- freq.env
        exp.freqs <- class.freq$freq
        # compute chi-square test statistics
        allScores <- (full.classes - exp.freqs) / exp.freqs
        batch[which(allScores == min(allScores))]
    }
}


# the core function of kBET
chi_batch_test <- function(knn.set, class.freq, batch, df) {
    # knn.set: indices of nearest neighbours
    # empirical frequencies in nn-environment (sample 1)
    if (all(is.na(knn.set))) { # if all values of a neighbourhood are NA
        return(NA)
    } else {
        freq.env <- table(batch[knn.set[!is.na(knn.set)]])
        full.classes <- rep(0, length(class.freq$class))
        full.classes[class.freq$class %in% names(freq.env)] <- freq.env
        exp.freqs <- class.freq$freq * length(knn.set)
        # compute chi-square test statistics
        chi.sq.value <- sum((full.classes - exp.freqs)^2 / exp.freqs)
        result <- 1 - pchisq(chi.sq.value, df) # p-value for the result
        if (is.na(result)) { # I actually would like to now when 'NA' arises.
            return(NA)
        } else {
            result
        }
    }
}

lrt_approximation <- function(knn.set, class.freq, batch, df) {
    # knn.set: indices of nearest neighbours
    # empirical frequencies in nn-environment (sample 1)
    if (all(is.na(knn.set))) { # if all values of a neighbourhood are NA
        return(NA)
    } else {
        # observed realisations of each category
        obs.env <- table(batch[knn.set[!is.na(knn.set)]])
        # observed 'probabilities'
        freq.env <- obs.env / sum(obs.env)
        full.classes <- rep(0, length(class.freq$class))
        obs.classes <- class.freq$class %in% names(freq.env)
        # for stability issues (to avoid the secret division by 0): introduce
        # another alternative model where the observed probability
        # is either the empirical frequency or 1/(sample size) at minimum
        if (length(full.classes) > sum(obs.classes)) {
            dummy.count <- length(full.classes) - sum(obs.classes)
            full.classes[obs.classes] <- obs.env / (sum(obs.env) + dummy.count)
            pmin <- 1 / (sum(obs.env) + dummy.count)
            full.classes[!obs.classes] <- pmin
        } else {
            full.classes[obs.classes] <- freq.env
        }
        exp.freqs <- class.freq$freq # expected 'probabilities'
        # compute likelihood ratio of null and alternative hypothesis,
        # test statistics converges to chi-square distribution
        full.obs <- rep(0, length(class.freq$class))
        full.obs[obs.classes] <- obs.env

        lrt.value <- -2 * sum(full.obs * log(exp.freqs / full.classes))

        result <- 1 - pchisq(lrt.value, df) # p-value for the result
        if (is.na(result)) { # I actually would like to now when 'NA' arises.
            return(NA)
        } else {
            result
        }
    }
}

# truncated normal distribution distribution function
ptnorm <- function(x, mu, sd, a = 0, b = 1, alpha = 0.05, verbose = FALSE) {
    # this is the cumulative density of the truncated normal distribution
    # x ~ N(mu, sd^2), but we condition on a <= x <= b
    if (!is.na(x)) {
        if (a > b) {
            warning("Lower and upper bound are interchanged.")
            tmp <- a
            a <- b
            b <- tmp
        }

        if (sd <= 0 || is.na(sd)) {
            if (verbose) {
                warning("Standard deviation must be positive.")
            }
            if (alpha <= 0) {
                stop("False positive rate alpha must be positive.")
            }
            sd <- alpha
        }
        if (x < a || x > b) {
            warning("x out of bounds.")
            cdf <- as.numeric(x > a)
        } else {
            alp <- pnorm((a - mu) / sd)
            bet <- pnorm((b - mu) / sd)
            zet <- pnorm((x - mu) / sd)
            cdf <- (zet - alp) / (bet - alp)
        }
        cdf
    } else {
        return(NA)
    }
}

# from EMT package (function is needed to run ExactMultinomialTest.R)
findVectors <- function(groups, size) {
    if (groups == 1) {
        mat <- size
    } else {
        mat <- matrix(rep(0, groups - 1), nrow = 1)
        for (i in seq_len(size)) {
            mat <- rbind(mat, findVectors(groups - 1, i))
        }
        mat <- cbind(mat, size - rowSums(mat))
    }
    invisible(mat)
}

# ExactMultinomialTest
# @description from EMT package (solemnly adapted to stay quiet)
#
# @param observed
# @param prob
# @param size
# @param groups
# @param numEvents
# @param verbose
#' @importFrom stats dmultinom
ExactMultinomialTest <- function(
    observed, prob, size,
    groups, numEvents, verbose) {
    pObs <- stats::dmultinom(observed, size = size, prob)
    eventMat <- findVectors(groups, size)
    if (nrow(eventMat) != numEvents) {
        stop("Wrong number of events calculated. \n This is probably a bug.")
    }
    eventProb <- apply(eventMat, 1, function(x) {
        dmultinom(x,
            size = size, prob = prob
        )
    })
    p.value <- sum(eventProb[eventProb <= pObs])
    if (round(sum(eventProb), digits = 2) != 1) {
        stop("Wrong values for probabilities. \n This is probably a bug.")
    }


    if (verbose) {
        head <- paste("\n Exact Multinomial Test, distance measure: p\n\n")
        tab <- as.data.frame(cbind(
            numEvents, round(pObs, digits = 4),
            round(p.value, digits = 4)
        ))
        colnames(tab) <- c("   Events", "   pObs", "   p.value")
        warning(head)
        warning(tab, row.names = FALSE)
    }

    invisible(list(
        id = "Exact Multinomial Test", size = size,
        groups = groups, stat = "lowP", allProb = sort(eventProb,
            decreasing = TRUE
        ), ntrial = NULL, p.value = round(p.value,
            digits = 4
        )
    ))
}

# from EMT package (function needed to run ExactMultinomialTest)

multinomial.test <- function(
    observed, prob, useChisq = FALSE,
    MonteCarlo = FALSE, ntrial = 1e+05, atOnce = 1e+06, verbose = FALSE) {
    if (!is.vector(observed, mode = "numeric")) {
        stop(
            " Observations have to be stored in a vector, ",
            "e.g.  'observed <- c(5,2,1)'"
        )
    }
    if (!is.vector(prob, mode = "numeric")) {
        stop(
            " Probabilities have to be stored in a vector, ",
            "e.g.  'prob <- c(0.25, 0.5, 0.25)'"
        )
    }
    if (round(sum(prob), digits = 1) != 1) {
        stop("Wrong input: sum of probabilities must not deviate from 1.")
    }
    if (length(observed) != length(prob)) {
        stop(" Observations and probabilities must have same dimensions.")
    }
    size <- sum(observed)
    groups <- length(observed)
    numEvents <- choose(size + groups - 1, groups - 1)
    res <- ExactMultinomialTest(
        observed, prob, size,
        groups, numEvents, verbose
    )

    invisible(res)
}

# wrapper for the multinomial exact test function
multiNom <- function(x, y, z) {
    z.f <- factor(z)
    tmp <- multinomial.test(as.numeric(table(z.f[x])), y)
    tmp$p.value
}

# significance test for pcRegression (two levels)
correlate.fun_two <- function(rot.data, batch, batch.levels) {
    # rot.data: some vector (numeric entries)
    # batch: some vector (categoric entries)
    a <- stats::lm(rot.data ~ batch)
    result <- numeric(2)
    result[1] <- summary(a)$r.squared # coefficient of determination
    result[2] <- summary(a)$coefficients[2, 4] # p-value (significance level)
    t.test.result <- t.test(rot.data[batch == batch.levels[1]],
        rot.data[batch == batch.levels[2]],
        paired = FALSE
    )
    result[3] <- t.test.result$p.value
    result
}

# significance test for pcRegression (more than two levels)
correlate.fun_gen <- function(rot.data, batch) {
    # rot.data: some vector (numeric covariate)
    # batch: some vector (categoric covariate)
    a <- stats::lm(rot.data ~ batch)
    result <- numeric(2)
    # coefficient of determination
    result[1] <- summary(a)$r.squared
    F.test.result <- aov(rot.data ~ batch)
    F.test.summary <- summary(F.test.result)

    # p-value (significance level)
    result[2] <- summary(a)$coefficients[2, 4]
    # p-value of the one-way anova test
    result[3] <- F.test.summary[[1]]$"Pr(>F)"[1]

    result
}

initialize.kbet <- function(
    df, batch, k0, knn, testSize, do.pca, dim.pca, heuristic, n_repeat,
    alpha = 0.05, addTest, verbose, adapt) {
    dof <- length(unique(batch)) - 1 # degrees of freedom
    if (is.factor(batch)) batch <- droplevels(batch)
    frequencies <- table(batch) / length(batch)
    batch.shuff <- replicate(3, batch[sample.int(length(batch))])
    class.frequency <- data.frame(
        class = names(frequencies), freq = as.numeric(frequencies))
    inputs <- validate.inputs(df, batch, verbose)
    if (!inputs$valid) return(NA)
    stopifnot(is(n_repeat, "numeric"), n_repeat > 0)
    k0_info <- determine.k0(k0, heuristic, class.frequency, knn,
        inputs$dim.dataset, verbose)
    if (is.null(k0_info$k0)) return(NA)
    k0 <- k0_info$k0
    do_heuristic <- k0_info$do_heuristic
    knn <- knn %||% find.knn(inputs$dataset, do.pca, k0, verbose, dim.pca,
        inputs$dim.dataset) # find KNNs
    knn <- knn.graph.backward.compatibility(knn, verbose)
    testSize <- set.number.tests(testSize, inputs$dim.dataset, verbose)
    if (adapt) {  # decide to adapt general frequencies
        adapt.freq.res <- adapt.freq(inputs$dim.dataset, knn, k0, inputs$batch,
            dof, class.frequency, alpha, verbose)
    }
    is.imbalanced <- if (adapt) adapt.freq.res$is.imbalanced else NULL
    new.class.frequency <- if (adapt) adapt.freq.res$new.class.freq else NULL
    outsider <- if (adapt) adapt.freq.res$outsider else NULL
    p.out <- if (adapt) adapt.freq.res$p.out else NULL
    if (do_heuristic) {
        k0 <- calculate_nb_size(scan_nb, k0, inputs$dataset,
            inputs$batch, knn, verbose)
    }
    rejection <- initialize.result.list(inputs$dim.dataset)
    # get average residual score
    env <- as.vector(cbind(
        knn[, seq_len(k0 - 1)], seq_len(inputs$dim.dataset[1])))
    cf <- if (adapt && is.imbalanced) new.class.frequency else class.frequency
    score <- k0 * residual_score_batch(env, cf, batch)
    rejection$average.pval <- 1 - pchisq(score, dof)
    return(list(
        dim.dataset = inputs$dim.dataset, testSize = testSize, k0 = k0,
        is.imbalanced = is.imbalanced, batch = inputs$batch, knn = knn,
        new.class.frequency = new.class.frequency, outsider = outsider,
        class.frequency = class.frequency, batch.shuff = batch.shuff,
        kBET.expected = numeric(n_repeat), kBET.observed = numeric(n_repeat),
        kBET.signif = numeric(n_repeat), rejection = rejection, dof = dof,
        p.out = p.out))
}

validate.inputs <- function(df, batch, verbose) {
    dim.dataset <- dim(df)
    # check the feasibility of data input
    if (dim.dataset[1] != length(batch) && dim.dataset[2] != length(batch)) {
        msg <- paste(
            "Input matrix and batch information do not match.",
            "Execution halted.",
            sep = " "
        )
        stop(msg)
    }

    if (dim.dataset[2] == length(batch) && dim.dataset[1] != length(batch)) {
        if (verbose) {
            warning(
                "Input matrix has samples as columns. ",
                "kBET needs samples as rows. Transposing...\n"
            )
        }
        df <- t(df)
        dim.dataset <- dim(df)
    }

    # check if the dataset is too small per se
    if (dim.dataset[1] <= 10) {
        if (verbose) {
            warning(
                "Your dataset has less than 10 samples.",
                "Abort and return NA.\n"
            )
        }
        return(list(valid = FALSE))
    }

    list(dataset = df, batch = batch, dim.dataset = dim.dataset, valid = TRUE)
}

determine.k0 <- function(k0, heuristic, class.freq, knn, dim.dataset, verbose) {
    do_heuristic <- FALSE
    if (is.null(k0) || k0 >= dim.dataset[1]) {
        do_heuristic <- heuristic
        k0 <- floor(mean(class.freq$freq) * dim.dataset[1] *
            ifelse(heuristic, 0.75, 0.25))

        if (verbose) {
            msg <- paste0("Initial neighborhood size set to ", k0, ".\n")
            warning(msg)
        }
    }

    if (k0 < 10 & (heuristic | is.null(knn))) {
        if (verbose) {
            warning(
                "Your dataset has too few samples to run a heuristic.\n",
                "Return NA.\n",
                "Please assign k0 and set heuristic = FALSE."
            )
        }
        return(NA)
    }

    list(k0 = k0, do_heuristic = do_heuristic)
}

find.knn <- function(dataset, do.pca, k0, verbose, dim.pca, dim.dataset) {
    if (!do.pca) {
        if (verbose) {
            warning("finding knns...")
            tic <- proc.time()
        }
        # use the nearest neighbour index directly for further use in the
        # package
        knn <- get.knn(dataset, k = k0, algorithm = "cover_tree")$nn.index
    } else {
        dim.comp <- min(dim.pca, dim.dataset[2])
        if (verbose) {
            warning("reducing dimensions with svd first...\n")
        }
        data.pca <- svd(x = dataset, nu = dim.comp, nv = 0)
        if (verbose) {
            warning("finding knns...")
            tic <- proc.time()
        }
        knn <- get.knn(data.pca$u, k = k0, algorithm = "cover_tree")
    }
    if (verbose) {
        warning("done. Time:\n")
        warning(proc.time() - tic)
    }

    return(knn)
}

knn.graph.backward.compatibility <- function(knn, verbose) {
    if (is(knn, "list")) {
        knn <- knn$nn.index
        if (verbose) {
            msg <- "KNN input is a list, extracting nearest neighbour index.\n"
            warning(msg)
        }
    }
    return(knn)
}

set.number.tests <- function(testSize, dim.dataset, verbose) {
    if (is.null(testSize) ||
        (floor(testSize) < 1 || dim.dataset[1] < testSize)) {
        test.frac <- 0.1
        testSize <- ceiling(dim.dataset[1] * test.frac)
        if (testSize < 25 && dim.dataset[1] > 25) {
            testSize <- 25
        }
        if (verbose) {
            warning(
                "Number of kBET tests is set to ",
                testSize,
                ".\n",
                sep = ""
            )
        }
    }

    return(testSize)
}

initialize.result.list <- function(dim.dataset) {
    rejection <- list()
    rejection$summary <- data.frame(kBET.expected = numeric(4),
                                    kBET.observed = numeric(4),
                                    kBET.signif = numeric(4))

    rejection$results <- data.frame(tested = numeric(dim.dataset[1]),
                                    kBET.pvalue.test = rep(0, dim.dataset[1]),
                                    kBET.pvalue.null = rep(0, dim.dataset[1]))

    return(rejection)
}

adapt.freq <- function(
    dim.dataset, knn, k0, batch, dof, class.frequency, alpha, verbose) {
    outsider <- which(!(seq_len(dim.dataset[1]) %in% knn[, seq_len(k0 - 1)]))
    is.imbalanced <- FALSE # initialisation
    new.class.frequency <- NULL
    p.out <- 1
    if (length(outsider) > 0) {
        p.out <- chi_batch_test(outsider, class.frequency, batch, dof)
        if (!is.na(p.out)) {
            is.imbalanced <- p.out < alpha
            if (is.imbalanced) {
                new.frequencies <- table(batch[-outsider]) /
                    length(batch[-outsider])
                new.class.frequency <- data.frame(
                    class = names(new.frequencies),
                    freq = as.numeric(new.frequencies)
                )
                if (verbose) {
                    percent <- length(outsider) / length(batch)
                    outs_percent <- round(percent * 100, 3)
                    msg <- paste(
                        sprintf(
                            paste0(
                                "There are %s cells (%s%%) that do ",
                                "not appear in any neighbourhood."
                            ),
                            length(outsider), outs_percent
                        ),
                        paste0(
                            "The expected frequencies for each category ",
                            "have been adapted."
                        ),
                        "Cell indexes are saved to result list.",
                        "", sep = "\n"
                    )
                    warning(msg)
                }
            } else {
                if (verbose) warning("No outsiders found.")
            }
        } else {
            if (verbose) warning("No outsiders found.")
        }
    }
    return(list(
        is.imbalanced = is.imbalanced, outsider = outsider, p.out = p.out,
        new.class.freq = new.class.frequency
    ))
}

calculate_nb_size <- function(scan_nb, k0, dataset, batch, knn, verbose) {
    if (verbose) {
        warning("Determining optimal neighbourhood size ...")
    }
    opt.k <- bisect(scan_nb,
        bounds = c(10, k0), known = NULL, dataset, batch,
        knn
    )
    # result
    if (length(opt.k) > 1) {
        k0 <- opt.k[2]
        if (verbose) {
            msg <- paste0(
                "done.\nNew size of neighbourhood is set to ",
                k0, ".\n"
            )
            warning(msg)
        }
    } else {
        if (verbose) {
            warning(
                "done.",
                "Heuristic did not change the neighbourhood.",
                sprintf(
                    "If results appear inconclusive, change k0 = %s.",
                    k0
                ),
                "",
                sep = "\n"
            )
        }
    }
    return(k0)
}

run.kbet.addTest <- function(initialize.kbet.res, adapt, alpha, n_repeat) {

    # initialize result list
    kbet.addTest.res <- initialize.kbet.addTest(
        rejection = initialize.kbet.res$rejection,
        k0 = initialize.kbet.res$k0,
        dof = initialize.kbet.res$dof,
        batch = initialize.kbet.res$batch,
        dim.dataset = initialize.kbet.res$dim.dataset,
        n_repeat = n_repeat
        )

    # run kBet test (with addTest)
    kbet.addTest.res <- kbet.addTest(
        rejection = kbet.addTest.res$rejection,
        n_repeat = n_repeat,
        dim.dataset = initialize.kbet.res$dim.dataset,
        testSize = initialize.kbet.res$testSize,
        k0 = initialize.kbet.res$k0,
        knn = initialize.kbet.res$knn,
        is.imbalanced = initialize.kbet.res$is.imbalanced,
        class.frequency = initialize.kbet.res$class.frequency,
        new.class.frequency = initialize.kbet.res$new.class.frequency,
        dof = initialize.kbet.res$dof,
        batch = initialize.kbet.res$batch,
        batch.shuff = initialize.kbet.res$batch.shuff,
        kBET.expected = initialize.kbet.res$kBET.expected,
        kBET.observed = initialize.kbet.res$kBET.observed,
        kBET.signif = initialize.kbet.res$kBET.signif,
        lrt.expected = kbet.addTest.res$lrt.expected,
        lrt.observed = kbet.addTest.res$lrt.observed,
        lrt.signif = kbet.addTest.res$lrt.signif,
        exact.expected = kbet.addTest.res$exact.expected,
        exact.observed = kbet.addTest.res$exact.observed,
        exact.signif = kbet.addTest.res$exact.signif,
        adapt = adapt,
        alpha = alpha
    )

    return(list(
        rejection = kbet.addTest.res$rejection,
        kBET.expected = kbet.addTest.res$kBET.expected,
        kBET.observed = kbet.addTest.res$kBET.observed,
        kBET.signif = kbet.addTest.res$kBET.signif,
        lrt.expected = kbet.addTest.res$lrt.expected,
        lrt.observed = kbet.addTest.res$lrt.observed,
        lrt.signif = kbet.addTest.res$lrt.signif
    ))
}

initialize.kbet.addTest <- function(
    rejection, k0, dof, batch, dim.dataset, n_repeat) {

    # initialize result list
    rejection$summary$lrt.expected <- numeric(4)
    rejection$summary$lrt.observed <- numeric(4)

    rejection$results$lrt.pvalue.test <- rep(0, dim.dataset[1])
    rejection$results$lrt.pvalue.null <- rep(0, dim.dataset[1])

    lrt.expected <- numeric(n_repeat)
    lrt.observed <- numeric(n_repeat)
    lrt.signif <- numeric(n_repeat)

    # decide to perform exact test or not
    if (choose(k0 + dof, dof) < 5e5 && k0 <= min(table(batch))) {
        exact.expected <- numeric(n_repeat)
        exact.observed <- numeric(n_repeat)
        exact.signif <- numeric(n_repeat)

        rejection$summary$exact.expected <- numeric(4)
        rejection$summary$exact.observed <- numeric(4)
        rejection$results$exact.pvalue.test <- rep(0, dim.dataset[1])
        rejection$results$exact.pvalue.null <- rep(0, dim.dataset[1])
    }

    return(list(
        rejection = rejection, lrt.expected = lrt.expected,
        lrt.observed = lrt.observed, lrt.signif = lrt.signif,
        exact.expected = exact.expected, exact.observed = exact.observed,
        exact.signif = exact.signif
    ))
}

kbet.addTest <- function(
    rejection, n_repeat, dim.dataset, testSize, k0, knn, is.imbalanced,
    class.frequency, new.class.frequency, batch, dof, alpha, batch.shuff,
    kBET.expected, kBET.observed, kBET.signif, lrt.expected, lrt.observed,
    lrt.signif, exact.expected, exact.observed, exact.signif, adapt) {
    for (i in seq_len(n_repeat)) {
        idx.runs <- sample.int(dim.dataset[1], size = testSize)
        env <- cbind(knn[idx.runs, seq_len(k0 - 1)], idx.runs)
        pka.res <- perform.kbet.addTest(
            env, adapt, is.imbalanced, class.frequency, new.class.frequency,
            batch, dof, alpha, batch.shuff)
        kBET.expected[i] <- sum(pka.res$p.val.test.null < alpha, na.rm = TRUE) /
            sum(!is.na(pka.res$p.val.test.null))
        kBET.observed[i] <- sum(pka.res$is.rejected, na.rm = TRUE) /
            sum(!is.na(pka.res$p.val.test))
        kBET.signif[i] <- 1 - ptnorm(
            kBET.observed[i], mu = kBET.expected[i],
            sd = sqrt(kBET.expected[i] * (1 - kBET.expected[i]) / testSize),
            alpha = alpha
        )
        rejection$results$tested[idx.runs] <- 1
        rejection$results$kBET.pvalue.test[idx.runs] <- pka.res$p.val.test
        rejection$results$kBET.pvalue.null[idx.runs] <- pka.res$p.val.test.null
        compute.LRT.res <- compute_LRT(
            rejection, lrt.expected, lrt.observed, lrt.signif, adapt,
            is.imbalanced, class.frequency, new.class.frequency, batch, dof,
            batch.shuff, env, alpha, i, testSize, idx.runs
        )
        rejection <- compute.LRT.res$rejection
        lrt.expected <- compute.LRT.res$lrt.expected
        lrt.observed <- compute.LRT.res$lrt.observed
        lrt.signif <- compute.LRT.res$lrt.signif
        if (exists(x = "exact.observed")) {
            exact.observed.res <- run.kbet.exact.observed(
                adapt, is.imbalanced, env, new.class.frequency, batch,
                batch.shuff, alpha, testSize, exact.expected, exact.observed,
                exact.signif, rejection, idx.runs, i, class.frequency
            )
            rejection <- exact.observed.res$rejection
            exact.expected <- exact.observed.res$exact.expected
            exact.observed <- exact.observed.res$exact.observed
            exact.signif <- exact.observed.res$exact.signif
        }
    }
    return(list(
        rejection = rejection, kBET.expected = kBET.expected,
        kBET.observed = kBET.observed, kBET.signif = kBET.signif,
        lrt.expected = lrt.expected, lrt.observed = lrt.observed,
        lrt.signif = lrt.signif))
}

perform.kbet.addTest <- function(
    env, adapt, is.imbalanced, class.frequency, new.class.frequency, batch, dof,
    alpha, batch.shuff) {

    # perform test
    if (adapt && is.imbalanced) {
        p.val.test <- apply(env, 1,
            FUN = chi_batch_test,
            new.class.frequency, batch, dof
            )
    } else {
        p.val.test <- apply(env, 1,
            FUN = chi_batch_test,
            class.frequency, batch, dof
        )
    }

    is.rejected <- p.val.test < alpha

    p.val.test.null <- apply(apply(
        batch.shuff, 2,
        function(x, freq, dof, envir) {
            apply(envir, 1, FUN = chi_batch_test, freq, x, dof)
        },
        class.frequency, dof, env
    ), 1, mean, na.rm = TRUE)

    return(list(p.val.test = p.val.test, p.val.test.null = p.val.test.null))
}

compute_LRT <- function(
    rejection, lrt.expected, lrt.observed, lrt.signif, adapt, is.imbalanced,
    class.frequency, new.class.frequency, batch, dof, batch.shuff, env, alpha,
    i, testSize, idx.runs) {

    cf <- if (adapt && is.imbalanced) {
        new.class.frequency
    } else {
        class.frequency
    }
    p.val.test.lrt <- apply(env, 1,
        FUN = lrt_approximation, cf, batch, dof
    )
    p.val.test.lrt.null <- apply(apply(
        batch.shuff, 2,
        function(x, freq, dof, envir) {
            apply(envir, 1, FUN = lrt_approximation, freq, x, dof)
        },
        class.frequency, dof, env
    ), 1, mean, na.rm = TRUE)

    lrt.expected[i] <-
        sum(p.val.test.lrt.null < alpha, na.rm = TRUE) /
            sum(!is.na(p.val.test.lrt.null))
    lrt.observed[i] <-
        sum(p.val.test.lrt < alpha, na.rm = TRUE) /
            sum(!is.na(p.val.test.lrt))

    lrt.signif[i] <-
        1 - ptnorm(lrt.observed[i],
            mu = lrt.expected[i], sd = sqrt(
                lrt.expected[i] * (1 - lrt.expected[i]) / testSize
            ),
            alpha = alpha
        )

    rejection$results$lrt.pvalue.test[idx.runs] <- p.val.test.lrt
    rejection$results$lrt.pvalue.null[idx.runs] <- p.val.test.lrt.null

    return(list(
        rejection = rejection, lrt.expected = lrt.expected,
        lrt.observed = lrt.observed, lrt.signif = lrt.signif))
}

run.kbet.exact.observed <- function(
    adapt, is.imbalanced, env, new.class.frequency, batch, batch.shuff, alpha,
    testSize, exact.expected, exact.observed, exact.signif, rejection,
    idx.runs, i, class.frequency) {
    if (adapt && is.imbalanced) {
        p.val.test.exact <- apply(
            env, 1, multiNom, new.class.frequency$freq, batch
        )
    } else {
        p.val.test.exact <- apply(
            env, 1, multiNom, class.frequency$freq, batch
        )
    }
    p.val.test.exact.null <- apply(apply(
        batch.shuff, 2, function(x, freq, envir) {
            apply(envir, 1, FUN = multiNom, freq, x)
        }, class.frequency$freq, env), 1, mean, na.rm = TRUE)
    exact.expected[i] <- sum(
        p.val.test.exact.null < alpha,
        na.rm = TRUE
    ) / testSize
    exact.observed[i] <- sum(
        p.val.test.exact < alpha,
        na.rm = TRUE
    ) / testSize
    # compute the significance level for the number of rejected data points
    exact.signif[i] <-
        1 - ptnorm(exact.observed[i],
            mu = exact.expected[i],
            sd = sqrt(
                exact.expected[i] * (1 - exact.expected[i]) / testSize
            ),
            alpha = alpha
        )
    # p-value distribution
    rejection$results$exact.pvalue.test[idx.runs] <- p.val.test.exact
    rejection$results$exact.pvalue.null[idx.runs] <- p.val.test.exact.null

    return(list(
        rejection = rejection, exact.expected = exact.expected,
        exact.observed = exact.observed, exact.signif = exact.signif))
}

run.kbet.only <- function(initialize.kbet.res, adapt, alpha, n_repeat) {
    rejection <- initialize.kbet.res$rejection
    dim.dataset <- initialize.kbet.res$dim.dataset
    testSize <- initialize.kbet.res$testSize
    k0 <- initialize.kbet.res$k0
    knn <- initialize.kbet.res$knn
    batch <- initialize.kbet.res$batch
    dof <- initialize.kbet.res$dof
    batch.shuff <- initialize.kbet.res$batch.shuff
    kBET.expected <- initialize.kbet.res$kBET.expected
    kBET.observed <- initialize.kbet.res$kBET.observed
    kBET.signif <- initialize.kbet.res$kBET.signif
    for (i in seq_len(n_repeat)) {
        # choose a random sample from dataset
        idx.runs <- sample.int(dim.dataset[1], size = testSize)
        env <- cbind(knn[idx.runs, seq_len(k0 - 1)], idx.runs)
        # perform test
        kbet.res <- perform.kbet.test(
            adapt, initialize.kbet.res$is.imbalanced,
            initialize.kbet.res$new.class.frequency,
            initialize.kbet.res$class.frequency,
            batch.shuff, env, batch, dof, alpha
        )
        # summarise test results
        kBET.expected[i] <- mean(apply(
            kbet.res$p.val.test.null, 2,
            function(x) sum(x < alpha, na.rm = TRUE) / sum(!is.na(x))
        ))
        kBET.observed[i] <- sum(kbet.res$is.rejected, na.rm = TRUE) /
            sum(!is.na(kbet.res$p.val.test))
        # compute significance
        kBET.signif[i] <- 1 - ptnorm(
            kBET.observed[i],
            mu = kBET.expected[i],
            sd = sqrt(kBET.expected[i] * (1 - kBET.expected[i]) / testSize),
            alpha = alpha
        )
        # assign results to result table
        rejection$results$tested[idx.runs] <- 1
        rejection$results$kBET.pvalue.test[idx.runs] <- kbet.res$p.val.test
        rejection$results$kBET.pvalue.null[idx.runs] <- rowMeans(
            kbet.res$p.val.test.null,
            na.rm = TRUE
        )
    }
    return(list(
        rejection = rejection, kBET.expected = kBET.expected,
        kBET.observed = kBET.observed, kBET.signif = kBET.signif
    ))
}

perform.kbet.test <- function(
    adapt, is.imbalanced, new.class.frequency, class.frequency, batch.shuff,
    env, batch, dof, alpha) {
    cf <- if (adapt && is.imbalanced) {
        new.class.frequency
    } else {
        class.frequency
    }
    p.val.test <- apply(env, 1, chi_batch_test, cf, batch, dof)
    is.rejected <- p.val.test < alpha
    p.val.test.null <- apply(
        batch.shuff, 2,
        function(x) {
            apply(
                env, 1, chi_batch_test, class.frequency,
                x, dof
            )
        }
    )
    return(list(
        p.val.test = p.val.test, p.val.test.null = p.val.test.null,
        is.rejected = is.rejected
    ))
}

mean_ci <- function(x, probs) {
    c(Mean = mean(x, na.rm = TRUE), quantile(x, probs, na.rm = TRUE))
}

summarize_kbet_results <- function(
    rejection, kBET.expected, kBET.observed, kBET.signif, lrt.expected = NULL,
    lrt.observed = NULL, lrt.signif = NULL, exact.observed = NULL,
    exact.expected = NULL, exact.signif = NULL, n_repeat = 1, addTest = FALSE) {
    if (n_repeat > 1) {
        # summarize chi2-results
        CI95 <- c(0.025, 0.5, 0.975)
        rejection$summary$kBET.expected <- mean_ci(kBET.expected, CI95)
        rownames(rejection$summary) <- c("mean", "2.5%", "50%", "97.5%")
        rejection$summary$kBET.observed <- mean_ci(kBET.observed, CI95)
        rejection$summary$kBET.signif <- mean_ci(kBET.signif, CI95)
        if (!addTest) {
            rejection$stats$kBET.expected <- kBET.expected
            rejection$stats$kBET.observed <- kBET.observed
            rejection$stats$kBET.signif <- kBET.signif
        } else {
            # summarize lrt-results
            rejection$summary$lrt.expected <- mean_ci(lrt.expected, CI95)
            rejection$summary$lrt.observed <- mean_ci(lrt.observed, CI95)
            rejection$summary$lrt.signif <- mean_ci(lrt.signif, CI95)
            # summarize exact test results
            if (!is.null(exact.observed)) {
                rejection$summary$exact.expected <- mean_ci(
                    exact.expected, CI95)
                rejection$summary$exact.observed <- mean_ci(
                    exact.observed, CI95)
                rejection$summary$exact.signif <- mean_ci(exact.signif, CI95)
            }
        }
        if (n_repeat < 10) {
            warning("The quantile computation for", n_repeat,
                "subset results is not meaningful.\n")
        }
    } else {
        rejection$summary$kBET.expected <- kBET.expected[1]
        rejection$summary$kBET.observed <- kBET.observed[1]
        rejection$summary$kBET.signif <- kBET.signif[1]
        if (addTest) {
            rejection$summary$lrt.expected <- lrt.expected
            rejection$summary$lrt.observed <- lrt.observed
            rejection$summary$lrt.signif <- lrt.signif
            if (!is.null(exact.observed)) {
                rejection$summary$exact.expected <- exact.expected
                rejection$summary$exact.observed <- exact.observed
                rejection$summary$exact.signif <- exact.signif
            }
        }
    }
    return(rejection)
}

plot_kbet_helper <- function(
    kBET.observed, kBET.expected, lrt.observed = NULL, lrt.expected = NULL,
    exact.observed = NULL, exact.expected = NULL, n_repeat) {
    if (!is.null(exact.observed)) {
        plot.data <- data.frame(
            class = rep(c(
                "kBET", "kBET (random)", "lrt", "lrt (random)",
                "exact", "exact (random)"
            ), each = n_repeat),
            data = c(
                kBET.observed, kBET.expected, lrt.observed,
                lrt.expected, exact.observed, exact.expected
            )
        )
    } else if (!is.null(lrt.observed)) {
        plot.data <- data.frame(
            class = rep(c("kBET", "kBET (random)", "lrt", "lrt (random)"),
                each = n_repeat
            ),
            data = c(
                kBET.observed, kBET.expected, lrt.observed,
                lrt.expected
            )
        )
    } else {
        plot.data <- data.frame(
            class = rep(c("observed(kBET)", "expected(random)"),
                        each = n_repeat),
            data = c(kBET.observed, kBET.expected)
        )
    }

    g <- ggplot(plot.data, aes(class, data)) +
        geom_boxplot() +
        theme_bw() +
        labs(x = "Test", y = "Rejection rate") +
        theme(axis.text.x = element_text(angle = 45, hjust = 1)) +
        scale_y_continuous(limits = c(0, 1))

    return(g)
}

#' Run BatchQC shiny app
#'
#' @return The shiny app will open
#' @rawNamespace import(shinyjs, except=show)
#' @import shinythemes
#' @param dev Run the application in developer mode
#'
#' @examples
#' if(interactive()){
#' BatchQC()
#' }
#' @export

BatchQC <- function(dev = FALSE) {
    appDir <- system.file("shiny", package = "BatchQC")
    if (appDir == "") {
        stop("Could not find BatchQC. Try re-installing `BatchQC`.",
                call. = FALSE)
    }
    if (dev) {
        options(shiny.autoreload = TRUE)
    }
    shiny::runApp(appDir, display.mode = "normal")
}

### UMAP TAB ###

## Plot umap
observeEvent(input$umap_plot, {
    req(reactivevalue$se)
    validate(need(input$num_neighbors < dim(reactivevalue$se)[2], "Please
            select a value for the number of neighbors that is less than the
            size of your dataset"),
        need(input$distance > 0 && input$distance < input$spread, "Please
            select a distance that is greater than 0 and less than spread"),
        need(!input$exploratory ||
                (input$exploratory && (dim(reactivevalue$se)[2] > 15)),
        "Exploratory plot is only possible for datasets with more than 15
        samples."))
    assay <- input$umap_assay
    msg <- sprintf('Generating plot for: %s...', paste(assay, collapse = ', '))
    withProgress(message = msg, {
        results <- umap(reactivevalue$se,
            input$umap_assay,
            input$batch_variable,
            input$covar_variable,
            input$num_neighbors,
            input$distance,
            input$spread,
            input$exploratory)
        setProgress(.8, 'Displaying figure...')
        output$umap <- renderPlot({
            results})
        setProgress(1, 'Complete.')
    })
})

options(shiny.maxRequestSize = 600 * 1024 ^ 2)
library(shiny)
library(shinyjs)
library(shinythemes)
library(sva)
library(SummarizedExperiment)
library(data.table)
library(reader)
library(abind)
library(DT)
library(plotly)

source(file.path("utils", "helpers.R"),  local = TRUE)

ui <- navbarPage(

  title = "BatchQC",
  id = "BatchQC",
  fluid = TRUE,
  theme = shinytheme("yeti"),
  source(file.path("ui", "ui_01_upload.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_02_batch_correction.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_03_experimental_design.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_04_variationAnalysis.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_10_kbet.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_05_heatmaps.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_06_dendrogram.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_07_pca.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_09_umap.R"), local = TRUE)$value,
  source(file.path("ui", "ui_08_differentialExpression.R"),
      local = TRUE)$value,
  source(file.path("ui", "ui_11_data_download.R"),  local = TRUE)$value
)

server <- function(input, output, session) {
    reactivevalue <- reactiveValues(counts = NULL,
                                    counts_location = NULL,
                                    metadata = "",
                                    metadata_location = NULL,
                                    se_location = NULL,
                                    se = NULL,
                                    DE_results = NULL)
    output$confounding_table <- NULL
    output$metadata <- NULL
    source(file.path("server/", "server_00_setUpSelections.R"),  local = TRUE)$value
    source(file.path("server/", "server_01_upload.R"), local = TRUE)$value
    source(file.path("server/", "server_02_batch_correction.R"),
        local = TRUE)$value
    source(file.path("server/", "server_03_experimentalDesign.R"),
        local = TRUE)$value
    source(file.path("server/", "server_04_variationAnalysis.R"),
        local = TRUE)$value
    source(file.path("server/", "server_10_kbet.R"), local = TRUE)$value
    source(file.path("server/", "server_05_heatmap.R"), local = TRUE)$value
    source(file.path("server/", "server_06_dendrogram.R"), local = TRUE)$value
    source(file.path("server/", "server_07_pca.R"), local = TRUE)$value
    source(file.path("server/", "server_09_umap.R"), local = TRUE)$value
    source(file.path("server/", "server_08_differentialExpression.R"),
        local = TRUE)$value
}

shinyApp(ui = ui, server = server)

#' Preprocess assay data
#' @param se Summarized Experiment object
#' @param assay Assay from SummarizedExperiment object
#' @param nfeature Number of variable features to use
#' @param log_option "True" if data should be logged, "False" otherwise
#' @return Returns processed data

preprocess <- function(se, assay, nfeature, log_option) {

    data <- assays(se)[[assay]]
    data <- as.matrix(data)
    data <- apply(data, c(1, 2), as.numeric)
    data <- data[rowSums(data) != 0, ]

    #Finding genes w/Highest variance
    vargenes <- apply(data, 1, stats::var)
    vargenes <- vargenes[order(vargenes, decreasing = TRUE)]
    # selects top n most variable genes
    vargenes <- vargenes[seq(1, nfeature)]

    if (log_option) {
        data <- log(data + 1)
    }
    data <- data[names(vargenes), ]
    data <- data + 1

    return(data)
}

### HEATMAP TAB ###

## Plot heatmap
observeEvent(input$heatmap_plot, {
    req(reactivevalue$se)

    validate(need(input$start_n_heatmap >= 1 &&
            input$start_n_heatmap <= (dim(reactivevalue$se)[1] - 1) &&
            input$start_n_heatmap < input$end_n_heatmap,
        paste0("Please enter a start value which is between 1 and ",
            dim(reactivevalue$se)[1],
            " and less than your desired end value (", input$end_n_heatmap,
            ")")),
        need(input$end_n_heatmap <= dim(reactivevalue$se)[1] &&
                input$end_n_heatmap > input$start_n_heatmap,
            paste0("Please select an end value greater than your start value (",
                input$start_n_heatmap, ") and less than ",
                dim(reactivevalue$se)[1])),
    )

    results <- heatmap_plotter(reactivevalue$se,
                            input$heatmap_assay_name,
                            input$end_n_heatmap,
                            input$variates_to_display,
                            input$log_option)

    output$correlation_heatmap <- renderPlot({
        validate(need(input$start_n_heatmap >= 1 &&
                input$start_n_heatmap <= (dim(reactivevalue$se)[1] - 1) &&
                input$start_n_heatmap < input$end_n_heatmap,
            paste0("Please enter a start value which is between 1 and ",
                dim(reactivevalue$se)[1],
                " and less than your desired end value (", input$end_n_heatmap,
                ")")),
            need(input$end_n_heatmap <= dim(reactivevalue$se)[1] &&
                    input$end_n_heatmap > input$start_n_heatmap,
                paste0("Please select an end value greater than your start value (",
                    input$start_n_heatmap, ") and less than ",
                    dim(reactivevalue$se)[1])),
        )
        results$correlation_heatmap
        }, height = function() {
        session$clientData$output_correlation_heatmap_width
    })

    output$topn_heatmap <- renderPlot({
        validate(need(input$start_n_heatmap >= 1 &&
                input$start_n_heatmap <= (dim(reactivevalue$se)[1] - 1) &&
                input$start_n_heatmap < input$end_n_heatmap,
            paste0("Please enter a start value which is between 1 and ",
                dim(reactivevalue$se)[1],
                " and less than your desired end value (", input$end_n_heatmap,
                ")")),
            need(input$end_n_heatmap <= dim(reactivevalue$se)[1] &&
                    input$end_n_heatmap > input$start_n_heatmap,
                paste0("Please select an end value greater than your start value (",
                    input$start_n_heatmap, ") and less than ",
                    dim(reactivevalue$se)[1])),
        )
        results$topn_heatmap
    }, height = function() {
        session$clientData$output_topn_heatmap_width
    })
})

tabPanel("Heatmaps",

    # Application title
    titlePanel("Heatmaps"),

    # Sidebar with a slider input for number of bins
    sidebarLayout(
        sidebarPanel(
            selectizeInput('heatmap_assay_name',
                'Choose assay to display',
                choices = c(),
                multiple = FALSE,
                selected = NULL,
                options = list(placeholder = 'Please select an option below',
                    onInitialize = I('function() { this.setValue(""); }'))),
            selectizeInput('variates_to_display',
                'Choose variable(s) to display on heatmap',
                choices = c(),
                multiple = TRUE,
                selected = NULL),
            numericInput('start_n_heatmap',
                'Choose start value for features (e.g., genes) to be used',
                value = 1,
                min = 1,
                max = 500),
            numericInput('end_n_heatmap',
                'Choose max value for features (e.g., genes) to be used
                (we recommend 500 or less for large datasets)',
                value = 500,
                min = 2),
            checkboxInput("log_option",
                label = "Select to log data (recommended for sequencing counts
                 data)", value = FALSE),
            actionButton('heatmap_plot', label = 'Here we go!')
        ),

        # Show a plot of the generated distribution
        mainPanel(
            tabsetPanel(
                tabPanel("Sample Correlations",
                    plotOutput('correlation_heatmap')
                ),
                tabPanel("Heatmap",
                    plotOutput('topn_heatmap')
                )
            )
        )
    )
)

if (requireNamespace("lintr", quietly = TRUE)) {
  test_that("Package Style", {
    lintr::expect_lint_free(linters = c(lintr::trailing_whitespace_linter(),
                                        lintr::whitespace_linter(),
                                        lintr::infix_spaces_linter(),
                                        lintr::brace_linter(),
                                        lintr::assignment_linter(),
                                        lintr::commas_linter(),
                                        lintr::spaces_left_parentheses_linter(),
                                        lintr::T_and_F_symbol_linter(),
                                        lintr::semicolon_linter()))
  })
}

---
title: "BatchQC Examples"
author:
- name: W. Evan Johnson
  affiliation:
  - Division of Infectious Disease, Department of Medicine, Rutgers University 
  - Director, Center for Data Science, Rutgers University
  email: wj183@njms.rutgers.edu 
- name: Jessica Anderson
  affiliation: 
  - Division of Infectious Disease, Department of Medicine, Rutgers University 
  email: anderson.jessica@rutgers.edu
- name: Solaiappan Manimaran
  affiliation:
  - Computational Biomedicine, Department of Medicine, Boston University
package: BatchQC
output:
  BiocStyle::html_document
vignette: |
  %\VignetteIndexEntry{BatchQC Examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Example 1: Protein Data
This data set is from protein expression data captured for 39 proteins.
It has two batches and two conditions corresponding to case and control.

```{r}
library(BatchQC)
data(protein_data)
data(protein_sample_info)
se_object <- BatchQC::summarized_experiment(protein_data, protein_sample_info)
```

# Example 2: Signature Data
This data set is from signature data captured when activating different growth 
pathway genes in human mammary epithelial cells (GEO accession: GSE73628). 
This data consists of three batches and ten different conditions corresponding 
to control and nine different pathways

```{r}
data(signature_data)
data(batch_indicator)
se_object <- BatchQC::summarized_experiment(signature_data, batch_indicator)
```

# Example 3: Bladderbatch Data
This data set is from bladder cancer data. This dataset has 57 bladder samples
with 5 batches and 3 covariate levels (cancer, biopsy, control). Batch 1 
contains only cancer, 2 has cancer and controls, 3 has only controls, 4 contains
only biopsy, and 5 contains cancer and biopsy. This data set is from the
bladderbatch package which must be installed to use this data example set (Leek 
JT (2023). bladderbatch: Bladder gene expression data illustrating batch 
effects. R package version 1.38.0).

```{r, eval = FALSE}
if (!requireNamespace("bladderbatch", quietly = TRUE))
    BiocManager::install("bladderbatch")
se_object <- BatchQC::bladder_data_upload()
```

# Example 4: TB Data
This is a whole blood gene expression profiling from well and malnourished
Indian individuals with TB and severely malnourished household contacts with
latent TB infection (LTBI). Severe malnutrition was defined as body mass index
<16. kg/m2 in adults and based on weight-for-height Z scores in children <18
years. Gene expression was measured using RNA-sequencing. (VanValkenburg A, et
al. Malnutrition leads to increased inflammation and expression of tuberculosis
risk signatures in recently exposed household contacts of pulmonary
tuberculosis. Front Immunol. 2022 Sep 28;13:1011166.
doi: 10.3389/fimmu.2022.1011166")

```{r, eval = FALSE}
if (!requireNamespace("curatedTBData", quietly = TRUE))
    BiocManager::install("curatedTBData")
se_object <- BatchQC::tb_data_upload()
```

# Session info {.unnumbered}

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

---
title: "Introduction to BatchQC"
author: 
- name: W. Evan Johnson
  affiliation:
  - Division of Infectious Disease, Department of Medicine, Rutgers University 
  - Director, Center for Data Science, Rutgers University
  email: wj183@njms.rutgers.edu 
- name: Jessica McClintock
  affiliation: 
  - Division of Infectious Disease, Department of Medicine, Rutgers University 
  email: jessica.mcclintock@rutgers.edu
date: '`r format(Sys.Date(), "%B %e, %Y")`'
package: BatchQC
output: 
  BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{Introdution to BatchQC}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

# Introduction
Sequencing and microarray samples are often collected or processed in multiple 
batches or at different times. This often produces technical biases that can 
lead to incorrect results in the downstream analysis. BatchQC is a software tool
that streamlines batch preprocessing and evaluation by providing shiny app
interactive diagnostics, visualizations, and statistical analyses to explore
the extent to which batch variation impacts the data. This is contained in a
full R package which allows reproducibility of all shiny evaluations and
analyses.  BatchQC diagnostics help determine whether batch adjustment needs to
be done, and how correction should be applied before proceeding with a
downstream analysis. Moreover, BatchQC interactively applies multiple common
batch effect approaches to the data, and the user can quickly see the benefits
of each method. BatchQC is developed as a Shiny App, but is reproducible at the
command line through the functions contained in the R package. The output of the
shiny app is organized into multiple tabs, and each tab features an important 
part of the batch effect analysis and visualization of the data. The BatchQC 
interface has the following features:

1. Upload Data
2. Batch Correction/Normalization (can also check data distribution)
3. Experimental Design to view summary of data
4. Variation Analysis (including lambda statistic)
5. kBET
6. Numerous Visualization methods including:
    - Heatmap plot of gene expressions/Median Correlation Plot
    - Dendrograms
    - Principal Component Analysis (PCA)
    - Uniform Manifold Approximation and Projection (UMAP)
7. Differential Expression Analysis and Volcano Plot
8. Data Download to export any corrections or normalizations as an SE object

`BatchQC()` is the function that launches the Shiny App in interactive mode.

# Installation
## Bioconductor Version
To begin, install [Bioconductor](http://www.bioconductor.org/) and then install
BatchQC:

```{R, eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("BatchQC")
```

## Github Version
To install the most up-to-date version of BatchQC, please install directly from
GitHub. You will need the devtools package. You can install both of these with
the following commands:
```{R, eval = FALSE}
if (!require("devtools", quietly = TRUE)) {
    install.packages("devtools")   
}

library(devtools)
install_github("wejlab/BatchQC")
```

## Load BatchQC and Launch Shiny App
You should now be able to load BatchQC and launch the shiny app.

```{R load package}
library(BatchQC)
```

```{R Launch Shiny App, eval = FALSE, echo = TRUE}
BatchQC()
```

# Example Usage
Below is an example of how you may use the shiny app to analyze your data:

## 1. Upload data set 
Upon launching the shiny app, you will be on the "Upload Data" screen. From
here, you may upload the following data:

1. **Your own data w/Count and Metadata file.** Count file should have sample
ID as column and item of interest as row. Metadata should have sample ID as row
and all meta data labeled in the column. Both files should be uploaded as .csv
by selecting "Browse" and selecting the appropriate files from your computer.
2. **Your own data as a Summarized Experiment object.** Upload should be a .RDS
file type and can be uploaded by selecting "Browse" and then the appropriate 
file from your computer
3. **An example data set.** See the examples vignette for additional
information on each example data set

A preview of the selected data will show up under the "Input Summary" and "Full
Metadata" tab. Input summary shows the counts file/assay on the Input summary
tab and the metadata on the "Full Metadata" tab. If a preview does not load
as expected, please ensure your dataset is structured properly and a valid file
type.

**You MUST hit "Upload" for your data to be available to interact with the shiny
app.**

All shiny app functions can also be run from the command line; all command line
execution require that your data is in a Summarized Experiment object. You
can create a summarized experiment object from a features and metadata matrix. 
This would be done as follows:

```{r Create Sumarized Experiment Object}
# Load package data examples
data(signature_data)
data(batch_indicator)

# Create SE Object
features <- signature_data
metadata <- batch_indicator
se_object <- BatchQC::summarized_experiment(features, metadata)

# Name your assay
SummarizedExperiment::assayNames(se_object) <- "log_intensity"

# Ensure all relevat metadata varaibles are factors
se_object$batch <- as.factor(se_object$batch)
se_object$condition <- as.factor(se_object$condition)
```

Likewise, you can upload your own data into a summarized experiment object for
use with command line functions by providing a data matrix and sample info
matrix to the summarized_experiment function.

The remainder of this documentation will utilize the TB example data
set included in the package. This can be loaded from the command line as a
summarized experiment object with the following:

```{R Load TB example data}
se_object <- tb_data_upload()
```

For ease of reference, we will also create variables to store the assay we are
interested in, the batch variable, and the covariable we are interested in.

```{R TB Variables}
assay_of_interest <- "features"
batch <- "Experiment"
covar <- "TBStatus"
```

## 2. Normalization/Batch Correction/Data Distribution Check
You may choose to use these tools in a different order, although we recommend
checking the distribution of your data before deciding the best batch
correction method. you may also choose to explore your raw data with the 
various tools within BatchQC to confirm the presence of a batch effect prior
to applying a batch correction method. In this example, we have applied a batch
correction method based on the results of the distribution check, but you may
choose to do further analysis before selecting (a) (or even comparing) batch 
correction method(s).

### Data Distribution Check
`compute_aic' can provide a negative binomial distribution value only for
positive discrete data. Positive non-discrete data will result in `compute_aic`
reporting only the voom and log-normal distribution analysis.

The negative binomial goodness-of-fit will provide an estimate for discrete or
non-discrete data.

These tools are found on the Batch Correction/Normalization tab as we recommend
utilizing these tools to inform your choice of batch correction method.

#### AIC
```{R AIC Model Fit}
TB_AIC <- compute_aic(se = se_object,
    assay_of_interest = assay_of_interest,
    batchind = batch,
    groupind = covar,
    maxit = 1000,
    zero_filt_percent = 15
    )
TB_AIC["AIC_table"]
TB_AIC["AIC_boxplot"]
```

The lowest AIC represented by "AIC Score" indicates the model that is
likely the best fit. In this dataset, the lowest "AIC Score" is lognormal.
Therefore, we should consider using ComBat as our correction method. 

#### Negative Binomial GoF
We can further confirm that a negative binomial distribution is not a good fit
by running the DESeq2 negative binomial check.

```{R Neg Binomial Model Fit TB Data}
# Set a seed if you would like reproducible results
set.seed(1)

TB_fit <- goodness_of_fit_nb(se = se_object,
    count_matrix = assay_of_interest,
    condition = covar,
    other_variables = batch,
    num_genes = 3000,
    method = "edgeR")

TB_fit$res_histogram
TB_fit$recommendation
```

The recommendation states that the edgeR negative binomial distribution is
likely being violated. Therefore we probably should not consider down stream 
methods that assume a negative binomial distribution.

### Apply Normalization methods
After you have successfully uploaded your data set of interest, you can apply
one of the following Normalization methods if appropriate:

1. **CPM or counts per million.** CPM calculates the counts mapped to a feature 
relative to the total counts mapped to a sample times one million. CPM may be 
used to adjust expression count biases introduced by sequencing depth. CPM 
adjusted values are not recommended for differential expression analysis or 
within sample comparison.
2. **DESeq.** DESeq calculates the counts mapped to a feature divided by sample-
specific size factors. Size factors are determined by the median ratio of gene 
counts relative to the geometric mean per feature. DESeq may be used to adjust
expression count biases introduced by sequencing depth and RNA composition. 
DESeq adjusted values are not recommended for within sample comparison.
3. **edgeR.** edgeR calculates scale factors using a trimmed mean of M-values
between each pair of samples and multiplies the scale factors with the original
library size to get the normalized library size. This normalization method
should be selected when you plan to use edgeR for differential expression
analysis. The original read (raw) counts should be the provided input for this
normalization method.
4. **voom.** Voom calculates logCPM, estimates mean-variance relationship and
uses this to compute observation-level weights. It is appropriate for count
data that is to be used with limma batch correction.

To use one of these two methods, select it from the normalization method drop 
down, select an assay to perform the normalization on from the drop down, and
name the new assay (the prepopulated name can be changed to your preference).
You may also choose to log-transform the results of either of these
normalization methods or your raw data by selecting the "Log transform" box. Hit
"Normalize" to complete the analysis. Your new assay will now be available for
further analysis.

Alternatively, this can be called directly from the command line on an SE
object and will return the same SE object with an additional assay with the name
that you provide as the output_assay_name.

Our TB data set is raw count data and to utilize ComBat we will need
normalized data. We will utilize log_CPM normalized data for ComBat. However,
examples are provided below on how you would complete many different
normalization methods if needed via the command line:

```{r}
# CPM Normalization
se_object <- BatchQC::normalize_SE(se = se_object, method = "CPM",
    log_bool = FALSE, assay_to_normalize = assay_of_interest,
    output_assay_name = "CPM_normalized_counts")

# CPM Normalization w/log
se_object <- BatchQC::normalize_SE(se = se_object, method = "CPM",
    log_bool = TRUE, assay_to_normalize = assay_of_interest,
    output_assay_name = "CPM_log_normalized_counts")

# DESeq Normalization
se_object <- BatchQC::normalize_SE(se = se_object, method = "DESeq",
    log_bool = FALSE, assay_to_normalize = assay_of_interest,
    output_assay_name = "DESeq_normalized_counts")

# DESeq Normalization w/log
se_object <- BatchQC::normalize_SE(se = se_object, method = "DESeq",
    log_bool = TRUE, assay_to_normalize = assay_of_interest,
    output_assay_name = "DESeq_log_normalized_counts")

# log adjust
se_object <- BatchQC::normalize_SE(se = se_object, method = "none",
    log_bool = TRUE, assay_to_normalize = assay_of_interest,
    output_assay_name = "log_normalized_counts")
```

### Apply Batch Effect Correction
Select the appropriate Batch Effect correction model to create a batch-corrected
assay for your data. You may select one of the following:

1. **ComBat-Seq.** ComBat-Seq uses a negative binomial regression to model batch
effects. It requires untransformed, raw count data to adjust for batch effect. 
Only use this model on an untransformed raw counts assay (not normalized 
assays).
2. **ComBat.** ComBat corrects for Batch effect using a parametric empirical 
Bayes framework and data should be cleaned and normalized. Select a normalized 
assay to run this on.
3. **limma.** limma fits a linear model to the data and then removes components
due to batch effect. It should be used on log transformed data.
4. **sva** sva identifies and estimates surrogate variables for unknown sources
of variation. It is appropriate when you know your experimental variable of
interest (like the disease state of samples), but unknown sources of variation.
It should be run on cleaned and log transformed data. This can be run with or
without additional adjustment variables.
5. **svaseq** svaseq is a variant of sva specifically for sequencing data. Data
should still be normalized.
6. **Harman** a PCA and constrained optimization based technique that maximizes
the removal of batch effects from datasets, with the constraint that the
probability of overcorrection (i.e. removing genuine biological signal along
with batch noise) is kept to a fraction (as set by the user).

To apply your desired batch effect correction model, select the method from the 
drop down menu, then select the appropriate assay in the second drop down menu,
select your batch variable (typically labeled batch in built-in examples), and 
the covariates that you would like to preserve (not including the batch
variable). If desired, revise the name for the corrected assay and then hit
"Correct."

A progress bar will pop up in the bottom right hand corner and a message will 
state "Correction Complete" in the bottom right hand corner when complete.

You are now ready to analyze your raw and batch effect corrected data sets!

The distribution analysis indicated that our TB data follows a log normal
distribution, so ComBat is likely an appropriate batch correction tool. We will
use the log-CPM assay since ComBat requires normalized data. Therefore, to run 
from the command line use the following:

```{r}
# ComBat correction
se_object <- BatchQC::batch_correct(se = se_object, method = "ComBat",
    assay_to_normalize = "CPM_log_normalized_counts", batch = batch,
    covar = c(covar), output_assay_name = "ComBat_corrected")
```

ComBat-Seq (or limma or sva or svaseq) can be applied to a data set by providing
"ComBat-Seq" (or "limma" or "sva" or "svaseq") to the method parameter in the
function call.

## 3. Experimental Design
### Batch Design
This tab allows you to view the batch status of each condition. Select the 
variable that represents your batch variable and then select the variable that 
represents the condition. A table will then appear listing each condition in the
row and how many samples in each condition are in each batch (displayed in the 
columns).

To recreate this table, run the following code, which will produce a tibble
as output:
```{R}
batch_design_tibble <- BatchQC::batch_design(se = se_object, batch = batch, 
    covariate = covar)
batch_design_tibble
```

### Confounding Statistics
This tab displays the Pearson correlation coefficient and Cramer's V estimation
based on the selected batch and condition.

This table can be recreated with the following code:
```{R}
confound_table <- BatchQC::confound_metrics(se = se_object, batch = batch)
confound_table
```

#### Pearson Correlation Coefficient
Pearson correlation coefficient indicates the strength of the linear 
relationship between your batch and condition variables. It can range from -1 to
1. The closer the value is to 0, the less likely there is a batch effect. The 
closer the value is to -1 or 1, the more likely there is a batch effect.

To calculate only the pearson correlation coefficient, run the following:
```{R}
pearson_cor_result <- BatchQC::std_pearson_corr_coef(batch_design_tibble)
pearson_cor_result
```

#### Cramer's V
This is an additional metric for batch effect and will be between 0 and 1. The 
closer the value is to 0, the lower the batch effect. The closer the value is to
1, the greater the batch effect

To calculate only Cramer's V, run the following:
```{R}
cramers_v_result <- BatchQC::cramers_v(batch_design_tibble)
cramers_v_result
```

If both the Pearson Correlation Coefficient and the Cramer's V test indicate low
batch effect, you likely do not need to adjust your results for batch. However,
if one or both metrics indicate some or high batch effect, you should use
additional analysis tools (listed below) to explore the need for a batch
correction for your results.

## 4. lambda/Variation Analysis
First, select your unadjusted assay, your batch variable and any covariates of
interest. Click "Here we go!" to see the lambda statistic and variation analysis
results. The "Individual Variable" tab shows the individual, or raw variation, 
explained by each variable. The "Residual" tab displays the type 2, or residual,
variation for each variable. A box plot is displayed along with a table with the
variation specific to each gene listed. This table is searchable and can be 
displayed in ascending or descending order. Notice the amount of variation 
attributed to batch. In our signature data set, most of the variation is 
attributed to batch rather than the condition. This indicates a need for a batch
correction.

The "Individual Variation Variable/Batch Ratio" tab displays the individual, or
raw variation, divided by the batch. A ratio greater that 1 indicates that batch
has a stronger affect than the variable of interest. And the "Residual Variation
Variable/Batch Ratio" displays the residual, or type 2 variation, divided by the
batch. A ratio greater that 1 indicates that batch has a stronger affect than
the variable of interest.

Next, select the batch corrected assay, with the same batch and condition
variable and observe the changes in variation. When batch effect is being
properly corrected, you should notice that the variation explained by the batch
variable has decreased in the batch corrected assay, indicating that the
variation in the data is now more likely to be associated with your condition of
interest rather than the batch. The ratio statistics should also be less than 1.
This is visible in our example data set and confirms the need for batch
adjustment.

To recreate the lambda statistic for the TB counts assay, use the following
code:

```{R lambda Statistic}
lambda_calculation <- run_lambda(se = se_object,
    assay = assay_of_interest,
    batch = batch,
    condition = covar)

lambda_calculation$correction_recommendation
lambda_calculation$lambda_stat
```

To recreate the variation analysis plot for the TB counts assay, use the
following code for individual variation:
```{r}
ex_variation <- batchqc_explained_variation(se = se_object,
    batch = batch, condition = covar, assay_name = assay_of_interest)
EV_boxplot <- BatchQC::EV_plotter(batchqc_ev = ex_variation[[1]])
EV_boxplot
BatchQC::summary_stats_EV_table(ex_variation[[1]])
```

To recreate the residual variation analysis table for the TB counts assay, use 
the following code:
```{r}
EV_boxplot_type_2 <- BatchQC::EV_plotter(batchqc_ev = ex_variation[[2]])
EV_boxplot_type_2
BatchQC::summary_stats_EV_table(ex_variation[[2]])
```

We can then compare our batch corrected assay, using the following code:
```{r}
ex_variation <- batchqc_explained_variation(se = se_object,
    batch = batch, condition = covar, assay_name = "ComBat_corrected")
EV_boxplot <- BatchQC::EV_plotter(batchqc_ev = ex_variation[[1]])
EV_boxplot
BatchQC::summary_stats_EV_table(ex_variation[[1]])
EV_boxplot_type_2 <- BatchQC::EV_plotter(batchqc_ev = ex_variation[[2]])
EV_boxplot_type_2
BatchQC::summary_stats_EV_table(ex_variation[[2]])
EV_table <- BatchQC::EV_table(batchqc_ev = ex_variation[[1]])
EV_table
EV_table_type_2 <- BatchQC::EV_table(batchqc_ev = ex_variation[[2]])
EV_table_type_2
```

## 5. kBET
To confirm the presence of batch effects in the simulated dataset, we use kBET, 
a method that quantifies batch effects through a chi-square test. kBET works by 
comparing the batch label distribution in each sample's local neighborhood to 
the global batch label distribution across the entire dataset. If these 
distributions differ significantly, it indicates the presence of batch effects.

Below is how we would run the kBET analysis on the TB data set.

```{R kBET}
TB_kBET <- BatchQC::run_kBET(se = se_object,
    assay_to_normalize = assay_of_interest,
    batch = batch)

BatchQC::plot_kBET(TB_kBET)
```

The boxplot shows that the observed rejection rate is significantly higher than 
the expected rejection rate, indicating a strong batch effect in the TB dataset.

## 6. Visualizations
BatchQC contains a number of visualization methods that may help you identify a 
batch effect (or to confirm that a batch effect has been removed).

### Heatmaps
The heatmap tab contains options for a sample correlation heatmap or for a more
traditional heatmap comparing samples and genes. Select your assay of interest
and the batch and condition(s) of interest. We recommend viewing no more than
500 elements at a time (or the max number in your data set). 

#### Sample Correlations
This heat map shows how similar each sample is to each other sample in your
data (samples are plotted on both the x and y axis, so are identical to 
their self). The samples are clustered together based on similarity. Ideally, 
you would expect your samples to cluster and be more similar to samples from
the same condition rather than from the sample batch. If there is clear visual 
clustering of the batch in your raw data, you should apply a batch correction 
method. Your batch corrected assay should have stronger clustering by condition.

Using the TB dataset, we initially see clustering based on batch, but
after apply ComBat, our adjusted data set clusters more based on condition.

Use the following code to recreate the sample correlation heatmap:
```{r}
heatmaps <- BatchQC::heatmap_plotter(se = se_object, assay = assay_of_interest,
    nfeature = 38, annotation_column = c(batch, covar),
    log_option = "FALSE")
correlation_heatmap <- heatmaps$correlation_heatmap
correlation_heatmap
```

#### Heatmap
The heatmap shoes the sample clustering on the x axis and the y axis is each
gene. The values on the heatmap represent gene expression. The dendrogram on the
x axis is the same clustering arrangement as was seen in the sample
correlations. This heatmap allows the viewer to see patterns of gene expression
across the samples based on the selected variables. Remember that if batch is
clustering together on your raw data, you should perform a batch effect 
correction to adjust your data.

Use the following code to recreate the heatmap:
```{r}
heatmaps <- BatchQC::heatmap_plotter(se = se_object, assay = assay_of_interest,
    nfeature = 38, annotation_column = c(batch, covar),
    log_option = FALSE)
heatmap <- heatmaps$topn_heatmap
heatmap
```


### Dendrograms
Within the dendrogram tab you can view either a traditional dendrogram or a
circular dendrogram.

#### Dendrogram
This tab shows a more detailed dendrogram than seen in the heatmap. When samples
cluster together by batch, you are likely to have a strong batch effect in your
data and you should consider a batch correction method for your data. Samples
are clustered based on similarity of gene expression and other metadata using an
Euclidian distance metric. Labels will include the batch variable and one
covariate of interest.

Use the following code to recreate the dendrogram for the TB data set: 
```{r}
dendrogram_plot <- BatchQC::dendrogram_plotter(se = se_object,
    assay = assay_of_interest,
    batch_var = batch,
    category_var = covar)
dendrogram_plot$dendrogram
```

#### Circular Dendrogram
This circularizes the previous dendrogram to better show relatedness of all 
branches on the dendrogram without the appearance of great distance of the 
samples at the top and the bottom of the chart.

Use the following code to recreate the circular dendrogram: 
```{r}
circular_dendrogram_plot <- BatchQC::dendrogram_plotter(
    se = se_object, assay = assay_of_interest, batch_var = batch,
    category_var = covar)
circular_dendrogram_plot$circular_dendrogram
```

### PCA Analysis
You may select multiple assays to plot PCA analysis side by side. Therefore, 
select your raw data assay and batch corrected assay, the batch variable as 
shape, condition as color, and the number of top variable features you are
interested in. Review the clustering pattern. In data sets where a strong batch
effect is seen, similar shapes (batch) will all cluster together. Whereas if a
batch effect is not seen, shapes (batch) should be dispersed throughout and you
should see clustering by condition (color).

Use the following code to recreate the PCA plot and associated table that lists
the explained variation of each PC for our TB data set. Please note that count
data should be log-normalized prior to PCA plotting, so we use the
log_normalized assay:
```{r}
pca_plot <- BatchQC::PCA_plotter(se = se_object, nfeature = 20, color = batch,
    shape = covar, batch = batch,
    assays = c("log_normalized_counts", "ComBat_corrected"),
    xaxisPC = 1, yaxisPC = 2, log_option = FALSE)
pca_plot$plot
pca_plot$var_explained
```

### UMAP Analysis
You may plot your data on a UMAP by selecting the assay, the batch variable, and
selecting a value for nearest neighbors and minimum distance for the points to
plot. The default for neighbors is 15 and minimum distance default is 0.1. We
expect each batch to cluster together when there is a batch effect present.
Otherwise, having dispersed points in each batch indicates there is not a strong
batch effect. You can adjust the nearest neighbor parameter, a lower value will
prioritize local structure and a higher value will represent bigger picture, but
lose finer details. A higher minimum distance value will also put less emphasis
on the global structure.

Use the following code to recreate the UMAP for the TB data set:

```{r UMAP}
umap_plot <- BatchQC::umap(se = se_object,
    assay_of_interest = assay_of_interest, batch = batch, covar = covar,
    neighbors = 15, min_distance = 0.1, spread = 1, log_option = TRUE)

umap_plot
```

This code creates a UMAP for the batch corrected assay:
```{r UMAP batch corrected}
umap_plot_batch_corrected <- BatchQC::umap(se = se_object,
    assay_of_interest = "ComBat_corrected", batch = batch, covar = covar,
    neighbors = 15, min_distance = 0.1, spread = 1)

umap_plot_batch_corrected
```

## 7. Differential Expression Analysis
To explore which features are differentially expressed between conditions, use
the differential expression tab. DE_Seq2 should be used for count data
(non-negative, integer values) with a negative binomial distribution, edgeR can
be used for edgeR-normalized data, ANOVA can be used for normal-distributed
data, Kruskal-Wallis test can be used as a nonparametric equivalent of one-way
ANOVA, and limma can be used for all other data. edgeR implemented in BatchQC
uses glm quasi-likelihood F-tests to test for differential expression.

Our TB data is count data with a log normal distribution, so we will use limma.
We will select the assay we are interested in, select the batch variable and all
covariates that we would like to include in the analysis, and select a method to
adjust p-values. The "Results Table" return a table for each condition within
each variable. The results table displays the log2 fold change, p-value and
adjusted p-value for each feature. The "P-Value Analysis" tab displays a box
plot of p-values for each analysis. In both the "Results Table" and "P-Value
Analysis" tabs, the tables can be sorted by ascending or descending order of a 
column of your choice. They are also searchable with the "Search" bar (which can
be helpful if you are interested in a specific feature or p-value threshold).

To run the limma differential expression analysis from the command line, run the
following code (which can be modified to run DESeq2 when appropriate by changing
the method argument to "DESeq2"):
```{r}
differential_expression <- BatchQC::DE_analyze(se = se_object,
    method = "limma", batch = batch, conditions = c(covar),
    assay_to_analyze = assay_of_interest, padj_method = "BH")
```

After running the differential expression analysis, you can see all the
completed analysis by running this code: 

```{r}
names(differential_expression)
```

You can then view the log2 fold change, p-value and adjusted p-value table for a
specific analysis of interest, by running the following:
```{r}
head(differential_expression[["TBStatusPTB"]])
```

After running a differential expression analysis, you may replicate the p-value
box plot and table from the command line by running the following code:

```{r}
pval_plotter(differential_expression)
head(pval_summary(differential_expression))
```

### Volcano Plot
The "Volcano Plot" is based on the limma or DESeq2 analysis results and must be
run after completing the differential expression analysis. You should select the
analysis result that you are interested in, specify the threshold for p-value
cut off (the default is 0.05), and the magnitude of expression change (the
shiny default is the average of the absolute value of the minimum and maximum 
value in the data set). After clinking "Run," a volcano plot will appear. All
values that are below the p-value threshold and exceed the set expression
change threshold will appear in red, indicating that these may be potential
features that differ between the conditions in that variable.

In a data set with a batch effect, you will likely see many significant values
in both your p-value analysis and in the volcano plot. When you apply a batch
correction analysis, you should see more significant values in your covariate
conditions than in your batch conditions and fewer significant values that meet
the volcano plot thresholds.

The volcano plot is interactive and you can select specific points to view the
feature information, log2fold change, and -log10(p-value).

To recreate the volcano plot for the "TBStatusPTB" assay (which you may change
by using any of the named results assays) differential expression result from 
our uncorrected TB data , run the following code:

```{r}
value <- round((max(abs(
    differential_expression[[length(differential_expression)]][, 1]))
    + min(
        abs(differential_expression[[length(differential_expression)]][, 1])))
    / 2)
volcano_plot(DE_results = differential_expression[["TBStatusPTB"]],
    pslider = 0.05,
    fcslider = value)
```

## 8. Data Download
You may download the data set that you have been working with int he shiny app,
including the  assays you added in the "Normalization" or "Batch Effect
Correction" steps when you uploaded your data. A preview is provided that shows
the Summarized Experiment (SE) object, including the dimensions, metadata table,
assays, rownames and colNames.

Click the "Download" button and you can save this SE object to your computer for
additional analysis outside of the BatchQC shiny app. The object is saved as a
.RDS Summarized Experiment object in the folder of your choice. You should give
it a descriptive name (default is "se.RDS").

If you run code from the command line according to the above instructions, all
assays will be saved to your original SE object and available for your use in
other applications. You may save it as an SE object by running the following:

```{r, eval = FALSE, echo = TRUE}
file_location <- "location/to/save/object"

saveRDS(object = se_object, file = file_location)
```

# Conclusion
Many of these analyses can be run again on the batch corrected assay to see
improvements in groupings based on your biological variable rather than the 
batch effect.

# Session info {.unnumbered}

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negative_binomial_check.R
\name{nb_histogram}
\alias{nb_histogram}
\title{This function creates a histogram from the negative binomial goodness-of-fit
adjusted pvalues.}
\usage{
nb_histogram(p_val_table)
}
\arguments{
\item{p_val_table}{table of adjusted p-values from the nb test}
}
\value{
a histogram of the number of genes within a p-value range
}
\description{
This function creates a histogram from the negative binomial goodness-of-fit
adjusted pvalues.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_statistics.R
\name{std_pearson_corr_coef}
\alias{std_pearson_corr_coef}
\title{Calculate a standardized Pearson correlation coefficient}
\usage{
std_pearson_corr_coef(bd)
}
\arguments{
\item{bd}{batch design}
}
\value{
standardized Pearson correlation coefficient
}
\description{
Calculate a standardized Pearson correlation coefficient
}
\examples{
library(scran)
se <- mockSCE()
batch_design_tibble <- batch_design(se, batch = "Mutation_Status",
                                                covariate = "Treatment")
pearson_cor_result <- BatchQC::std_pearson_corr_coef(batch_design_tibble)
pearson_cor_result

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explained_variation.R
\name{get.res}
\alias{get.res}
\title{Helper function to get residuals}
\usage{
get.res(y, X)
}
\arguments{
\item{y}{assay}

\item{X}{model matrix design}
}
\value{
residuals
}
\description{
Helper function to get residuals
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{PCA_plotter}
\alias{PCA_plotter}
\title{This function allows you to plot PCA}
\usage{
PCA_plotter(se, nfeature, color, shape, batch, assays, xaxisPC,
yaxisPC, log_option = FALSE)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{nfeature}{number of features}

\item{color}{choose a color}

\item{shape}{choose a shape}

\item{batch}{variable representing batch (for ellipses)}

\item{assays}{array of assay names from \code{se}}

\item{xaxisPC}{the PC to plot as the x axis}

\item{yaxisPC}{the PC to plot as the y axis}

\item{log_option}{TRUE if data should be logged before plotting (recommended
for sequencing counts), FALSE if data should not be logged (for instance,
data is already logged); FALSE by default}
}
\value{
List containing PCA info, PCA variance and PCA plot
}
\description{
This function allows you to plot PCA
}
\examples{
library(scran)
se <- mockSCE()
se_object_ComBat_Seq <- BatchQC::batch_correct(se, method = "ComBat-Seq",
                                            assay_to_normalize = "counts",
                                            batch = "Mutation_Status",
                                            covar = "Treatment",
                                            output_assay_name =
                                                "ComBat_Seq_Corrected")
pca_plot <- BatchQC::PCA_plotter(se = se_object_ComBat_Seq,
                            nfeature = 2, color = "Mutation_Status",
                            shape = "Treatment", batch = "batch",
                            assays = c("counts", "ComBat_Seq_Corrected"),
                            xaxisPC = 1, yaxisPC = 2, log_option = FALSE)
pca_plot$plot
pca_plot$var_explained

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{heatmap_plotter}
\alias{heatmap_plotter}
\title{Heatmap Plotter}
\usage{
heatmap_plotter(se, assay, nfeature, annotation_column, log_option)
}
\arguments{
\item{se}{SummarizedExperiment}

\item{assay}{normalized or corrected assay}

\item{nfeature}{number of features to display}

\item{annotation_column}{choose column}

\item{log_option}{TRUE if data should be logged before plotting (recommended
for sequencing counts), FALSE if data should not be logged (for instance,
data is already logged)}
}
\value{
heatmap plot
}
\description{
This function allows you to plot a heatmap
}
\examples{
library(scran)
se <- mockSCE()
heatmaps <- BatchQC::heatmap_plotter(se,
                                assay = "counts",
                                nfeature = 15,
                                annotation_column = c("Mutation_Status",
                                "Treatment"), log_option = FALSE)
correlation_heatmap <- heatmaps$correlation_heatmap
correlation_heatmap

heatmap <- heatmaps$topn_heatmap
heatmap

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_aic.R
\name{compute_aic}
\alias{compute_aic}
\title{Compute the AIC for lognormal (ComBat) model, negative binomial (ComBat-seq)
model and the Voom model}
\usage{
compute_aic(
  se,
  assay_of_interest,
  batchind,
  groupind,
  maxit = 25,
  zero_filt_percent = 100
)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{assay_of_interest}{The assay name from se that you are interested in
analyzing. This assay need to be a counts assay containing only
non-negative integers.}

\item{batchind}{Factor or numeric vector of length = ncol(dat);
batch indicator for each sample.}

\item{groupind}{Factor or numeric vector of length = ncol(dat);
biological group label/indicator for each sample.}

\item{maxit}{Integer giving the maximal number of IWLS iterations. Default is
25.}

\item{zero_filt_percent}{Numeric value between 0 and 100, the percentage of
zeros allowed for each gene to be included in the AIC calculation. Genes
with more than this percentage of zeros will be filtered out. Default is
100.}
}
\value{
A list with a  boxplot of the AIC_boxplot and a dataframe containing
the total_AIC values, the frequency of the min_AIC values, the AIC_score, and
the median_AIC value, for the following three elements:
\describe{
\item{nb}{The metric value calculated based on the negative binomial
model.}
\item{lognormal}{The metric value calculated based on the lognormal
model.}
\item{voom}{The metric value calculated based on the voom-based model.}
}
}
\description{
\describe{
\item{nb_result}{A vector contains the AIC based on negative binomial
model for individual genes.}
\item{lognormal_result}{A vector contains the AIC based on lognormal
model for individual genes.}
\item{voom_result}{A vector contains the AIC based on voom transformation
for individual genes.}
\item{total_AIC}{The sum of AICs across all genes for the models in
comparison.}
\item{min_AIC}{The number of minimum AIC across the models in
comparison for individual genes.}
\item{AIC_score}{The ratios between total_AIC and min_AIC for the
models in comparison. The optimal model is the one has minimum value.}
}
}
\details{
This function calculates the AIC based on lognormal distribution,
negative binomial distribution as well as the voom transformation.
It then compares the AICs of the three models across different genes and
yields AIC-based metric values. Must have non-negative data and non-discrete
data will only analyze the lognormal and voom distribution.
}
\examples{
library(scran)
se <- mockSCE()
compare_aic <- compute_aic(se, assay_of_interest = "counts",
                            batchind = "Cell_Cycle",
                            groupind = c("Treatment", "Mutation_Status"))
print(compare_aic)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explained_variation.R
\name{EV_plotter}
\alias{EV_plotter}
\title{This function allows you to plot explained variation}
\usage{
EV_plotter(batchqc_ev)
}
\arguments{
\item{batchqc_ev}{table of explained variation from
batchqc_explained_variation}
}
\value{
boxplot of explained variation
}
\description{
This function allows you to plot explained variation
}
\examples{
library(scran)
se <- mockSCE()
se$Mutation_Status <- as.factor(se$Mutation_Status)
se$Treatment <- as.factor(se$Treatment)
expl_var_result <- batchqc_explained_variation(se, batch = "Mutation_Status",
                            condition = "Treatment", assay_name = "counts")
EV_boxplot <- BatchQC::EV_plotter(expl_var_result[[1]])
EV_boxplot

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umap.R
\name{possible_k_neighbors}
\alias{possible_k_neighbors}
\title{Create a vector of possible nearest neighbor values from 5, 15, 25, 50, and
100}
\usage{
possible_k_neighbors(data_size)
}
\arguments{
\item{data_size}{size of the data set used to create umaps}
}
\value{
k nearest neighbor list
}
\description{
Create a vector of possible nearest neighbor values from 5, 15, 25, 50, and
100
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{protein_data}
\alias{protein_data}
\title{Protein data with 39 protein expression levels}
\format{
A data frame with 39 rows and 24 variables
}
\usage{
data(protein_data)
}
\description{
This data consists of two batches and two conditions
corresponding to case and control. The columns are case/control
samples, and the rows represent 39 different proteins.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{batch_indicator}
\alias{batch_indicator}
\title{Batch and Condition indicator for signature data}
\format{
A data frame with 89 rows and 2 variables:
\describe{
\item{batch}{batch}
\item{condition}{condition}
}
}
\usage{
data(batch_indicator)
}
\description{
This dataset is from signature data captured when activating different growth
pathway genes in human mammary epithelial cells (GEO accession: GSE73628).
This data consists of three batches and ten different conditions
corresponding to control and nine different pathways.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_correction.R
\name{batch_correct}
\alias{batch_correct}
\title{Batch Correct
This function allows you to Add batch corrected count matrix to the SE object}
\usage{
batch_correct(se, method, assay_to_normalize, batch, group = NULL,
covar, output_assay_name, psva, num_sv, limit, numrepeats)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{method}{Normalization Method ("ComBat-Seq", "ComBat", "limma", "sva",
svaseq, "Harman")}

\item{assay_to_normalize}{Which assay use to do normalization}

\item{batch}{The batch}

\item{group}{The group variable}

\item{covar}{list of covariates}

\item{output_assay_name}{name of results assay}

\item{psva}{boolean; default: FALSE. Only used if normalization method is
"sva". If set to TRUE and no covariate input, psva function from the sva
package will be used to remove batch effect.}

\item{num_sv}{boolean; default: FALSE. Only used if normalization method is
"svaseq". The number of estimated latent factor is set to 1 for a small
number of samples. If set to TRUE, svaseq function will estimate the number
of latent factors for you.}

\item{limit}{numeric; default: 0.95. Only used if normalization method is
"Harman". Indicates the limit of confidence in which to stop removing a batch
effect. Must be between 0 and 1.}

\item{numrepeats}{integer; default: 100000L. Only used if normalization
method is "Harman". The number of repeats in which to run the simulated batch
mean distribution estimator using the random selection algorithm.}
}
\value{
a summarized experiment object with normalized assay appended
}
\description{
Batch Correct
This function allows you to Add batch corrected count matrix to the SE object
}
\examples{
library(scran)
se <- mockSCE()
se <- BatchQC::batch_correct(se, method = "ComBat-Seq",
                                    assay_to_normalize = "counts",
                                    batch = "Mutation_Status",
                                    covar = "Treatment",
                                    output_assay_name =
                                        "ComBat_Seq_Corrected")
se <- BatchQC::batch_correct(se, method = "ComBat",
                                    assay_to_normalize = "counts",
                                    batch = "Mutation_Status",
                                    covar = "Treatment",
                                    output_assay_name =
                                        "ComBat_Corrected")
se

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kBET.R
\name{kBET}
\alias{kBET}
\title{kBET - k-nearest neighbour batch effect test}
\usage{
kBET(
  df,
  batch,
  k0 = NULL,
  knn = NULL,
  testSize = NULL,
  do.pca = TRUE,
  dim.pca = 50,
  heuristic = TRUE,
  n_repeat = 100,
  alpha = 0.05,
  addTest = FALSE,
  verbose = FALSE,
  plot = TRUE,
  adapt = TRUE
)
}
\arguments{
\item{df}{dataset (rows: cells, columns: features)}

\item{batch}{batch id for each cell or a data frame with
both condition and replicates}

\item{k0}{number of nearest neighbours to test on (neighbourhood size)}

\item{knn}{an n x k matrix of nearest neighbours for each cell (optional)}

\item{testSize}{number of data points to test,
(10 percent sample size default, but at least 25)}

\item{do.pca}{perform a pca prior to knn search? (defaults to TRUE)}

\item{dim.pca}{if do.pca=TRUE, choose the number of dimensions
to consider (defaults to 50)}

\item{heuristic}{compute an optimal neighbourhood size k
(defaults to TRUE)}

\item{n_repeat}{to create a statistics on batch estimates,
evaluate 'n_repeat' subsets}

\item{alpha}{significance level}

\item{addTest}{perform an LRT-approximation to the multinomial
test AND a multinomial exact test (if appropriate)}

\item{verbose}{displays stages of current computation (defaults to FALSE)}

\item{plot}{if stats > 10, then a boxplot of the resulting
rejection rates is created}

\item{adapt}{In some cases, a number of cells do not contribute
to any neighbourhood
and this may cause an imbalance in observed and expected batch
label frequencies.
Frequencies will be adapted if adapt=TRUE (default).}
}
\value{
list object
\enumerate{
\item \code{summary} - a rejection rate for the data,
an expected rejection rate for random
labeling and the significance for the observed result
\item \code{results} - detailed list for each tested cells;
p-values for expected and observed label distribution
\item \code{average.pval} - significance level over the averaged
batch label distribution in all neighbourhoods
\item \code{stats} - extended test summary for every sample
\item \code{params} - list of input parameters and adapted parameters,
respectively
\item \code{outsider} - only shown if \code{adapt=TRUE}. List of samples
without mutual nearest neighbour: \itemize{
\item \code{index} - index of each outsider sample)
\item \code{categories} - tabularised labels of outsiders
\item \code{p.val} - Significance level of outsider batch label
distribution vs expected frequencies.
If the significance level is lower than \code{alpha},
expected frequencies will be adapted}
}

If the optimal neighbourhood size (k0) is smaller than 10, NA is
returned.
}
\description{
adapted from kBET package (https://github.com/theislab/kBET).
\code{kBET} runs a chi square test to evaluate
the probability of a batch effect.
}
\examples{
library(scran)
se <- mockSCE()
df <- as.matrix(assays(se)[["counts"]])
batch <- data.frame(colData(se))[, "Treatment"]

batch.estimate <- kBET(df, batch)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_statistics.R
\name{cor_props}
\alias{cor_props}
\title{This function allows you to calculate correlation properties}
\usage{
cor_props(bd)
}
\arguments{
\item{bd}{batch design}
}
\value{
correlation properties
}
\description{
This function allows you to calculate correlation properties
}
\examples{
library(scran)
se <- mockSCE()
batch_design_tibble <- batch_design(se, batch = "Mutation_Status",
                                                covariate = "Treatment")
correlation_property <- BatchQC::cor_props(batch_design_tibble)
correlation_property

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_statistics.R
\name{cramers_v}
\alias{cramers_v}
\title{This function allows you to calculate Cramer's V}
\usage{
cramers_v(bd)
}
\arguments{
\item{bd}{batch design}
}
\value{
Cramer's V
}
\description{
This function allows you to calculate Cramer's V
}
\examples{
library(scran)
se <- mockSCE()
batch_design_tibble <- batch_design(se, batch = "Mutation_Status",
                                                covariate = "Treatment")
cramers_v_result <- BatchQC::cramers_v(batch_design_tibble)
cramers_v_result

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variation_ratios.R
\name{variation_ratios}
\alias{variation_ratios}
\title{Creates Ratios of batch to variable variation statistic}
\usage{
variation_ratios(ex_variation_table, batch)
}
\arguments{
\item{ex_variation_table}{table of explained variation results from
batchqc_explained_variation}

\item{batch}{batch}
}
\value{
dataframe with condition/batch ratios
}
\description{
Creates Ratios of batch to variable variation statistic
}
\examples{
library(scran)
se <- mockSCE()
se$Mutation_Status <- as.factor(se$Mutation_Status)
se$Treatment <- as.factor(se$Treatment)
expl_var_result <- batchqc_explained_variation(se, batch = "Mutation_Status",
                            condition = "Treatment", assay_name = "counts")
ratios_results <- variation_ratios(expl_var_result[[1]],
    batch = "Mutation_Status")
ratios_results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lambda_statistic.R
\name{compute_lambda}
\alias{compute_lambda}
\title{Compute the lambda index for determining a need for batch correction}
\usage{
compute_lambda(dat, batchind, groupind)
}
\arguments{
\item{dat}{Numeric matrix of dimension (genes x samples) where each row
represents one gene's expression across samples.}

\item{batchind}{Factor or numeric vector of length = ncol(dat); batch
indicator for each sample}

\item{groupind}{Factor or numeric vector of length = ncol(dat); biological
group label/indicator for each sample.}
}
\value{
dataframe with columns:
\describe{
\item{BatchV}{Proportion of total variance explained by batch effects.}
\item{GroupV}{Proportion of total variance explained by group effects.}
\item{ResidV}{Proportion of total variance that is residual noise.}
\item{lambda_raw}{Raw lambda index = total SS_batch / total SS_group.}
\item{lambda_adj}{Adjusted lambda = lambda_raw * ResidV/(1-ResidV).}
}
}
\description{
This function calculates the proportions of variation explained by batch,
group, and residual for each gene using two-way ANOVA and computes the lambda
index based on these three proportions.
}
\examples{

library(scran)
se <- mockSCE()
res <- BatchQC::compute_lambda(assays(se)[["counts"]],
  colData(se)$Mutation_Status,
  colData(se)$Treatment)
print(res)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kBET.R
\name{plot_kBET}
\alias{plot_kBET}
\title{kBET Rejection Plotter}
\usage{
plot_kBET(kBET_res)
}
\arguments{
\item{kBET_res}{list object output from kBET function}
}
\value{
ggplot object containing kBET rejection boxplot
}
\description{
This function generates a boxplot of observed and expected
rejection rates for the provided kBET output list object
}
\examples{
library(scran)
se <- mockSCE()
df <- as.matrix(assays(se)[["counts"]])
batch <- data.frame(colData(se))[, "Treatment"]

batch.estimate <- kBET(df, batch)
plot_kBET(batch.estimate)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{tb_data_upload}
\alias{tb_data_upload}
\title{TB data upload
This function uploads the TB data set from the curatedTBData package.}
\usage{
tb_data_upload()
}
\value{
a SE object with raw counts data and metadata
}
\description{
TB data upload
This function uploads the TB data set from the curatedTBData package.
}
\examples{
library(curatedTBData)
se_object <- tb_data_upload()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendrogram.R
\name{dendrogram_plotter}
\alias{dendrogram_plotter}
\title{Dendrogram Plot}
\usage{
dendrogram_plotter(se, assay, batch_var, category_var)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{assay}{assay to plot}

\item{batch_var}{sample metadata column representing batch}

\item{category_var}{sample metadata column representing category of interest}
}
\value{
named list of dendrogram plots

dendrogram is a dendrogram ggplot

circular_dendrogram is a circular dendrogram ggplot
}
\description{
This function creates a dendrogram plot
}
\examples{
library(scran)
se <- mockSCE()
dendrogram_plot <- BatchQC::dendrogram_plotter(se,
                                            "counts",
                                            "Mutation_Status",
                                            "Treatment")
dendrogram_plot$dendrogram
dendrogram_plot$circular_dendrogram

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umap.R
\name{umap}
\alias{umap}
\title{Create a umap plot; wrapper function for umap package plus custom plotting}
\usage{
umap(
  se_object,
  assay_of_interest,
  batch,
  covar,
  neighbors = 15,
  min_distance = 0.1,
  spread = 1,
  exploratory = FALSE,
  log_option = FALSE
)
}
\arguments{
\item{se_object}{se_object; containing data of interest}

\item{assay_of_interest}{string; the assay in the se_object to plot}

\item{batch}{string; representing batch}

\item{covar}{string; representing biological variable}

\item{neighbors}{integer; number of nearest neighbors, default 15 per umap;
lower values prioritize local structure, higher values will represent
bigger picture but lose finer details}

\item{min_distance}{numeric; how close points appear in final layout; higher
values puts less emphasis on global structure; must be less than spread}

\item{spread}{numeric; dispersion of points in umap}

\item{exploratory}{Boolean; default is FALSE, if TRUE, a 5x5 grid with
k = 15, 25, 50, 100 and min_distance = 0.1, .2, .5, .75, .99 will
be plotted}

\item{log_option}{Boolean; default is FALSE, if TRUE,
log(assay_of_interest + 1) is computed and used for plotting; useful for
count data}
}
\value{
umap plot
}
\description{
Create a umap plot; wrapper function for umap package plus custom plotting
}
\examples{
library(scran)
se <- mockSCE()
se$Treatment <- as.factor(se$Treatment)
se$Mutation_Status <- as.factor(se$Mutation_Status)
umap_plot <- BatchQC::umap(se_object = se, assay_of_interest = "counts",
batch = "Treatment", covar = "Mutation_Status", log_option = TRUE)
umap_plot

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/volcano_plot.R
\name{volcano_plot}
\alias{volcano_plot}
\title{Volcano plot}
\usage{
volcano_plot(DE_results, pslider = 0.05, fcslider)
}
\arguments{
\item{DE_results}{a dataframe with the results of one of the DE Analysis;
must include "log2FoldChange" and "pvalue" columns}

\item{pslider}{Magnitude of significance value threshold, default is 0.05}

\item{fcslider}{Magnitude of expression change value threshold}
}
\value{
A volcano plot of expression change and significance value data
}
\description{
This function allows you to plot DE analysis results as a volcano plot
}
\examples{
library(scran)
se <- mockSCE()
differential_expression <- BatchQC::DE_analyze(se = se,
                                                method = "DESeq2",
                                                batch = "Treatment",
                                                conditions = c(
                                                "Mutation_Status",
                                                "Cell_Cycle"),
                                                assay_to_analyze = "counts",
                                                padj_method = "BH")
value <- round((max(abs(
    differential_expression[[length(differential_expression)]][, 1]))
    + min(abs(
    differential_expression[[length(differential_expression)]][, 1]))) / 2)

volcano_plot(differential_expression[[1]], pslider = 0.05, fcslider = value)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lambda_statistic.R
\name{run_lambda}
\alias{run_lambda}
\title{Provide a recommendation on batch correction based on lambda calculation}
\usage{
run_lambda(se, assay, batch, condition)
}
\arguments{
\item{se}{summarized experiment object}

\item{assay}{string, the assay to analyze}

\item{batch}{string, batch variable}

\item{condition}{string, condition variable}
}
\value{
a named list with:
\describe{
\item{lambda_stat}{provides the output of \code{compute_lambda} function}
\item{correction_recommendation}{string, rec for batch correction}
}

a list with 2 parameters, 'lambda_stat' which contains the adj lambda
value from lambda_compute (ln(lambda)) and 'correction_recommendation'
which contains a string with a recommendation on if batch correction
should be completed
}
\description{
This functions determines if an experimental design is balanced, then
calculates the lambda statistic for balanced designs and provides a
recommendation on if batch correction should be utilized. In general,
unbalanced designs always benefit from batch correction, while balanced
designs with a lambda greater than -2 benefit from batch correction.
}
\examples{

library(scran)
se <- mockSCE()
lambda_calculation <- run_lambda(se,
                                 assay = "counts",
                                 batch = "Mutation_Status",
                                 condition = "Treatment")
 print(lambda_calculation$correction_recommendation)
 print(lambda_calculation$lambda_stat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negative_binomial_check.R
\name{pvalues_all_genes}
\alias{pvalues_all_genes}
\title{This function calculates goodness-of-fit p-values for each condition level
for each gene size, and estimated NB mean}
\usage{
pvalues_all_genes(condition, size, mu_matrix, count_matrix)
}
\arguments{
\item{condition}{string; name of the se colData with the condition status}

\item{size}{numeric; an estimated size parameter of the NB distributions for
the gene}

\item{mu_matrix}{matrix;estimated mu parameter of the NB distributions for
different samples of each gene}

\item{count_matrix}{string; name of the assay with gene expression matrix
(in counts)}
}
\value{
a data frame of p-values for each gene under each biological
condition
}
\description{
This function calculates goodness-of-fit p-values for each condition level
for each gene size, and estimated NB mean
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umap.R
\name{possible_distances}
\alias{possible_distances}
\title{Create potential min_distance values for exploratory analysis based on the
value of spread}
\usage{
possible_distances(spread)
}
\arguments{
\item{spread}{numeric; the value of spread used in the exploratory analysis}
}
\value{
vector of min_distance values to use in exploratory analysis
}
\description{
Create potential min_distance values for exploratory analysis based on the
value of spread
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negative_binomial_check.R
\name{edgeR_large_analysis}
\alias{edgeR_large_analysis}
\title{This function calculated the goodness of fit of edgeR for larger sample
sizes (intended for more than 150 samples).}
\usage{
edgeR_large_analysis(
  count_matrix,
  condition,
  other_variables,
  conditions_df,
  model_formula,
  num_samples,
  sampled,
  small_sample_cutoff
)
}
\arguments{
\item{count_matrix}{matrix containing the data to be analyzed}

\item{condition}{a vector containing a factor of the condition of interest
(typically batch)}

\item{other_variables}{a vector of strings of other variables of interest}

\item{conditions_df}{data frame containing information for the other
variables of interest (columns in order of the other_variables vector)}

\item{model_formula}{the stat formula to be used in the DESeq analysis}

\item{num_samples}{total number of samples to analyze}

\item{sampled}{the down sampled matrix}

\item{small_sample_cutoff}{value at which non-parametric test was used
(considered "large sample size") vs parametric was used (considered
"small sample size")}
}
\value{
a list containing the string recommendation
}
\description{
This function calculated the goodness of fit of edgeR for larger sample
sizes (intended for more than 150 samples).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_correction.R
\name{Harman_correction}
\alias{Harman_correction}
\title{Harman Correction
This function applies Harman correction to a summarized experiment object
with and reconstructs the data back into the original feature space}
\usage{
Harman_correction(
  se,
  assay_to_normalize,
  batch,
  covar,
  output_assay_name,
  limit = 0.95,
  numrepeats = 100000L
)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{assay_to_normalize}{string; name of assay that should be corrected}

\item{batch}{factor; The variable that represents batch}

\item{covar}{string; name of covariate.}

\item{output_assay_name}{string; name of results assay}

\item{limit}{numeric; default: 0.95 Indicates the limit of confidence in
which to stop removing a batch effect. Must be between 0 and 1}

\item{numrepeats}{integer; default: 100000L the number of repeats in which
to run the simulated batch mean distribution estimator using the random
selection algorithm.}
}
\value{
SE object with an added Harman corrected reconstructed data
}
\description{
Harman Correction
This function applies Harman correction to a summarized experiment object
with and reconstructs the data back into the original feature space
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runApp.R
\name{BatchQC}
\alias{BatchQC}
\title{Run BatchQC shiny app}
\usage{
BatchQC(dev = FALSE)
}
\arguments{
\item{dev}{Run the application in developer mode}
}
\value{
The shiny app will open
}
\description{
Run BatchQC shiny app
}
\examples{
if(interactive()){
BatchQC()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negative_binomial_check.R
\name{permuted_DESeq}
\alias{permuted_DESeq}
\title{This function performs DESeq on the permuted dataset.}
\usage{
permuted_DESeq(
  count_matrix,
  condition,
  other_variables,
  conditions_df,
  model_formula
)
}
\arguments{
\item{count_matrix}{matrix containing the data to be analyzed}

\item{condition}{a vector containing a factor of the condition of interest
(typically batch)}

\item{other_variables}{a vector of strings of other variables of interest}

\item{conditions_df}{data frame containing information for the other
variables of interest (columns in order of the other_variables vector)}

\item{model_formula}{the stat formula to be used in the DESeq analysis}
}
\value{
a DESeq2 object
}
\description{
This function performs DESeq on the permuted dataset.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differential_expression.R
\name{DE_analyze}
\alias{DE_analyze}
\title{Differential Expression Analysis}
\usage{
DE_analyze(se, method, batch, conditions, assay_to_analyze, padj_method)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{method}{DE analysis method option
('DESeq2', 'limma', 'edgeR', 'ANOVA', or 'Kruskal-Wallis')}

\item{batch}{metadata column in the se object representing batch}

\item{conditions}{metadata columns in the se object representing additional
analysis covariates}

\item{assay_to_analyze}{Assay in the se object (either counts for DESeq2 or
normalized data for limma or edgeR) for DE analysis}

\item{padj_method}{correction method for adjusted p-value from
p.adjust.methods}
}
\value{
A named list containing the log2FoldChange, fvalue (ANOVA only),
pvalue and adjusted pvalue (padj) for each analysis returned by DESeq2,
limma, edgeR, ANOVA, or Kruskal-Wallis.
}
\description{
This function runs DE analysis on a count matrix (DESeq), a normalized log or
log-CPM matrix (limma), an edgeR TMM-normalized matrix (edgeR) or perform
ANOVA or Kruskal-Wallis test on the data contained in the se object.
}
\examples{
library(scran)
se <- mockSCE()
differential_expression <- BatchQC::DE_analyze(se = se,
                                                method = "DESeq2",
                                                batch = "Treatment",
                                                conditions = c(
                                                "Mutation_Status"),
                                                assay_to_analyze = "counts",
                                                padj_method = "BH")
pval_summary(differential_expression)
pval_plotter(differential_expression)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_correction.R
\name{ComBat_seq_correction}
\alias{ComBat_seq_correction}
\title{ComBat-Seq Correction
This function applies ComBat-seq correction to your summarized experiment
object}
\usage{
ComBat_seq_correction(se, assay_to_normalize, batch, group, covar,
output_assay_name)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{assay_to_normalize}{Assay that should be corrected}

\item{batch}{The variable that represents batch}

\item{group}{The group variable}

\item{covar}{list of covariates}

\item{output_assay_name}{name of results assay}
}
\value{
SE object with an added ComBat-seq corrected array
}
\description{
ComBat-Seq Correction
This function applies ComBat-seq correction to your summarized experiment
object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{bladder_data_upload}
\alias{bladder_data_upload}
\title{Bladder data upload
This function uploads the Bladder data set from the bladderbatch package.
This dataset is from bladder cancer data with 22,283 different microarray
gene expression data. It has 57 bladder samples with 3 metadata variables
(batch, outcome and cancer). It contains 5 batches, 3 cancer types (cancer,
biopsy, control), and 5 outcomes (Biopsy, mTCC, sTCC-CIS, sTCC+CIS, and
Normal). Batch 1 contains only cancer, 2 has cancer and controls, 3 has only
controls, 4 contains only biopsy, and 5 contains cancer and biopsy}
\usage{
bladder_data_upload()
}
\value{
a SE object with counts data and metadata
}
\description{
Bladder data upload
This function uploads the Bladder data set from the bladderbatch package.
This dataset is from bladder cancer data with 22,283 different microarray
gene expression data. It has 57 bladder samples with 3 metadata variables
(batch, outcome and cancer). It contains 5 batches, 3 cancer types (cancer,
biopsy, control), and 5 outcomes (Biopsy, mTCC, sTCC-CIS, sTCC+CIS, and
Normal). Batch 1 contains only cancer, 2 has cancer and controls, 3 has only
controls, 4 contains only biopsy, and 5 contains cancer and biopsy
}
\examples{
library(bladderbatch)
se_object <- bladder_data_upload()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_correction.R
\name{svaseq_correction}
\alias{svaseq_correction}
\title{svaseq Correction
This function applies sva correction to a summarized experiment object
with count based RNA-seq data}
\usage{
svaseq_correction(
  se,
  assay_to_normalize,
  var_of_interest,
  covar,
  output_assay_name,
  num_sv = FALSE
)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{assay_to_normalize}{string; name of assay that should be corrected}

\item{var_of_interest}{string; name of  experimental variable of interest}

\item{covar}{list; sting list  of covariates to include in sva analysis}

\item{output_assay_name}{string; name of results assay}

\item{num_sv}{boolean; Default is FALSE: the number of estimated latent
factor is set to 1 for a small number of samples. If set to TRUE, svaseq
function will estimate the number of latent factors for you.}
}
\value{
SE object with an added sva corrected array
}
\description{
svaseq Correction
This function applies sva correction to a summarized experiment object
with count based RNA-seq data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap_num_to_char_convertor.R
\name{heatmap_num_to_char_converter}
\alias{heatmap_num_to_char_converter}
\title{Heatmap numeric to character converter}
\usage{
heatmap_num_to_char_converter(ann_col)
}
\arguments{
\item{ann_col}{column data of heatmap}
}
\value{
ann_col modified column data of heatmap
}
\description{
This function converts any found numerics to characters
}
\examples{
library(scran)
se <- mockSCE()
col_info <- colData(se)
ann_col <- heatmap_num_to_char_converter(ann_col = col_info)
ann_col

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differential_expression.R
\name{pval_summary}
\alias{pval_summary}
\title{Returns summary table for p-values of explained variation}
\usage{
pval_summary(res_list)
}
\arguments{
\item{res_list}{Differential Expression analysis result (a named list of
dataframes corresponding to each analysis completed with a "pvalue" column)}
}
\value{
summary table for p-values of explained variation for each analysis
}
\description{
Returns summary table for p-values of explained variation
}
\examples{
library(scran)
se <- mockSCE()
differential_expression <- BatchQC::DE_analyze(se = se,
                                                method = "DESeq2",
                                                batch = "Treatment",
                                                conditions = c(
                                                "Mutation_Status"),
                                                assay_to_analyze = "counts",
                                                padj_method = "BH")
pval_summary(differential_expression)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{BMI_data}
\alias{BMI_data}
\title{This function returns BMI data that comes form the data in "Comparing
tuberculosis gene signatures in malnourished individuals using the
TBSignatureProfiler" paper. Subject IDs were matched as shown on
"github.com/jessmcc22/BatchQCv2_Manuscript/blob/devel/R/subjectID_match.R"}
\usage{
BMI_data(meta)
}
\arguments{
\item{meta}{dataframe; metadata that needs to be matched to BMI}
}
\value{
dataframe provided as input with BMI info added
}
\description{
This function returns BMI data that comes form the data in "Comparing
tuberculosis gene signatures in malnourished individuals using the
TBSignatureProfiler" paper. Subject IDs were matched as shown on
"github.com/jessmcc22/BatchQCv2_Manuscript/blob/devel/R/subjectID_match.R"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_aic.R
\name{AIC_boxplots}
\alias{AIC_boxplots}
\title{Boxplots for the distribution of AIC for each method}
\usage{
AIC_boxplots(AIC_data, num_methods)
}
\arguments{
\item{AIC_data}{dataframe with the data to be plotted}

\item{num_methods}{integer representing the number of distribution methods}
}
\value{
AIC_boxplot; a boxplot for each method showing distribution of data
}
\description{
This function creates a boxplot of all the AIC values for each gene under
each tested distribution to aid in identifying outliers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendrogram_alpha_numeric_check.R
\name{dendrogram_alpha_numeric_check}
\alias{dendrogram_alpha_numeric_check}
\title{Dendrogram alpha or numeric checker}
\usage{
dendrogram_alpha_numeric_check(dendro_var)
}
\arguments{
\item{dendro_var}{column from dendrogram object representing category}
}
\value{
geom_label label for the legend of category variable
}
\description{
This function checks if there is any numeric or strings for plotting legend
}
\examples{
library(scran)
se <- mockSCE()
dendro_alpha_numeric_check <- dendrogram_alpha_numeric_check(
                                        dendro_var = "Treatment")
dendro_alpha_numeric_check

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kBET.R
\name{run_kBET}
\alias{run_kBET}
\title{kBET rejection rate}
\usage{
run_kBET(
  se,
  assay_to_normalize,
  batch,
  k0 = NULL,
  knn = NULL,
  testSize = NULL,
  do.pca = TRUE,
  dim.pca = 50,
  heuristic = TRUE,
  n_repeat = 100,
  alpha = 0.05,
  addTest = FALSE,
  verbose = FALSE,
  adapt = TRUE
)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{assay_to_normalize}{string; assay from se object to do normalization}

\item{batch}{character string of column name that represents batch}

\item{k0}{integer representing number of nearest neighbors to test on
(neighborhood size)}

\item{knn}{n x k matrix of nearest neighbors for each cell (optional)}

\item{testSize}{integer representing number of data points to test}

\item{do.pca}{Boolean, if TRUE, perform a pca prior to knn search
(defaults to TRUE)}

\item{dim.pca}{Boolean, if do.pca=TRUE, choose the number of dimensions to
consider (defaults to 50)}

\item{heuristic}{Boolean, if true, compute an optimal neighborhood size k
(defaults to TRUE)}

\item{n_repeat}{numeric representing 'n_repeat' subsets to evaluate in order
to create a statistics on batch estimates}

\item{alpha}{numeric for significance level}

\item{addTest}{Boolean, if TRUE, perform an LRT-approximation to the
multinomial test AND a multinomial exact test (if appropriate)}

\item{verbose}{Boolean, if TRUE, display stages of current computation
(defaults to FALSE)}

\item{adapt}{Boolean, if TRUE, frequencies will be adapted (defaults to TRUE)}
}
\value{
list object from kBET() function
\enumerate{
\item \code{summary} - a rejection rate for the data,
an expected rejection rate for random
labeling and the significance for the observed result
\item \code{results} - detailed list for each tested cells;
p-values for expected and observed label distribution
\item \code{average.pval} - significance level over the averaged
batch label distribution in all neighbourhoods
\item \code{stats} - extended test summary for every sample
\item \code{params} - list of input parameters and adapted parameters,
respectively
\item \code{outsider} - only shown if \code{adapt=TRUE}. List of samples
without mutual nearest neighbour: \itemize{
\item \code{index} - index of each outsider sample)
\item \code{categories} - tabularised labels of outsiders
\item \code{p.val} - Significance level of outsider batch label
distribution vs expected frequencies.
If the significance level is lower than \code{alpha},
expected frequencies will be adapted}
}
}
\description{
This function runs the k-nearest neighbor batch effect test
(kBET) to evaluate whether the data has detectable batch effect.
}
\examples{
library(scran)
se <- mockSCE()
kBET_result <- BatchQC::run_kBET(
  se=se,
  assay_to_normalize="counts",
  batch="Treatment"
)

BatchQC::plot_kBET(kBET_result)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_correction.R
\name{sva_correction}
\alias{sva_correction}
\title{sva Correction
This function applies sva correction to a summarized experiment object
(implementation adapted from sva::psva)}
\usage{
sva_correction(
  se,
  assay_to_normalize,
  var_of_interest,
  covar,
  output_assay_name,
  psva = FALSE
)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{assay_to_normalize}{string; name of assay that should be corrected}

\item{var_of_interest}{string; name of  experimental variable of interest}

\item{covar}{list; sting list  of covariates to include in sva analysis}

\item{output_assay_name}{string; name of results assay}

\item{psva}{boolean; default: FALSE. If set to TRUE and no covariate input,
psva function from the sva package will be used to remove batch effect.}
}
\value{
SE object with an added sva corrected array
}
\description{
sva Correction
This function applies sva correction to a summarized experiment object
(implementation adapted from sva::psva)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negative_binomial_check.R
\name{nb_down_sample}
\alias{nb_down_sample}
\title{Performs down sampling for negative binomial model fit check.}
\usage{
nb_down_sample(count_matrix, num_genes)
}
\arguments{
\item{count_matrix}{matrix; contains the feature data (should be counts)}

\item{num_genes}{integer; number of genes to use in down sampling}
}
\value{
list containing "sampled" a down sampled assay and "count_matrix"
with the down sampled count_matrix
}
\description{
Performs down sampling for negative binomial model fit check.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negative_binomial_check.R
\name{goodness_of_fit_nb}
\alias{goodness_of_fit_nb}
\title{This function calculates goodness-of-fit pvalues for all genes by looking at
how the NB model by edgeR or DESeq2 fit the data}
\usage{
goodness_of_fit_nb(
  se,
  count_matrix,
  condition,
  other_variables = NULL,
  method = "edgeR",
  num_genes = 500,
  small_sample_cutoff = 21
)
}
\arguments{
\item{se}{the se object; se object where all the data is contained}

\item{count_matrix}{string; name of the assay with gene expression matrix
(in counts)}

\item{condition}{string; name of the se colData with the condition status}

\item{other_variables}{string; name of the se colData containing other
variables of interest that should be considered in the model}

\item{method}{string; method to use for the parametric or non-parametric
test;either "DESeq2" or "edgeR"; default is "edgeR"}

\item{num_genes}{down sample value, default is 500 (or all genes if less)}

\item{small_sample_cutoff}{value at which non-parametric test will be used
(considered "large sample size") vs parametric will be used (considered
"small sample size); default is 21}
}
\value{
a matrix of p-values where each row is a gene and each column is a
level within the condition of interest
}
\description{
This function calculates goodness-of-fit pvalues for all genes by looking at
how the NB model by edgeR or DESeq2 fit the data
}
\examples{
# example code for small sample
library(scran)
se <- mockSCE(ncells = 20)
se$Treatment <- as.factor(se$Treatment)
se$Mutation_Status <- as.factor(se$Mutation_Status)
nb_results <- goodness_of_fit_nb(se = se, count_matrix = "counts",
  condition = "Treatment", other_variables = "Mutation_Status",
  method = "edgeR")
nb_results[1]
nb_results[2]
nb_results[3]

# example code for large sample
library(scran)
se <- mockSCE(ncells = 150)
se$Treatment <- as.factor(se$Treatment)
se$Mutation_Status <- as.factor(se$Mutation_Status)
nb_results <- goodness_of_fit_nb(se = se, count_matrix = "counts",
  condition = "Treatment", other_variables = "Mutation_Status",
  method = "edgeR")
nb_results[1]
nb_results[2]
nb_results[3]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explained_variation.R
\name{EV_table}
\alias{EV_table}
\title{EV Table
Returns table with percent variation explained for specified number of genes}
\usage{
EV_table(batchqc_ev)
}
\arguments{
\item{batchqc_ev}{explained variation results from
batchqc_explained_variation}
}
\value{
List of explained variation by batch and condition
}
\description{
EV Table
Returns table with percent variation explained for specified number of genes
}
\examples{
library(scran)
se <- mockSCE()
se$Mutation_Status <- as.factor(se$Mutation_Status)
se$Treatment <- as.factor(se$Treatment)
exp_var_result <- BatchQC::batchqc_explained_variation(se,
                                    batch = "Mutation_Status",
                                    condition = "Treatment",
                                    assay_name = "counts")
EV_table <- BatchQC::EV_table(exp_var_result[[1]])

EV_table

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_palette.R
\name{color_palette}
\alias{color_palette}
\title{Color palette}
\usage{
color_palette(n, first_hue = 25, last_hue = 360)
}
\arguments{
\item{n}{numeric object representing number of colors to be created}

\item{first_hue}{numeric object to set the first hue value}

\item{last_hue}{numeric object to set the final hue value}
}
\value{
color_list list of colors generated
}
\description{
This function creates the base color palette used in BatchQC
}
\examples{
library(scran)
n <- 100
color_list <- color_palette(n)
color_list

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Normalization.R
\name{normalize_SE}
\alias{normalize_SE}
\title{This function allows you to add normalized count matrix to the SE object}
\usage{
normalize_SE(
  se,
  method,
  log_bool,
  assay_to_normalize,
  output_assay_name,
  condition = NULL,
  batch = NULL
)
}
\arguments{
\item{se}{SummarizedExperiment Object}

\item{method}{string; Normalization Method, either 'CPM', 'DESeq', 'edgeR',
'voom', or 'none' for log(x+1) only}

\item{log_bool}{True or False; True to log normalize the data set after
normalization method}

\item{assay_to_normalize}{string; SE assay to do normalization on}

\item{output_assay_name}{string; name for the resulting normalized assay}

\item{condition}{string; the biological variable of interest, required for
voom, default 'NULL'}

\item{batch}{string; the batch variable, required for voom, default 'NULL'}
}
\value{
the original SE object with normalized assay appended
}
\description{
This function allows you to add normalized count matrix to the SE object
}
\examples{
library(scran)
se <- mockSCE()
se_CPM_normalized <- BatchQC::normalize_SE(se, method = "CPM",
                                log_bool = FALSE,
                                assay_to_normalize = "counts",
                                output_assay_name =
                                    "CPM_normalized_counts")
se_DESeq_normalized <- BatchQC::normalize_SE(se, method = "DESeq",
                                log_bool = FALSE,
                                assay_to_normalize = "counts",
                                output_assay_name =
                                    "DESeq_normalized_counts")
se_CPM_normalized
se_DESeq_normalized

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explained_variation.R
\name{check_valid_input}
\alias{check_valid_input}
\title{Helper function to save variables as factors if not already factors}
\usage{
check_valid_input(se, batch, condition)
}
\arguments{
\item{se}{se object}

\item{batch}{batch}

\item{condition}{condition}
}
\value{
se se object
}
\description{
Helper function to save variables as factors if not already factors
}
\keyword{Internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{summarized_experiment}
\alias{summarized_experiment}
\title{This function creates a summarized experiment object from count
and metadata files uploaded by the user}
\usage{
summarized_experiment(counts, columndata)
}
\arguments{
\item{counts}{counts matrix}

\item{columndata}{metadata dataframe}
}
\value{
a summarized experiment object
}
\description{
This function creates a summarized experiment object from count
and metadata files uploaded by the user
}
\examples{
data(protein_data)
data(protein_sample_info)
se_object <- summarized_experiment(protein_data, protein_sample_info)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{protein_sample_info}
\alias{protein_sample_info}
\title{Batch and Condition indicator for protein expression data}
\format{
A data frame with 24 rows and 2 variables:
\describe{
\item{batch}{Batch Indicator}
\item{category}{Condition (Case vs Control) Indicator}
}
}
\usage{
data(protein_sample_info)
}
\description{
This data consists of two batches and two conditions
corresponding to case and control for the protein expression data
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lambda_statistic.R
\name{is_design_balanced}
\alias{is_design_balanced}
\title{Check if the experimental design is balanced or unbalanced}
\usage{
is_design_balanced(se, batch, covariate)
}
\arguments{
\item{se}{summarized experiment object}

\item{batch}{string, batch variable}

\item{covariate}{string, biological covariate}
}
\value{
Boolean Value, TRUE if the experimental design is balanced, FALSE if
the experimental design is not balanced
}
\description{
Used in conjunction with the lambda
}
\examples{

library(scran)
se <- mockSCE()
balanced_design_check <- is_design_balanced(se, batch = "Mutation_Status",
                                                covariate = "Treatment")
balanced_design_check

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_statistics.R
\name{confound_metrics}
\alias{confound_metrics}
\title{Combine std. Pearson correlation coefficient and Cramer's V}
\usage{
confound_metrics(se, batch)
}
\arguments{
\item{se}{summarized experiment}

\item{batch}{batch variable}
}
\value{
metrics of confounding
}
\description{
Combine std. Pearson correlation coefficient and Cramer's V
}
\examples{
library(scran)
se <- mockSCE()
confound_table <- BatchQC::confound_metrics(se, batch = "Mutation_Status")
confound_table

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covariates_not_confounded.R
\name{covariates_not_confounded}
\alias{covariates_not_confounded}
\title{Returns list of covariates not confounded by batch; helper function for
explained variation and for populating shiny app condition options}
\usage{
covariates_not_confounded(se, batch)
}
\arguments{
\item{se}{Summarized experiment object}

\item{batch}{Batch variable}
}
\value{
List of explained variation by batch and condition
}
\description{
Returns list of covariates not confounded by batch; helper function for
explained variation and for populating shiny app condition options
}
\examples{
library(scran)
se <- mockSCE()
covariates_not_confounded <- BatchQC::covariates_not_confounded(se,
                                            batch = "Mutation_Status")
covariates_not_confounded

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negative_binomial_check.R
\name{counts2pvalue}
\alias{counts2pvalue}
\title{This function calculates p-values for each gene given counts, estimated NB
size, and estimated NB mean}
\usage{
counts2pvalue(counts, size, mu)
}
\arguments{
\item{counts}{a vector of gene expression values (in counts)}

\item{size}{an estimated size parameter of the NB distributions for the gene}

\item{mu}{a vector of estimated mu parameter of the NB distributions for
different samples of the gene}
}
\value{
a p-value based on estimated NB size and mean
}
\description{
This function calculates p-values for each gene given counts, estimated NB
size, and estimated NB mean
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variation_ratios.R
\name{ratio_plotter}
\alias{ratio_plotter}
\title{This function allows you to plot ratios of explained variation}
\usage{
ratio_plotter(ev_ratio)
}
\arguments{
\item{ev_ratio}{table of ratios from variation_ratios()}
}
\value{
boxplot of ratios
}
\description{
This function allows you to plot ratios of explained variation
}
\examples{
library(scran)
se <- mockSCE()
se$Mutation_Status <- as.factor(se$Mutation_Status)
se$Treatment <- as.factor(se$Treatment)
expl_var_result <- batchqc_explained_variation(se, batch = "Mutation_Status",
                            condition = "Treatment", assay_name = "counts")
ratios_results <- variation_ratios(expl_var_result[[1]],
    batch = "Mutation_Status")
ratio_boxplot <- BatchQC::ratio_plotter(ratios_results)
ratio_boxplot

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negative_binomial_check.R
\name{commentary}
\alias{commentary}
\title{This function creates the commentary recommendation when there are more than
20 samples.}
\usage{
commentary(nb_fit_pval, count_below_value_pval, proportion, low_pval, method)
}
\arguments{
\item{nb_fit_pval}{Boolean representing if the p-val count is below threshold}

\item{count_below_value_pval}{number of features below p-val threshold}

\item{proportion}{numeric; proportion of genes below the p-value}

\item{low_pval}{pval threshold}

\item{method}{string; method utilized for the parametric or non-parametric
test; either "DESeq2" or "edgeR"}
}
\value{
a commentary string statement
}
\description{
This function creates the commentary recommendation when there are more than
20 samples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_aic.R
\name{run_AIC_models}
\alias{run_AIC_models}
\title{Helper function that contains the code to run the lognormal, voom, and
negative binomial AIC models for \code{compute_aic}}
\usage{
run_AIC_models(dat, design, nb_test, maxit)
}
\arguments{
\item{dat}{dataframe of the data to analyze}

\item{design}{stats design model to be used in the analyses}

\item{nb_test}{boolean; should negative binomial run (must be discrete data)}

\item{maxit}{integer; the max number of IWLS iterations}
}
\value{
data frame; containing the AIC results of each method
}
\description{
Helper function that contains the code to run the lognormal, voom, and
negative binomial AIC models for \code{compute_aic}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendrogram.R
\name{process_dendrogram}
\alias{process_dendrogram}
\title{Process Dendrogram}
\usage{
process_dendrogram(se, assay)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{assay}{assay to plot}
}
\value{
named list of dendrogram data

dendrogram_segments is data representing segments of the dendrogram

dendrogram_ends is data representing ends of the dendrogram
}
\description{
This function processes count data for dendrogram plotting
}
\examples{
library(scran)
se <- mockSCE()
process_dendro <- BatchQC::process_dendrogram(se, "counts")
process_dendro

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differential_expression.R
\name{pval_plotter}
\alias{pval_plotter}
\title{P-value Plotter
This function allows you to plot p-values of explained variation}
\usage{
pval_plotter(DE_results)
}
\arguments{
\item{DE_results}{Differential Expression analysis result (a named list of
dataframes corresponding to each analysis completed with a "pvalue" column)}
}
\value{
boxplots of pvalues for each condition
}
\description{
P-value Plotter
This function allows you to plot p-values of explained variation
}
\examples{
library(scran)
se <- mockSCE()
differential_expression <- BatchQC::DE_analyze(se = se,
                                                method = "DESeq2",
                                                batch = "Treatment",
                                                conditions = c(
                                                "Mutation_Status"),
                                                assay_to_analyze = "counts",
                                                padj_method = "BH")
pval_summary(differential_expression)
pval_plotter(differential_expression)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendrogram_color_palette.R
\name{dendrogram_color_palette}
\alias{dendrogram_color_palette}
\title{Dendrogram color palette}
\usage{
dendrogram_color_palette(col, dendrogram_info)
}
\arguments{
\item{col}{string object representing color of the label}

\item{dendrogram_info}{dendrogram_ends object}
}
\value{
annotation_color vector of colors corresponding to col variable
}
\description{
This function creates the color palette used in the dendrogram plotter
}
\examples{
library(scran)
se <- mockSCE()
process_dendro <- BatchQC::process_dendrogram(se, "counts")
dendrogram_ends <- process_dendro$dendrogram_ends
col <- process_dendro$condition_var
dendro_colors <- dendrogram_color_palette(col = "Treatment",
                                    dendrogram_info = dendrogram_ends)
dendro_colors

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kBET.R
\name{bisect}
\alias{bisect}
\title{bisect - a generic bisection function}
\usage{
bisect(foo, bounds, known = NULL, ..., tolx = 5, toly = 0.01)
}
\arguments{
\item{foo}{a function mapping a one-dim argument to one-dim value}

\item{bounds}{a vector of length 2 with real valued numbers
(i.e. two arguments of \code{foo})}

\item{known}{tells for which of the arguments a value is known
(defaults to NULL)}

\item{...}{additional parameters for \code{foo}}

\item{tolx}{break condition for argument (defaults to 10)}

\item{toly}{break condition for value (defaults to 0.01)}
}
\value{
A range of bounds where \code{foo} is maximal.
}
\description{
adapted from kBET package (https://github.com/theislab/kBET).
Provides recursive bisection algorithm for an arbitrary function. It
evaluates the function \code{foo} at the bounds and replaces one of the
boundaries until a maximum is found or the interval becomes too small
}
\examples{
get_maximum <- bisect(function(x) {
  -(x - 2)^2
}, c(-5, 50))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{signature_data}
\alias{signature_data}
\title{Signature data with 1600 gene expression levels}
\format{
A data frame with 1600 rows and 89 variables
}
\usage{
data(signature_data)
}
\description{
This data consists of three batches and ten conditions.
The columns are samples, and the rows represent
1600 different genes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/negative_binomial_check.R
\name{nb_proportion}
\alias{nb_proportion}
\title{This function determines the proportion of p-values below a specific value
and compares to the previously determined threshold}
\usage{
nb_proportion(
  p_val_table,
  low_pval,
  threshold,
  num_samples,
  small_sample_cutoff,
  method
)
}
\arguments{
\item{p_val_table}{table of p-values from the nb test}

\item{low_pval}{value of the p-value cut off to use in proportion}

\item{threshold}{the value to compare the proportion of p-values to for data
sets}

\item{num_samples}{the number of samples in the analysis}

\item{small_sample_cutoff}{value at which non-parametric test will be used
(considered "large sample size") vs parametric will be used (considered
"small sample size); default is 20}

\item{method}{string; method utilized for the parametric or non-parametric
test; either "DESeq2" or "edgeR"}
}
\value{
a statement about whether DESeq2 is appropriate to use for analysis
}
\description{
This function determines the proportion of p-values below a specific value
and compares to the previously determined threshold
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_correction.R
\name{ComBat_correction}
\alias{ComBat_correction}
\title{ComBat Correction
This function applies ComBat correction to your summarized experiment object}
\usage{
ComBat_correction(se, assay_to_normalize, batch, covar, output_assay_name)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{assay_to_normalize}{Assay that should be corrected}

\item{batch}{The variable that represents batch}

\item{covar}{list of covariates}

\item{output_assay_name}{name of results assay}
}
\value{
SE object with an added ComBat corrected array
}
\description{
ComBat Correction
This function applies ComBat correction to your summarized experiment object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_statistics.R
\name{batch_design}
\alias{batch_design}
\title{This function allows you to make a batch design matrix}
\usage{
batch_design(se, batch, covariate)
}
\arguments{
\item{se}{summarized experiment object}

\item{batch}{string, batch variable}

\item{covariate}{string, biological covariate}
}
\value{
design table
}
\description{
This function allows you to make a batch design matrix
}
\examples{
library(scran)
se <- mockSCE()
batch_design_tibble <- batch_design(se, batch = "Mutation_Status",
                                                covariate = "Treatment")
batch_design_tibble

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explained_variation.R
\name{batchqc_explained_variation}
\alias{batchqc_explained_variation}
\title{Returns a list of explained variation by batch and condition
combinations}
\usage{
batchqc_explained_variation(se, batch, condition = NULL, assay_name)
}
\arguments{
\item{se}{Summarized experiment object}

\item{batch}{Batch covariate}

\item{condition}{Condition covariate(s) of interest if desired, default is
NULL}

\item{assay_name}{Assay of choice}
}
\value{
List of explained variation by batch and condition
}
\description{
Returns a list of explained variation by batch and condition
combinations
}
\examples{
library(scran)
se <- mockSCE()
batchqc_explained_variation <- BatchQC::batchqc_explained_variation(se,
                                        batch = "Mutation_Status",
                                        condition = "Treatment",
                                        assay_name = "counts")
batchqc_explained_variation

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{merged_IDs}
\alias{merged_IDs}
\title{BMI and matched sample names for TB data}
\format{
A data frame with 91 rows and 3 columns
\describe{
\item{subjectID_curatedTBData}{Subject ID found in curatedTBData}
\item{subjectID_TB_Paper}{Subject ID in the original paper}
\item{BMI}{subject's BMI from the original study}
}
}
\usage{
data(merged_IDs)
}
\description{
This is support data for the TB data set that contains the BMI data and ID
numbers from both the curatedTBData database and the original study the data
was used in
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{plot_data}
\alias{plot_data}
\title{This function formats the PCA plot using ggplot}
\usage{
plot_data(pca_plot_data, color, shape, batch, xaxisPC, yaxisPC)
}
\arguments{
\item{pca_plot_data}{Data for all assays to plot}

\item{color}{variable that will be plotted as color}

\item{shape}{variable that will be plotted as shape}

\item{batch}{variable representing batch for the ellipses}

\item{xaxisPC}{the PC to plot as the x axis}

\item{yaxisPC}{the PC to plot as the y axis}
}
\value{
PCA plot
}
\description{
This function formats the PCA plot using ggplot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocess.R
\name{preprocess}
\alias{preprocess}
\title{Preprocess assay data}
\usage{
preprocess(se, assay, nfeature, log_option)
}
\arguments{
\item{se}{Summarized Experiment object}

\item{assay}{Assay from SummarizedExperiment object}

\item{nfeature}{Number of variable features to use}

\item{log_option}{"True" if data should be logged, "False" otherwise}
}
\value{
Returns processed data
}
\description{
Preprocess assay data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_correction.R
\name{limma_correction}
\alias{limma_correction}
\title{Limma Correction
This function applies limma batch correction to your provided assay}
\usage{
limma_correction(se, assay_to_normalize, batch, covar, output_assay_name)
}
\arguments{
\item{se}{SummarizedExperiment object}

\item{assay_to_normalize}{Log assay that should be corrected}

\item{batch}{Factor containing batch information}

\item{covar}{list of covariates}

\item{output_assay_name}{name of results assay}
}
\value{
SE object with an added limma corrected array
}
\description{
Limma Correction
This function applies limma batch correction to your provided assay
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explained_variation.R
\name{summary_stats_EV_table}
\alias{summary_stats_EV_table}
\title{Summary Stats EV Table
Returns table with min, 1st quartile, mean, 2nd quartile, and max for each
variable in the explained variation boxplot}
\usage{
summary_stats_EV_table(batchqc_ev)
}
\arguments{
\item{batchqc_ev}{explained variation results from}
}
\value{
summary_stats_table dataframe containing the min, 1st quartile, mean,
2nd quartile, and max for each variable in the explained variation boxplot
}
\description{
Summary Stats EV Table
Returns table with min, 1st quartile, mean, 2nd quartile, and max for each
variable in the explained variation boxplot
}
\examples{
library(scran)
se <- mockSCE()
se$Mutation_Status <- as.factor(se$Mutation_Status)
se$Treatment <- as.factor(se$Treatment)
exp_var_result <- BatchQC::batchqc_explained_variation(se,
                                    batch = "Mutation_Status",
                                    condition = "Treatment",
                                    assay_name = "counts")
summary_stats_table <- BatchQC::summary_stats_EV_table(exp_var_result[[1]])

summary_stats_table

}
